package com.alibaba.fastjson;

import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.parser.deserializer.ExtraProcessor;
import com.alibaba.fastjson.parser.deserializer.ExtraTypeProvider;
import com.alibaba.fastjson.parser.deserializer.FieldTypeResolver;
import com.alibaba.fastjson.parser.deserializer.ParseProcess;
import com.alibaba.fastjson.serializer.JSONSerializable;
import com.alibaba.fastjson.serializer.JSONSerializer;
import com.alibaba.fastjson.serializer.JavaBeanSerializer;
import com.alibaba.fastjson.serializer.ObjectSerializer;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.alibaba.fastjson.serializer.SerializeFilter;
import com.alibaba.fastjson.serializer.SerializeWriter;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alibaba.fastjson.util.IOUtils;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.lang.reflect.Array;
import java.lang.reflect.Type;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;
import java.util.TreeMap;
import java.util.Map.Entry;

public abstract class JSON implements JSONStreamAware, JSONAware {
   public static TimeZone defaultTimeZone = TimeZone.getDefault();
   public static Locale defaultLocale = Locale.getDefault();
   public static String DEFAULT_TYPE_KEY = "@type";
   static final SerializeFilter[] emptyFilters = new SerializeFilter[0];
   public static String DEFFAULT_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
   public static int DEFAULT_PARSER_FEATURE;
   public static int DEFAULT_GENERATE_FEATURE;
   private static final ThreadLocal<byte[]> bytesLocal;
   private static final ThreadLocal<char[]> charsLocal;
   public static final String VERSION = "1.2.47";

   public static void setDefaultTypeKey(String typeKey) {
      DEFAULT_TYPE_KEY = typeKey;
      ParserConfig.global.symbolTable.addSymbol(typeKey, 0, typeKey.length(), typeKey.hashCode(), true);
   }

   public static Object parse(String text) {
      return parse(text, DEFAULT_PARSER_FEATURE);
   }

   public static Object parse(String text, ParserConfig config) {
      return parse(text, config, DEFAULT_PARSER_FEATURE);
   }

   public static Object parse(String text, ParserConfig config, int features) {
      if (text == null) {
         return null;
      } else {
         DefaultJSONParser parser = new DefaultJSONParser(text, config, features);
         Object value = parser.parse();
         parser.handleResovleTask(value);
         parser.close();
         return value;
      }
   }

   public static Object parse(String text, int features) {
      return parse(text, ParserConfig.getGlobalInstance(), features);
   }

   public static Object parse(byte[] input, Feature... features) {
      char[] chars = allocateChars(input.length);
      int len = IOUtils.decodeUTF8(input, 0, input.length, chars);
      return len < 0 ? null : parse(new String(chars, 0, len), features);
   }

   public static Object parse(byte[] input, int off, int len, CharsetDecoder charsetDecoder, Feature... features) {
      if (input != null && input.length != 0) {
         int featureValues = DEFAULT_PARSER_FEATURE;
         Feature[] var6 = features;
         int var7 = features.length;

         for(int var8 = 0; var8 < var7; ++var8) {
            Feature feature = var6[var8];
            featureValues = Feature.config(featureValues, feature, true);
         }

         return parse(input, off, len, charsetDecoder, featureValues);
      } else {
         return null;
      }
   }

   public static Object parse(byte[] input, int off, int len, CharsetDecoder charsetDecoder, int features) {
      charsetDecoder.reset();
      int scaleLength = (int)((double)len * (double)charsetDecoder.maxCharsPerByte());
      char[] chars = allocateChars(scaleLength);
      ByteBuffer byteBuf = ByteBuffer.wrap(input, off, len);
      CharBuffer charBuf = CharBuffer.wrap(chars);
      IOUtils.decode(charsetDecoder, byteBuf, charBuf);
      int position = charBuf.position();
      DefaultJSONParser parser = new DefaultJSONParser(chars, position, ParserConfig.getGlobalInstance(), features);
      Object value = parser.parse();
      parser.handleResovleTask(value);
      parser.close();
      return value;
   }

   public static Object parse(String text, Feature... features) {
      int featureValues = DEFAULT_PARSER_FEATURE;
      Feature[] var3 = features;
      int var4 = features.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         Feature feature = var3[var5];
         featureValues = Feature.config(featureValues, feature, true);
      }

      return parse(text, featureValues);
   }

   public static JSONObject parseObject(String text, Feature... features) {
      return (JSONObject)parse(text, features);
   }

   public static JSONObject parseObject(String text) {
      Object obj = parse(text);
      if (obj instanceof JSONObject) {
         return (JSONObject)obj;
      } else {
         try {
            return (JSONObject)toJSON(obj);
         } catch (RuntimeException var3) {
            throw new JSONException("can not cast to JSONObject.", var3);
         }
      }
   }

   public static <T> T parseObject(String text, TypeReference<T> type, Feature... features) {
      return parseObject(text, type.type, ParserConfig.global, DEFAULT_PARSER_FEATURE, features);
   }

   public static <T> T parseObject(String json, Class<T> clazz, Feature... features) {
      return parseObject(json, clazz, ParserConfig.global, (ParseProcess)null, DEFAULT_PARSER_FEATURE, features);
   }

   public static <T> T parseObject(String text, Class<T> clazz, ParseProcess processor, Feature... features) {
      return parseObject(text, clazz, ParserConfig.global, processor, DEFAULT_PARSER_FEATURE, features);
   }

   public static <T> T parseObject(String json, Type type, Feature... features) {
      return parseObject(json, type, ParserConfig.global, DEFAULT_PARSER_FEATURE, features);
   }

   public static <T> T parseObject(String input, Type clazz, ParseProcess processor, Feature... features) {
      return parseObject(input, clazz, ParserConfig.global, processor, DEFAULT_PARSER_FEATURE, features);
   }

   public static <T> T parseObject(String input, Type clazz, int featureValues, Feature... features) {
      if (input == null) {
         return null;
      } else {
         Feature[] var4 = features;
         int var5 = features.length;

         for(int var6 = 0; var6 < var5; ++var6) {
            Feature feature = var4[var6];
            featureValues = Feature.config(featureValues, feature, true);
         }

         DefaultJSONParser parser = new DefaultJSONParser(input, ParserConfig.getGlobalInstance(), featureValues);
         T value = parser.parseObject(clazz);
         parser.handleResovleTask(value);
         parser.close();
         return value;
      }
   }

   public static <T> T parseObject(String input, Type clazz, ParserConfig config, Feature... features) {
      return parseObject(input, clazz, config, (ParseProcess)null, DEFAULT_PARSER_FEATURE, features);
   }

   public static <T> T parseObject(String input, Type clazz, ParserConfig config, int featureValues, Feature... features) {
      return parseObject(input, clazz, config, (ParseProcess)null, featureValues, features);
   }

   public static <T> T parseObject(String input, Type clazz, ParserConfig config, ParseProcess processor, int featureValues, Feature... features) {
      if (input == null) {
         return null;
      } else {
         if (features != null) {
            Feature[] var6 = features;
            int var7 = features.length;

            for(int var8 = 0; var8 < var7; ++var8) {
               Feature feature = var6[var8];
               featureValues |= feature.mask;
            }
         }

         DefaultJSONParser parser = new DefaultJSONParser(input, config, featureValues);
         if (processor != null) {
            if (processor instanceof ExtraTypeProvider) {
               parser.getExtraTypeProviders().add((ExtraTypeProvider)processor);
            }

            if (processor instanceof ExtraProcessor) {
               parser.getExtraProcessors().add((ExtraProcessor)processor);
            }

            if (processor instanceof FieldTypeResolver) {
               parser.setFieldTypeResolver((FieldTypeResolver)processor);
            }
         }

         T value = parser.parseObject((Type)clazz, (Object)null);
         parser.handleResovleTask(value);
         parser.close();
         return value;
      }
   }

   public static <T> T parseObject(byte[] bytes, Type clazz, Feature... features) {
      return parseObject(bytes, 0, bytes.length, (Charset)IOUtils.UTF8, clazz, features);
   }

   public static <T> T parseObject(byte[] bytes, int offset, int len, Charset charset, Type clazz, Feature... features) {
      if (charset == null) {
         charset = IOUtils.UTF8;
      }

      String strVal;
      if (charset == IOUtils.UTF8) {
         char[] chars = allocateChars(bytes.length);
         int chars_len = IOUtils.decodeUTF8(bytes, offset, len, chars);
         if (chars_len < 0) {
            return null;
         }

         strVal = new String(chars, 0, chars_len);
      } else {
         if (len < 0) {
            return null;
         }

         strVal = new String(bytes, offset, len, charset);
      }

      return parseObject(strVal, clazz, features);
   }

   public static <T> T parseObject(byte[] input, int off, int len, CharsetDecoder charsetDecoder, Type clazz, Feature... features) {
      charsetDecoder.reset();
      int scaleLength = (int)((double)len * (double)charsetDecoder.maxCharsPerByte());
      char[] chars = allocateChars(scaleLength);
      ByteBuffer byteBuf = ByteBuffer.wrap(input, off, len);
      CharBuffer charByte = CharBuffer.wrap(chars);
      IOUtils.decode(charsetDecoder, byteBuf, charByte);
      int position = charByte.position();
      return parseObject(chars, position, clazz, features);
   }

   public static <T> T parseObject(char[] input, int length, Type clazz, Feature... features) {
      if (input != null && input.length != 0) {
         int featureValues = DEFAULT_PARSER_FEATURE;
         Feature[] var5 = features;
         int var6 = features.length;

         for(int var7 = 0; var7 < var6; ++var7) {
            Feature feature = var5[var7];
            featureValues = Feature.config(featureValues, feature, true);
         }

         DefaultJSONParser parser = new DefaultJSONParser(input, length, ParserConfig.getGlobalInstance(), featureValues);
         T value = parser.parseObject(clazz);
         parser.handleResovleTask(value);
         parser.close();
         return value;
      } else {
         return null;
      }
   }

   public static <T> T parseObject(InputStream is, Type type, Feature... features) throws IOException {
      return parseObject(is, IOUtils.UTF8, type, features);
   }

   public static <T> T parseObject(InputStream is, Charset charset, Type type, Feature... features) throws IOException {
      if (charset == null) {
         charset = IOUtils.UTF8;
      }

      byte[] bytes = allocateBytes(65536);
      int offset = 0;

      while(true) {
         int readCount = is.read(bytes, offset, bytes.length - offset);
         if (readCount == -1) {
            return parseObject(bytes, 0, offset, (Charset)charset, type, features);
         }

         offset += readCount;
         if (offset == bytes.length) {
            byte[] newBytes = new byte[bytes.length * 3 / 2];
            System.arraycopy(bytes, 0, newBytes, 0, bytes.length);
            bytes = newBytes;
         }
      }
   }

   public static <T> T parseObject(String text, Class<T> clazz) {
      return parseObject(text, clazz);
   }

   public static JSONArray parseArray(String text) {
      if (text == null) {
         return null;
      } else {
         DefaultJSONParser parser = new DefaultJSONParser(text, ParserConfig.getGlobalInstance());
         JSONLexer lexer = parser.lexer;
         JSONArray array;
         if (lexer.token() == 8) {
            lexer.nextToken();
            array = null;
         } else if (lexer.token() == 20) {
            array = null;
         } else {
            array = new JSONArray();
            parser.parseArray((Collection)array);
            parser.handleResovleTask(array);
         }

         parser.close();
         return array;
      }
   }

   public static <T> List<T> parseArray(String text, Class<T> clazz) {
      if (text == null) {
         return null;
      } else {
         DefaultJSONParser parser = new DefaultJSONParser(text, ParserConfig.getGlobalInstance());
         JSONLexer lexer = parser.lexer;
         int token = lexer.token();
         ArrayList list;
         if (token == 8) {
            lexer.nextToken();
            list = null;
         } else if (token == 20 && lexer.isBlankInput()) {
            list = null;
         } else {
            list = new ArrayList();
            parser.parseArray((Class)clazz, (Collection)list);
            parser.handleResovleTask(list);
         }

         parser.close();
         return list;
      }
   }

   public static List<Object> parseArray(String text, Type[] types) {
      if (text == null) {
         return null;
      } else {
         DefaultJSONParser parser = new DefaultJSONParser(text, ParserConfig.getGlobalInstance());
         Object[] objectArray = parser.parseArray(types);
         List list;
         if (objectArray == null) {
            list = null;
         } else {
            list = Arrays.asList(objectArray);
         }

         parser.handleResovleTask(list);
         parser.close();
         return list;
      }
   }

   public static String toJSONString(Object object) {
      return toJSONString(object, emptyFilters);
   }

   public static String toJSONString(Object object, SerializerFeature... features) {
      return toJSONString(object, DEFAULT_GENERATE_FEATURE, features);
   }

   public static String toJSONString(Object object, int defaultFeatures, SerializerFeature... features) {
      SerializeWriter out = new SerializeWriter((Writer)null, defaultFeatures, features);

      String var5;
      try {
         JSONSerializer serializer = new JSONSerializer(out);
         serializer.write(object);
         var5 = out.toString();
      } finally {
         out.close();
      }

      return var5;
   }

   public static String toJSONStringWithDateFormat(Object object, String dateFormat, SerializerFeature... features) {
      return toJSONString(object, SerializeConfig.globalInstance, (SerializeFilter[])null, dateFormat, DEFAULT_GENERATE_FEATURE, features);
   }

   public static String toJSONString(Object object, SerializeFilter filter, SerializerFeature... features) {
      return toJSONString(object, SerializeConfig.globalInstance, new SerializeFilter[]{filter}, (String)null, DEFAULT_GENERATE_FEATURE, features);
   }

   public static String toJSONString(Object object, SerializeFilter[] filters, SerializerFeature... features) {
      return toJSONString(object, SerializeConfig.globalInstance, filters, (String)null, DEFAULT_GENERATE_FEATURE, features);
   }

   public static byte[] toJSONBytes(Object object, SerializerFeature... features) {
      return toJSONBytes(object, DEFAULT_GENERATE_FEATURE, features);
   }

   public static byte[] toJSONBytes(Object object, SerializeFilter filter, SerializerFeature... features) {
      return toJSONBytes(object, SerializeConfig.globalInstance, new SerializeFilter[]{filter}, DEFAULT_GENERATE_FEATURE, features);
   }

   public static byte[] toJSONBytes(Object object, int defaultFeatures, SerializerFeature... features) {
      return toJSONBytes(object, SerializeConfig.globalInstance, defaultFeatures, features);
   }

   public static String toJSONString(Object object, SerializeConfig config, SerializerFeature... features) {
      return toJSONString(object, config, (SerializeFilter)null, features);
   }

   public static String toJSONString(Object object, SerializeConfig config, SerializeFilter filter, SerializerFeature... features) {
      return toJSONString(object, config, new SerializeFilter[]{filter}, (String)null, DEFAULT_GENERATE_FEATURE, features);
   }

   public static String toJSONString(Object object, SerializeConfig config, SerializeFilter[] filters, SerializerFeature... features) {
      return toJSONString(object, config, filters, (String)null, DEFAULT_GENERATE_FEATURE, features);
   }

   public static String toJSONString(Object object, SerializeConfig config, SerializeFilter[] filters, String dateFormat, int defaultFeatures, SerializerFeature... features) {
      SerializeWriter out = new SerializeWriter((Writer)null, defaultFeatures, features);

      String var15;
      try {
         JSONSerializer serializer = new JSONSerializer(out, config);
         if (dateFormat != null && dateFormat.length() != 0) {
            serializer.setDateFormat(dateFormat);
            serializer.config(SerializerFeature.WriteDateUseDateFormat, true);
         }

         if (filters != null) {
            SerializeFilter[] var8 = filters;
            int var9 = filters.length;

            for(int var10 = 0; var10 < var9; ++var10) {
               SerializeFilter filter = var8[var10];
               serializer.addFilter(filter);
            }
         }

         serializer.write(object);
         var15 = out.toString();
      } finally {
         out.close();
      }

      return var15;
   }

   /** @deprecated */
   public static String toJSONStringZ(Object object, SerializeConfig mapping, SerializerFeature... features) {
      return toJSONString(object, mapping, emptyFilters, (String)null, 0, features);
   }

   public static byte[] toJSONBytes(Object object, SerializeConfig config, SerializerFeature... features) {
      return toJSONBytes(object, config, emptyFilters, DEFAULT_GENERATE_FEATURE, features);
   }

   public static byte[] toJSONBytes(Object object, SerializeConfig config, int defaultFeatures, SerializerFeature... features) {
      return toJSONBytes(object, config, emptyFilters, defaultFeatures, features);
   }

   public static byte[] toJSONBytes(Object object, SerializeFilter[] filters, SerializerFeature... features) {
      return toJSONBytes(object, SerializeConfig.globalInstance, filters, DEFAULT_GENERATE_FEATURE, features);
   }

   public static byte[] toJSONBytes(Object object, SerializeConfig config, SerializeFilter filter, SerializerFeature... features) {
      return toJSONBytes(object, config, new SerializeFilter[]{filter}, DEFAULT_GENERATE_FEATURE, features);
   }

   public static byte[] toJSONBytes(Object object, SerializeConfig config, SerializeFilter[] filters, int defaultFeatures, SerializerFeature... features) {
      SerializeWriter out = new SerializeWriter((Writer)null, defaultFeatures, features);

      byte[] var14;
      try {
         JSONSerializer serializer = new JSONSerializer(out, config);
         if (filters != null) {
            SerializeFilter[] var7 = filters;
            int var8 = filters.length;

            for(int var9 = 0; var9 < var8; ++var9) {
               SerializeFilter filter = var7[var9];
               serializer.addFilter(filter);
            }
         }

         serializer.write(object);
         var14 = out.toBytes(IOUtils.UTF8);
      } finally {
         out.close();
      }

      return var14;
   }

   public static String toJSONString(Object object, boolean prettyFormat) {
      return !prettyFormat ? toJSONString(object) : toJSONString(object, SerializerFeature.PrettyFormat);
   }

   /** @deprecated */
   public static void writeJSONStringTo(Object object, Writer writer, SerializerFeature... features) {
      writeJSONString(writer, object, features);
   }

   public static void writeJSONString(Writer writer, Object object, SerializerFeature... features) {
      writeJSONString(writer, object, DEFAULT_GENERATE_FEATURE, features);
   }

   public static void writeJSONString(Writer writer, Object object, int defaultFeatures, SerializerFeature... features) {
      SerializeWriter out = new SerializeWriter(writer, defaultFeatures, features);

      try {
         JSONSerializer serializer = new JSONSerializer(out);
         serializer.write(object);
      } finally {
         out.close();
      }

   }

   public static final int writeJSONString(OutputStream os, Object object, SerializerFeature... features) throws IOException {
      return writeJSONString(os, object, DEFAULT_GENERATE_FEATURE, features);
   }

   public static final int writeJSONString(OutputStream os, Object object, int defaultFeatures, SerializerFeature... features) throws IOException {
      return writeJSONString(os, IOUtils.UTF8, object, SerializeConfig.globalInstance, (SerializeFilter[])null, (String)null, defaultFeatures, features);
   }

   public static final int writeJSONString(OutputStream os, Charset charset, Object object, SerializerFeature... features) throws IOException {
      return writeJSONString(os, charset, object, SerializeConfig.globalInstance, (SerializeFilter[])null, (String)null, DEFAULT_GENERATE_FEATURE, features);
   }

   public static final int writeJSONString(OutputStream os, Charset charset, Object object, SerializeConfig config, SerializeFilter[] filters, String dateFormat, int defaultFeatures, SerializerFeature... features) throws IOException {
      SerializeWriter writer = new SerializeWriter((Writer)null, defaultFeatures, features);

      int var11;
      try {
         JSONSerializer serializer = new JSONSerializer(writer, config);
         if (dateFormat != null && dateFormat.length() != 0) {
            serializer.setDateFormat(dateFormat);
            serializer.config(SerializerFeature.WriteDateUseDateFormat, true);
         }

         if (filters != null) {
            SerializeFilter[] var10 = filters;
            var11 = filters.length;

            for(int var12 = 0; var12 < var11; ++var12) {
               SerializeFilter filter = var10[var12];
               serializer.addFilter(filter);
            }
         }

         serializer.write(object);
         int len = writer.writeToEx(os, charset);
         var11 = len;
      } finally {
         writer.close();
      }

      return var11;
   }

   public String toString() {
      return this.toJSONString();
   }

   public String toJSONString() {
      SerializeWriter out = new SerializeWriter();

      String var2;
      try {
         (new JSONSerializer(out)).write((Object)this);
         var2 = out.toString();
      } finally {
         out.close();
      }

      return var2;
   }

   public void writeJSONString(Appendable appendable) {
      SerializeWriter out = new SerializeWriter();

      try {
         (new JSONSerializer(out)).write((Object)this);
         appendable.append(out.toString());
      } catch (IOException var7) {
         throw new JSONException(var7.getMessage(), var7);
      } finally {
         out.close();
      }

   }

   public static Object toJSON(Object javaObject) {
      return toJSON(javaObject, SerializeConfig.globalInstance);
   }

   /** @deprecated */
   public static Object toJSON(Object javaObject, ParserConfig parserConfig) {
      return toJSON(javaObject, SerializeConfig.globalInstance);
   }

   public static Object toJSON(Object javaObject, SerializeConfig config) {
      if (javaObject == null) {
         return null;
      } else if (javaObject instanceof JSON) {
         return javaObject;
      } else {
         JSONObject json;
         int len;
         if (javaObject instanceof Map) {
            Map<Object, Object> map = (Map)javaObject;
            len = map.size();
            Object innerMap;
            if (map instanceof LinkedHashMap) {
               innerMap = new LinkedHashMap(len);
            } else if (map instanceof TreeMap) {
               innerMap = new TreeMap();
            } else {
               innerMap = new HashMap(len);
            }

            json = new JSONObject((Map)innerMap);
            Iterator var24 = map.entrySet().iterator();

            while(var24.hasNext()) {
               Entry<Object, Object> entry = (Entry)var24.next();
               Object key = entry.getKey();
               String jsonKey = TypeUtils.castToString(key);
               Object jsonValue = toJSON(entry.getValue());
               json.put(jsonKey, jsonValue);
            }

            return json;
         } else {
            Object item;
            if (javaObject instanceof Collection) {
               Collection<Object> collection = (Collection)javaObject;
               JSONArray array = new JSONArray(collection.size());
               Iterator var19 = collection.iterator();

               while(var19.hasNext()) {
                  Object item = var19.next();
                  item = toJSON(item);
                  array.add(item);
               }

               return array;
            } else if (javaObject instanceof JSONSerializable) {
               String json = toJSONString(javaObject);
               return parse(json);
            } else {
               Class<?> clazz = javaObject.getClass();
               if (clazz.isEnum()) {
                  return ((Enum)javaObject).name();
               } else if (clazz.isArray()) {
                  len = Array.getLength(javaObject);
                  JSONArray array = new JSONArray(len);

                  for(int i = 0; i < len; ++i) {
                     item = Array.get(javaObject, i);
                     Object jsonValue = toJSON(item);
                     array.add(jsonValue);
                  }

                  return array;
               } else if (ParserConfig.isPrimitive2(clazz)) {
                  return javaObject;
               } else {
                  ObjectSerializer serializer = config.getObjectWriter(clazz);
                  if (serializer instanceof JavaBeanSerializer) {
                     JavaBeanSerializer javaBeanSerializer = (JavaBeanSerializer)serializer;
                     json = new JSONObject();

                     try {
                        Map<String, Object> values = javaBeanSerializer.getFieldValuesMap(javaObject);
                        Iterator var7 = values.entrySet().iterator();

                        while(var7.hasNext()) {
                           Entry<String, Object> entry = (Entry)var7.next();
                           json.put((String)entry.getKey(), toJSON(entry.getValue()));
                        }

                        return json;
                     } catch (Exception var11) {
                        throw new JSONException("toJSON error", var11);
                     }
                  } else {
                     String text = toJSONString(javaObject);
                     return parse(text);
                  }
               }
            }
         }
      }
   }

   public static <T> T toJavaObject(JSON json, Class<T> clazz) {
      return TypeUtils.cast(json, (Class)clazz, ParserConfig.getGlobalInstance());
   }

   public <T> T toJavaObject(Class<T> clazz) {
      return TypeUtils.cast(this, (Class)clazz, ParserConfig.getGlobalInstance());
   }

   public <T> T toJavaObject(Type type) {
      return TypeUtils.cast(this, (Type)type, ParserConfig.getGlobalInstance());
   }

   public <T> T toJavaObject(TypeReference typeReference) {
      Type type = typeReference != null ? typeReference.getType() : null;
      return TypeUtils.cast(this, (Type)type, ParserConfig.getGlobalInstance());
   }

   private static byte[] allocateBytes(int length) {
      byte[] chars = (byte[])bytesLocal.get();
      if (chars == null) {
         if (length <= 65536) {
            chars = new byte[65536];
            bytesLocal.set(chars);
         } else {
            chars = new byte[length];
         }
      } else if (chars.length < length) {
         chars = new byte[length];
      }

      return chars;
   }

   private static char[] allocateChars(int length) {
      char[] chars = (char[])charsLocal.get();
      if (chars == null) {
         if (length <= 65536) {
            chars = new char[65536];
            charsLocal.set(chars);
         } else {
            chars = new char[length];
         }
      } else if (chars.length < length) {
         chars = new char[length];
      }

      return chars;
   }

   public static <T> void handleResovleTask(DefaultJSONParser parser, T value) {
      parser.handleResovleTask(value);
   }

   static {
      int features = 0;
      int features = features | Feature.AutoCloseSource.getMask();
      features |= Feature.InternFieldNames.getMask();
      features |= Feature.UseBigDecimal.getMask();
      features |= Feature.AllowUnQuotedFieldNames.getMask();
      features |= Feature.AllowSingleQuotes.getMask();
      features |= Feature.AllowArbitraryCommas.getMask();
      features |= Feature.SortFeidFastMatch.getMask();
      features |= Feature.IgnoreNotMatch.getMask();
      DEFAULT_PARSER_FEATURE = features;
      features = 0;
      features = features | SerializerFeature.QuoteFieldNames.getMask();
      features |= SerializerFeature.SkipTransientField.getMask();
      features |= SerializerFeature.WriteEnumUsingName.getMask();
      features |= SerializerFeature.SortField.getMask();
      String featuresProperty = IOUtils.getStringProperty("fastjson.serializerFeatures.MapSortField");
      int mask = SerializerFeature.MapSortField.getMask();
      if ("true".equals(featuresProperty)) {
         features |= mask;
      } else if ("false".equals(featuresProperty)) {
         features &= ~mask;
      }

      DEFAULT_GENERATE_FEATURE = features;
      bytesLocal = new ThreadLocal();
      charsLocal = new ThreadLocal();
   }
}
