package com.alibaba.fastjson;

import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.parser.deserializer.FieldDeserializer;
import com.alibaba.fastjson.parser.deserializer.JavaBeanDeserializer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import com.alibaba.fastjson.serializer.FieldSerializer;
import com.alibaba.fastjson.serializer.JavaBeanSerializer;
import com.alibaba.fastjson.serializer.ObjectSerializer;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.alibaba.fastjson.util.IOUtils;
import com.alibaba.fastjson.util.TypeUtils;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class JSONPath implements JSONAware {
   private static ConcurrentMap<String, JSONPath> pathCache = new ConcurrentHashMap(128, 0.75F, 1);
   private final String path;
   private JSONPath.Segement[] segments;
   private SerializeConfig serializeConfig;
   private ParserConfig parserConfig;
   static final long SIZE = 5614464919154503228L;
   static final long LENGTH = -1580386065683472715L;

   public JSONPath(String path) {
      this(path, SerializeConfig.getGlobalInstance(), ParserConfig.getGlobalInstance());
   }

   public JSONPath(String path, SerializeConfig serializeConfig, ParserConfig parserConfig) {
      if (path != null && path.length() != 0) {
         this.path = path;
         this.serializeConfig = serializeConfig;
         this.parserConfig = parserConfig;
      } else {
         throw new JSONPathException("json-path can not be null or empty");
      }
   }

   protected void init() {
      if (this.segments == null) {
         if ("*".equals(this.path)) {
            this.segments = new JSONPath.Segement[]{JSONPath.WildCardSegement.instance};
         } else {
            JSONPath.JSONPathParser parser = new JSONPath.JSONPathParser(this.path);
            this.segments = parser.explain();
         }

      }
   }

   public Object eval(Object rootObject) {
      if (rootObject == null) {
         return null;
      } else {
         this.init();
         Object currentObject = rootObject;

         for(int i = 0; i < this.segments.length; ++i) {
            JSONPath.Segement segement = this.segments[i];
            currentObject = segement.eval(this, rootObject, currentObject);
         }

         return currentObject;
      }
   }

   public boolean contains(Object rootObject) {
      if (rootObject == null) {
         return false;
      } else {
         this.init();
         Object currentObject = rootObject;

         for(int i = 0; i < this.segments.length; ++i) {
            currentObject = this.segments[i].eval(this, rootObject, currentObject);
            if (currentObject == null) {
               return false;
            }
         }

         return true;
      }
   }

   public boolean containsValue(Object rootObject, Object value) {
      Object currentObject = this.eval(rootObject);
      if (currentObject == value) {
         return true;
      } else if (currentObject == null) {
         return false;
      } else if (currentObject instanceof Iterable) {
         Iterator it = ((Iterable)currentObject).iterator();

         Object item;
         do {
            if (!it.hasNext()) {
               return false;
            }

            item = it.next();
         } while(!eq(item, value));

         return true;
      } else {
         return eq(currentObject, value);
      }
   }

   public int size(Object rootObject) {
      if (rootObject == null) {
         return -1;
      } else {
         this.init();
         Object currentObject = rootObject;

         for(int i = 0; i < this.segments.length; ++i) {
            currentObject = this.segments[i].eval(this, rootObject, currentObject);
         }

         return this.evalSize(currentObject);
      }
   }

   public void arrayAdd(Object rootObject, Object... values) {
      if (values != null && values.length != 0) {
         if (rootObject != null) {
            this.init();
            Object currentObject = rootObject;
            Object parentObject = null;

            for(int i = 0; i < this.segments.length; ++i) {
               if (i == this.segments.length - 1) {
                  parentObject = currentObject;
               }

               currentObject = this.segments[i].eval(this, rootObject, currentObject);
            }

            if (currentObject == null) {
               throw new JSONPathException("value not found in path " + this.path);
            } else {
               int length;
               if (currentObject instanceof Collection) {
                  Collection collection = (Collection)currentObject;
                  Object[] var12 = values;
                  length = values.length;

                  for(int var15 = 0; var15 < length; ++var15) {
                     Object value = var12[var15];
                     collection.add(value);
                  }

               } else {
                  Class<?> resultClass = currentObject.getClass();
                  if (!resultClass.isArray()) {
                     throw new JSONException("unsupported array put operation. " + resultClass);
                  } else {
                     length = Array.getLength(currentObject);
                     Object descArray = Array.newInstance(resultClass.getComponentType(), length + values.length);
                     System.arraycopy(currentObject, 0, descArray, 0, length);

                     for(int i = 0; i < values.length; ++i) {
                        Array.set(descArray, length + i, values[i]);
                     }

                     JSONPath.Segement lastSegement = this.segments[this.segments.length - 1];
                     if (lastSegement instanceof JSONPath.PropertySegement) {
                        JSONPath.PropertySegement propertySegement = (JSONPath.PropertySegement)lastSegement;
                        propertySegement.setValue(this, parentObject, descArray);
                     } else if (lastSegement instanceof JSONPath.ArrayAccessSegement) {
                        ((JSONPath.ArrayAccessSegement)lastSegement).setValue(this, parentObject, descArray);
                     } else {
                        throw new UnsupportedOperationException();
                     }
                  }
               }
            }
         }
      }
   }

   public boolean remove(Object rootObject) {
      if (rootObject == null) {
         return false;
      } else {
         this.init();
         Object currentObject = rootObject;
         Object parentObject = null;

         for(int i = 0; i < this.segments.length; ++i) {
            if (i == this.segments.length - 1) {
               parentObject = currentObject;
               break;
            }

            currentObject = this.segments[i].eval(this, rootObject, currentObject);
            if (currentObject == null) {
               break;
            }
         }

         if (parentObject == null) {
            return false;
         } else {
            JSONPath.Segement lastSegement = this.segments[this.segments.length - 1];
            if (!(lastSegement instanceof JSONPath.PropertySegement)) {
               if (lastSegement instanceof JSONPath.ArrayAccessSegement) {
                  return ((JSONPath.ArrayAccessSegement)lastSegement).remove(this, parentObject);
               } else {
                  throw new UnsupportedOperationException();
               }
            } else {
               JSONPath.PropertySegement propertySegement = (JSONPath.PropertySegement)lastSegement;
               if (parentObject instanceof Collection && this.segments.length > 1) {
                  JSONPath.Segement parentSegement = this.segments[this.segments.length - 2];
                  if (parentSegement instanceof JSONPath.RangeSegement || parentSegement instanceof JSONPath.MultiIndexSegement) {
                     Collection collection = (Collection)parentObject;
                     boolean removedOnce = false;
                     Iterator var9 = collection.iterator();

                     while(var9.hasNext()) {
                        Object item = var9.next();
                        boolean removed = propertySegement.remove(this, item);
                        if (removed) {
                           removedOnce = true;
                        }
                     }

                     return removedOnce;
                  }
               }

               return propertySegement.remove(this, parentObject);
            }
         }
      }
   }

   public boolean set(Object rootObject, Object value) {
      return this.set(rootObject, value, true);
   }

   public boolean set(Object rootObject, Object value, boolean p) {
      if (rootObject == null) {
         return false;
      } else {
         this.init();
         Object currentObject = rootObject;
         Object parentObject = null;

         for(int i = 0; i < this.segments.length; ++i) {
            parentObject = currentObject;
            JSONPath.Segement segment = this.segments[i];
            currentObject = segment.eval(this, rootObject, currentObject);
            if (currentObject == null) {
               JSONPath.Segement nextSegement = null;
               if (i < this.segments.length - 1) {
                  nextSegement = this.segments[i + 1];
               }

               Object newObj = null;
               if (nextSegement instanceof JSONPath.PropertySegement) {
                  JavaBeanDeserializer beanDeserializer = null;
                  Class<?> fieldClass = null;
                  if (segment instanceof JSONPath.PropertySegement) {
                     String propertyName = ((JSONPath.PropertySegement)segment).propertyName;
                     Class<?> parentClass = parentObject.getClass();
                     JavaBeanDeserializer parentBeanDeserializer = this.getJavaBeanDeserializer(parentClass);
                     if (parentBeanDeserializer != null) {
                        FieldDeserializer fieldDeserializer = parentBeanDeserializer.getFieldDeserializer(propertyName);
                        fieldClass = fieldDeserializer.fieldInfo.fieldClass;
                        beanDeserializer = this.getJavaBeanDeserializer(fieldClass);
                     }
                  }

                  if (beanDeserializer != null) {
                     if (beanDeserializer.beanInfo.defaultConstructor == null) {
                        return false;
                     }

                     newObj = beanDeserializer.createInstance((DefaultJSONParser)null, (Type)fieldClass);
                  } else {
                     newObj = new JSONObject();
                  }
               } else if (nextSegement instanceof JSONPath.ArrayAccessSegement) {
                  newObj = new JSONArray();
               }

               if (newObj == null) {
                  break;
               }

               if (segment instanceof JSONPath.PropertySegement) {
                  JSONPath.PropertySegement propSegement = (JSONPath.PropertySegement)segment;
                  propSegement.setValue(this, parentObject, newObj);
                  currentObject = newObj;
               } else {
                  if (!(segment instanceof JSONPath.ArrayAccessSegement)) {
                     break;
                  }

                  JSONPath.ArrayAccessSegement arrayAccessSegement = (JSONPath.ArrayAccessSegement)segment;
                  arrayAccessSegement.setValue(this, parentObject, newObj);
                  currentObject = newObj;
               }
            }
         }

         if (parentObject == null) {
            return false;
         } else {
            JSONPath.Segement lastSegement = this.segments[this.segments.length - 1];
            if (lastSegement instanceof JSONPath.PropertySegement) {
               JSONPath.PropertySegement propertySegement = (JSONPath.PropertySegement)lastSegement;
               propertySegement.setValue(this, parentObject, value);
               return true;
            } else if (lastSegement instanceof JSONPath.ArrayAccessSegement) {
               return ((JSONPath.ArrayAccessSegement)lastSegement).setValue(this, parentObject, value);
            } else {
               throw new UnsupportedOperationException();
            }
         }
      }
   }

   public static Object eval(Object rootObject, String path) {
      JSONPath jsonpath = compile(path);
      return jsonpath.eval(rootObject);
   }

   public static int size(Object rootObject, String path) {
      JSONPath jsonpath = compile(path);
      Object result = jsonpath.eval(rootObject);
      return jsonpath.evalSize(result);
   }

   public static boolean contains(Object rootObject, String path) {
      if (rootObject == null) {
         return false;
      } else {
         JSONPath jsonpath = compile(path);
         return jsonpath.contains(rootObject);
      }
   }

   public static boolean containsValue(Object rootObject, String path, Object value) {
      JSONPath jsonpath = compile(path);
      return jsonpath.containsValue(rootObject, value);
   }

   public static void arrayAdd(Object rootObject, String path, Object... values) {
      JSONPath jsonpath = compile(path);
      jsonpath.arrayAdd(rootObject, values);
   }

   public static boolean set(Object rootObject, String path, Object value) {
      JSONPath jsonpath = compile(path);
      return jsonpath.set(rootObject, value);
   }

   public static boolean remove(Object root, String path) {
      JSONPath jsonpath = compile(path);
      return jsonpath.remove(root);
   }

   public static JSONPath compile(String path) {
      if (path == null) {
         throw new JSONPathException("jsonpath can not be null");
      } else {
         JSONPath jsonpath = (JSONPath)pathCache.get(path);
         if (jsonpath == null) {
            jsonpath = new JSONPath(path);
            if (pathCache.size() < 1024) {
               pathCache.putIfAbsent(path, jsonpath);
               jsonpath = (JSONPath)pathCache.get(path);
            }
         }

         return jsonpath;
      }
   }

   public static Object read(String json, String path) {
      Object object = JSON.parse(json);
      JSONPath jsonpath = compile(path);
      return jsonpath.eval(object);
   }

   public static Map<String, Object> paths(Object javaObject) {
      return paths(javaObject, SerializeConfig.globalInstance);
   }

   public static Map<String, Object> paths(Object javaObject, SerializeConfig config) {
      Map<Object, String> values = new IdentityHashMap();
      Map<String, Object> paths = new HashMap();
      paths(values, paths, "/", javaObject, config);
      return paths;
   }

   private static void paths(Map<Object, String> values, Map<String, Object> paths, String parent, Object javaObject, SerializeConfig config) {
      if (javaObject != null) {
         String p = (String)values.put(javaObject, parent);
         if (p != null) {
            boolean basicType = javaObject instanceof String || javaObject instanceof Number || javaObject instanceof Date || javaObject instanceof UUID;
            if (!basicType) {
               return;
            }
         }

         paths.put(parent, javaObject);
         if (javaObject instanceof Map) {
            Map map = (Map)javaObject;
            Iterator var19 = map.entrySet().iterator();

            while(var19.hasNext()) {
               Object entryObj = var19.next();
               Entry entry = (Entry)entryObj;
               Object key = entry.getKey();
               if (key instanceof String) {
                  String path = parent.equals("/") ? "/" + key : parent + "/" + key;
                  paths(values, paths, path, entry.getValue(), config);
               }
            }

         } else {
            int len;
            Object item;
            String path;
            if (javaObject instanceof Collection) {
               Collection collection = (Collection)javaObject;
               len = 0;

               for(Iterator var21 = collection.iterator(); var21.hasNext(); ++len) {
                  item = var21.next();
                  path = parent.equals("/") ? "/" + len : parent + "/" + len;
                  paths(values, paths, path, item, config);
               }

            } else {
               Class<?> clazz = javaObject.getClass();
               if (clazz.isArray()) {
                  len = Array.getLength(javaObject);

                  for(int i = 0; i < len; ++i) {
                     item = Array.get(javaObject, i);
                     path = parent.equals("/") ? "/" + i : parent + "/" + i;
                     paths(values, paths, path, item, config);
                  }

               } else if (!ParserConfig.isPrimitive2(clazz) && !clazz.isEnum()) {
                  ObjectSerializer serializer = config.getObjectWriter(clazz);
                  if (serializer instanceof JavaBeanSerializer) {
                     JavaBeanSerializer javaBeanSerializer = (JavaBeanSerializer)serializer;

                     try {
                        Map<String, Object> fieldValues = javaBeanSerializer.getFieldValuesMap(javaObject);
                        Iterator var10 = fieldValues.entrySet().iterator();

                        while(var10.hasNext()) {
                           Entry<String, Object> entry = (Entry)var10.next();
                           String key = (String)entry.getKey();
                           if (key instanceof String) {
                              String path = parent.equals("/") ? "/" + key : parent + "/" + key;
                              paths(values, paths, path, entry.getValue(), config);
                           }
                        }

                     } catch (Exception var14) {
                        throw new JSONException("toJSON error", var14);
                     }
                  }
               }
            }
         }
      }
   }

   private static void paths(Map<Object, String> paths, String parent, Object javaObject, SerializeConfig config) {
      if (javaObject != null) {
         if (!paths.containsKey(javaObject)) {
            paths.put(javaObject, parent);
            if (javaObject instanceof Map) {
               Map map = (Map)javaObject;
               Iterator var16 = map.entrySet().iterator();

               while(var16.hasNext()) {
                  Object entryObj = var16.next();
                  Entry entry = (Entry)entryObj;
                  Object key = entry.getKey();
                  if (key instanceof String) {
                     String path = parent.equals("/") ? "/" + key : parent + "/" + key;
                     paths(paths, path, entry.getValue(), config);
                  }
               }

            } else {
               int len;
               Object item;
               String path;
               if (javaObject instanceof Collection) {
                  Collection collection = (Collection)javaObject;
                  len = 0;

                  for(Iterator var18 = collection.iterator(); var18.hasNext(); ++len) {
                     item = var18.next();
                     path = parent.equals("/") ? "/" + len : parent + "/" + len;
                     paths(paths, path, item, config);
                  }

               } else {
                  Class<?> clazz = javaObject.getClass();
                  if (clazz.isArray()) {
                     len = Array.getLength(javaObject);

                     for(int i = 0; i < len; ++i) {
                        item = Array.get(javaObject, i);
                        path = parent.equals("/") ? "/" + i : parent + "/" + i;
                        paths(paths, path, item, config);
                     }

                  } else if (!ParserConfig.isPrimitive2(clazz) && !clazz.isEnum()) {
                     ObjectSerializer serializer = config.getObjectWriter(clazz);
                     if (serializer instanceof JavaBeanSerializer) {
                        JavaBeanSerializer javaBeanSerializer = (JavaBeanSerializer)serializer;

                        try {
                           Map<String, Object> fieldValues = javaBeanSerializer.getFieldValuesMap(javaObject);
                           Iterator var8 = fieldValues.entrySet().iterator();

                           while(var8.hasNext()) {
                              Entry<String, Object> entry = (Entry)var8.next();
                              String key = (String)entry.getKey();
                              if (key instanceof String) {
                                 String path = parent.equals("/") ? "/" + key : parent + "/" + key;
                                 paths(paths, path, entry.getValue(), config);
                              }
                           }

                        } catch (Exception var12) {
                           throw new JSONException("toJSON error", var12);
                        }
                     }
                  }
               }
            }
         }
      }
   }

   public String getPath() {
      return this.path;
   }

   protected Object getArrayItem(Object currentObject, int index) {
      if (currentObject == null) {
         return null;
      } else if (currentObject instanceof List) {
         List list = (List)currentObject;
         if (index >= 0) {
            return index < list.size() ? list.get(index) : null;
         } else {
            return Math.abs(index) <= list.size() ? list.get(list.size() + index) : null;
         }
      } else if (currentObject.getClass().isArray()) {
         int arrayLenth = Array.getLength(currentObject);
         if (index >= 0) {
            return index < arrayLenth ? Array.get(currentObject, index) : null;
         } else {
            return Math.abs(index) <= arrayLenth ? Array.get(currentObject, arrayLenth + index) : null;
         }
      } else if (currentObject instanceof Map) {
         Map map = (Map)currentObject;
         Object value = map.get(index);
         if (value == null) {
            value = map.get(Integer.toString(index));
         }

         return value;
      } else if (currentObject instanceof Collection) {
         Collection collection = (Collection)currentObject;
         int i = 0;

         for(Iterator var5 = collection.iterator(); var5.hasNext(); ++i) {
            Object item = var5.next();
            if (i == index) {
               return item;
            }
         }

         return null;
      } else {
         throw new UnsupportedOperationException();
      }
   }

   public boolean setArrayItem(JSONPath path, Object currentObject, int index, Object value) {
      if (currentObject instanceof List) {
         List list = (List)currentObject;
         if (index >= 0) {
            list.set(index, value);
         } else {
            list.set(list.size() + index, value);
         }

         return true;
      } else {
         Class<?> clazz = currentObject.getClass();
         if (clazz.isArray()) {
            int arrayLenth = Array.getLength(currentObject);
            if (index >= 0) {
               if (index < arrayLenth) {
                  Array.set(currentObject, index, value);
               }
            } else if (Math.abs(index) <= arrayLenth) {
               Array.set(currentObject, arrayLenth + index, value);
            }

            return true;
         } else {
            throw new JSONPathException("unsupported set operation." + clazz);
         }
      }
   }

   public boolean removeArrayItem(JSONPath path, Object currentObject, int index) {
      if (currentObject instanceof List) {
         List list = (List)currentObject;
         if (index >= 0) {
            if (index >= list.size()) {
               return false;
            }

            list.remove(index);
         } else {
            int newIndex = list.size() + index;
            if (newIndex < 0) {
               return false;
            }

            list.remove(newIndex);
         }

         return true;
      } else {
         Class<?> clazz = currentObject.getClass();
         throw new JSONPathException("unsupported set operation." + clazz);
      }
   }

   protected Collection<Object> getPropertyValues(Object currentObject) {
      Class<?> currentClass = currentObject.getClass();
      JavaBeanSerializer beanSerializer = this.getJavaBeanSerializer(currentClass);
      if (beanSerializer != null) {
         try {
            return beanSerializer.getFieldValues(currentObject);
         } catch (Exception var5) {
            throw new JSONPathException("jsonpath error, path " + this.path, var5);
         }
      } else if (currentObject instanceof Map) {
         Map map = (Map)currentObject;
         return map.values();
      } else {
         throw new UnsupportedOperationException();
      }
   }

   static boolean eq(Object a, Object b) {
      if (a == b) {
         return true;
      } else if (a != null && b != null) {
         if (a.getClass() == b.getClass()) {
            return a.equals(b);
         } else if (a instanceof Number) {
            return b instanceof Number ? eqNotNull((Number)a, (Number)b) : false;
         } else {
            return a.equals(b);
         }
      } else {
         return false;
      }
   }

   static boolean eqNotNull(Number a, Number b) {
      Class clazzA = a.getClass();
      boolean isIntA = isInt(clazzA);
      Class clazzB = b.getClass();
      boolean isIntB = isInt(clazzB);
      if (a instanceof BigDecimal) {
         BigDecimal decimalA = (BigDecimal)a;
         if (isIntB) {
            return decimalA.equals(BigDecimal.valueOf(b.longValue()));
         }
      }

      BigInteger bigIntB;
      BigInteger bigIntA;
      if (isIntA) {
         if (isIntB) {
            return a.longValue() == b.longValue();
         }

         if (b instanceof BigInteger) {
            bigIntB = (BigInteger)a;
            bigIntA = BigInteger.valueOf(a.longValue());
            return bigIntA.equals(bigIntB);
         }
      }

      if (isIntB && a instanceof BigInteger) {
         bigIntB = (BigInteger)a;
         bigIntA = BigInteger.valueOf(b.longValue());
         return bigIntB.equals(bigIntA);
      } else {
         boolean isDoubleA = isDouble(clazzA);
         boolean isDoubleB = isDouble(clazzB);
         if ((!isDoubleA || !isDoubleB) && (!isDoubleA || !isIntB) && (!isDoubleB || !isIntA)) {
            return false;
         } else {
            return a.doubleValue() == b.doubleValue();
         }
      }
   }

   protected static boolean isDouble(Class<?> clazzA) {
      return clazzA == Float.class || clazzA == Double.class;
   }

   protected static boolean isInt(Class<?> clazzA) {
      return clazzA == Byte.class || clazzA == Short.class || clazzA == Integer.class || clazzA == Long.class;
   }

   protected Object getPropertyValue(Object currentObject, String propertyName, long propertyNameHash) {
      if (currentObject == null) {
         return null;
      } else if (!(currentObject instanceof Map)) {
         Class<?> currentClass = currentObject.getClass();
         JavaBeanSerializer beanSerializer = this.getJavaBeanSerializer(currentClass);
         if (beanSerializer != null) {
            try {
               return beanSerializer.getFieldValue(currentObject, propertyName, propertyNameHash, false);
            } catch (Exception var20) {
               throw new JSONPathException("jsonpath error, path " + this.path + ", segement " + propertyName, var20);
            }
         } else if (currentObject instanceof List) {
            List list = (List)currentObject;
            if (5614464919154503228L != propertyNameHash && -1580386065683472715L != propertyNameHash) {
               List<Object> fieldValues = new JSONArray(list.size());

               for(int i = 0; i < list.size(); ++i) {
                  Object obj = list.get(i);
                  if (obj == list) {
                     fieldValues.add(obj);
                  } else {
                     Object itemValue = this.getPropertyValue(obj, propertyName, propertyNameHash);
                     if (itemValue instanceof Collection) {
                        Collection collection = (Collection)itemValue;
                        fieldValues.addAll(collection);
                     } else if (itemValue != null) {
                        fieldValues.add(itemValue);
                     }
                  }
               }

               return fieldValues;
            } else {
               return list.size();
            }
         } else {
            long YEAR;
            long MONTH;
            if (currentObject instanceof Enum) {
               YEAR = -4270347329889690746L;
               MONTH = -1014497654951707614L;
               Enum e = (Enum)currentObject;
               if (-4270347329889690746L == propertyNameHash) {
                  return e.name();
               }

               if (-1014497654951707614L == propertyNameHash) {
                  return e.ordinal();
               }
            }

            if (currentObject instanceof Calendar) {
               YEAR = 8963398325558730460L;
               MONTH = -811277319855450459L;
               long DAY = -3851359326990528739L;
               long HOUR = 4647432019745535567L;
               long MINUTE = 6607618197526598121L;
               long SECOND = -6586085717218287427L;
               Calendar e = (Calendar)currentObject;
               if (8963398325558730460L == propertyNameHash) {
                  return e.get(1);
               }

               if (-811277319855450459L == propertyNameHash) {
                  return e.get(2);
               }

               if (-3851359326990528739L == propertyNameHash) {
                  return e.get(5);
               }

               if (4647432019745535567L == propertyNameHash) {
                  return e.get(11);
               }

               if (6607618197526598121L == propertyNameHash) {
                  return e.get(12);
               }

               if (-6586085717218287427L == propertyNameHash) {
                  return e.get(13);
               }
            }

            return null;
         }
      } else {
         Map map = (Map)currentObject;
         Object val = map.get(propertyName);
         if (val == null && (5614464919154503228L == propertyNameHash || -1580386065683472715L == propertyNameHash)) {
            val = map.size();
         }

         return val;
      }
   }

   protected void deepScan(Object currentObject, String propertyName, List<Object> results) {
      if (currentObject != null) {
         if (currentObject instanceof Map) {
            Map<?, ?> map = (Map)currentObject;
            if (map.containsKey(propertyName)) {
               Object val = map.get(propertyName);
               results.add(val);
            } else {
               Iterator var14 = map.values().iterator();

               while(var14.hasNext()) {
                  Object val = var14.next();
                  this.deepScan(val, propertyName, results);
               }

            }
         } else {
            Class<?> currentClass = currentObject.getClass();
            JavaBeanSerializer beanSerializer = this.getJavaBeanSerializer(currentClass);
            if (beanSerializer != null) {
               try {
                  FieldSerializer fieldDeser = beanSerializer.getFieldSerializer(propertyName);
                  if (fieldDeser != null) {
                     try {
                        Object val = fieldDeser.getPropertyValueDirect(currentObject);
                        results.add(val);
                     } catch (InvocationTargetException var10) {
                        throw new JSONException("getFieldValue error." + propertyName, var10);
                     } catch (IllegalAccessException var11) {
                        throw new JSONException("getFieldValue error." + propertyName, var11);
                     }
                  } else {
                     List<Object> fieldValues = beanSerializer.getFieldValues(currentObject);
                     Iterator var20 = fieldValues.iterator();

                     while(var20.hasNext()) {
                        Object val = var20.next();
                        this.deepScan(val, propertyName, results);
                     }

                  }
               } catch (Exception var12) {
                  throw new JSONPathException("jsonpath error, path " + this.path + ", segement " + propertyName, var12);
               }
            } else if (currentObject instanceof List) {
               List list = (List)currentObject;

               for(int i = 0; i < list.size(); ++i) {
                  Object val = list.get(i);
                  this.deepScan(val, propertyName, results);
               }

            }
         }
      }
   }

   protected void deepSet(Object currentObject, String propertyName, long propertyNameHash, Object value) {
      if (currentObject != null) {
         if (currentObject instanceof Map) {
            Map map = (Map)currentObject;
            if (map.containsKey(propertyName)) {
               map.get(propertyName);
               map.put(propertyName, value);
            } else {
               Iterator var15 = map.values().iterator();

               while(var15.hasNext()) {
                  Object val = var15.next();
                  this.deepSet(val, propertyName, propertyNameHash, value);
               }

            }
         } else {
            Class<?> currentClass = currentObject.getClass();
            JavaBeanDeserializer beanDeserializer = this.getJavaBeanDeserializer(currentClass);
            if (beanDeserializer != null) {
               try {
                  FieldDeserializer fieldDeser = beanDeserializer.getFieldDeserializer(propertyName);
                  if (fieldDeser != null) {
                     fieldDeser.setValue(currentObject, value);
                  } else {
                     JavaBeanSerializer beanSerializer = this.getJavaBeanSerializer(currentClass);
                     List<Object> fieldValues = beanSerializer.getObjectFieldValues(currentObject);
                     Iterator var11 = fieldValues.iterator();

                     while(var11.hasNext()) {
                        Object val = var11.next();
                        this.deepSet(val, propertyName, propertyNameHash, value);
                     }

                  }
               } catch (Exception var13) {
                  throw new JSONPathException("jsonpath error, path " + this.path + ", segement " + propertyName, var13);
               }
            } else if (currentObject instanceof List) {
               List list = (List)currentObject;

               for(int i = 0; i < list.size(); ++i) {
                  Object val = list.get(i);
                  this.deepSet(val, propertyName, propertyNameHash, value);
               }

            }
         }
      }
   }

   protected boolean setPropertyValue(Object parent, String name, long propertyNameHash, Object value) {
      if (parent instanceof Map) {
         ((Map)parent).put(name, value);
         return true;
      } else if (parent instanceof List) {
         Iterator var9 = ((List)parent).iterator();

         while(var9.hasNext()) {
            Object element = var9.next();
            if (element != null) {
               this.setPropertyValue(element, name, propertyNameHash, value);
            }
         }

         return true;
      } else {
         ObjectDeserializer derializer = this.parserConfig.getDeserializer((Type)parent.getClass());
         JavaBeanDeserializer beanDerializer = null;
         if (derializer instanceof JavaBeanDeserializer) {
            beanDerializer = (JavaBeanDeserializer)derializer;
         }

         if (beanDerializer != null) {
            FieldDeserializer fieldDeserializer = beanDerializer.getFieldDeserializer(propertyNameHash);
            if (fieldDeserializer == null) {
               return false;
            } else {
               fieldDeserializer.setValue(parent, value);
               return true;
            }
         } else {
            throw new UnsupportedOperationException();
         }
      }
   }

   protected boolean removePropertyValue(Object parent, String name) {
      if (parent instanceof Map) {
         Object origin = ((Map)parent).remove(name);
         return origin != null;
      } else {
         ObjectDeserializer derializer = this.parserConfig.getDeserializer((Type)parent.getClass());
         JavaBeanDeserializer beanDerializer = null;
         if (derializer instanceof JavaBeanDeserializer) {
            beanDerializer = (JavaBeanDeserializer)derializer;
         }

         if (beanDerializer != null) {
            FieldDeserializer fieldDeserializer = beanDerializer.getFieldDeserializer(name);
            if (fieldDeserializer == null) {
               return false;
            } else {
               fieldDeserializer.setValue(parent, (String)null);
               return true;
            }
         } else {
            throw new UnsupportedOperationException();
         }
      }
   }

   protected JavaBeanSerializer getJavaBeanSerializer(Class<?> currentClass) {
      JavaBeanSerializer beanSerializer = null;
      ObjectSerializer serializer = this.serializeConfig.getObjectWriter(currentClass);
      if (serializer instanceof JavaBeanSerializer) {
         beanSerializer = (JavaBeanSerializer)serializer;
      }

      return beanSerializer;
   }

   protected JavaBeanDeserializer getJavaBeanDeserializer(Class<?> currentClass) {
      JavaBeanDeserializer beanDeserializer = null;
      ObjectDeserializer deserializer = this.parserConfig.getDeserializer((Type)currentClass);
      if (deserializer instanceof JavaBeanDeserializer) {
         beanDeserializer = (JavaBeanDeserializer)deserializer;
      }

      return beanDeserializer;
   }

   int evalSize(Object currentObject) {
      if (currentObject == null) {
         return -1;
      } else if (currentObject instanceof Collection) {
         return ((Collection)currentObject).size();
      } else if (currentObject instanceof Object[]) {
         return ((Object[])((Object[])currentObject)).length;
      } else if (currentObject.getClass().isArray()) {
         return Array.getLength(currentObject);
      } else if (currentObject instanceof Map) {
         int count = 0;
         Iterator var3 = ((Map)currentObject).values().iterator();

         while(var3.hasNext()) {
            Object value = var3.next();
            if (value != null) {
               ++count;
            }
         }

         return count;
      } else {
         JavaBeanSerializer beanSerializer = this.getJavaBeanSerializer(currentObject.getClass());
         if (beanSerializer == null) {
            return -1;
         } else {
            try {
               return beanSerializer.getSize(currentObject);
            } catch (Exception var5) {
               throw new JSONPathException("evalSize error : " + this.path, var5);
            }
         }
      }
   }

   public String toJSONString() {
      return JSON.toJSONString(this.path);
   }

   static class FilterGroup implements JSONPath.Filter {
      private boolean and;
      private List<JSONPath.Filter> fitlers;

      public FilterGroup(JSONPath.Filter left, JSONPath.Filter right, boolean and) {
         this.fitlers = new ArrayList(2);
         this.fitlers.add(left);
         this.fitlers.add(right);
         this.and = and;
      }

      public FilterGroup(List<JSONPath.Filter> fitlers, boolean and) {
         this.and = and;
         this.fitlers = fitlers;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Iterator var5;
         JSONPath.Filter fitler;
         if (this.and) {
            var5 = this.fitlers.iterator();

            do {
               if (!var5.hasNext()) {
                  return true;
               }

               fitler = (JSONPath.Filter)var5.next();
            } while(fitler.apply(path, rootObject, currentObject, item));

            return false;
         } else {
            var5 = this.fitlers.iterator();

            do {
               if (!var5.hasNext()) {
                  return false;
               }

               fitler = (JSONPath.Filter)var5.next();
            } while(!fitler.apply(path, rootObject, currentObject, item));

            return true;
         }
      }
   }

   interface Filter {
      boolean apply(JSONPath var1, Object var2, Object var3, Object var4);
   }

   public static class FilterSegement implements JSONPath.Segement {
      private final JSONPath.Filter filter;

      public FilterSegement(JSONPath.Filter filter) {
         this.filter = filter;
      }

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         if (currentObject == null) {
            return null;
         } else {
            List<Object> items = new JSONArray();
            if (currentObject instanceof Iterable) {
               Iterator it = ((Iterable)currentObject).iterator();

               while(it.hasNext()) {
                  Object item = it.next();
                  if (this.filter.apply(path, rootObject, currentObject, item)) {
                     items.add(item);
                  }
               }

               return items;
            } else {
               return this.filter.apply(path, rootObject, currentObject, currentObject) ? currentObject : null;
            }
         }
      }
   }

   static enum Operator {
      EQ,
      NE,
      GT,
      GE,
      LT,
      LE,
      LIKE,
      NOT_LIKE,
      RLIKE,
      NOT_RLIKE,
      IN,
      NOT_IN,
      BETWEEN,
      NOT_BETWEEN,
      And,
      Or;
   }

   static class StringOpSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final String value;
      private final JSONPath.Operator op;

      public StringOpSegement(String propertyName, String value, JSONPath.Operator op) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.value = value;
         this.op = op;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (this.op == JSONPath.Operator.EQ) {
            return this.value.equals(propertyValue);
         } else if (this.op == JSONPath.Operator.NE) {
            return !this.value.equals(propertyValue);
         } else if (propertyValue == null) {
            return false;
         } else {
            int compareResult = this.value.compareTo(propertyValue.toString());
            if (this.op == JSONPath.Operator.GE) {
               return compareResult <= 0;
            } else if (this.op == JSONPath.Operator.GT) {
               return compareResult < 0;
            } else if (this.op == JSONPath.Operator.LE) {
               return compareResult >= 0;
            } else if (this.op == JSONPath.Operator.LT) {
               return compareResult > 0;
            } else {
               return false;
            }
         }
      }
   }

   static class RlikeSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final Pattern pattern;
      private final boolean not;

      public RlikeSegement(String propertyName, String pattern, boolean not) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.pattern = Pattern.compile(pattern);
         this.not = not;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (propertyValue == null) {
            return false;
         } else {
            String strPropertyValue = propertyValue.toString();
            Matcher m = this.pattern.matcher(strPropertyValue);
            boolean match = m.matches();
            if (this.not) {
               match = !match;
            }

            return match;
         }
      }
   }

   static class MatchSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final String startsWithValue;
      private final String endsWithValue;
      private final String[] containsValues;
      private final int minLength;
      private final boolean not;

      public MatchSegement(String propertyName, String startsWithValue, String endsWithValue, String[] containsValues, boolean not) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.startsWithValue = startsWithValue;
         this.endsWithValue = endsWithValue;
         this.containsValues = containsValues;
         this.not = not;
         int len = 0;
         if (startsWithValue != null) {
            len += startsWithValue.length();
         }

         if (endsWithValue != null) {
            len += endsWithValue.length();
         }

         if (containsValues != null) {
            String[] var7 = containsValues;
            int var8 = containsValues.length;

            for(int var9 = 0; var9 < var8; ++var9) {
               String item = var7[var9];
               len += item.length();
            }
         }

         this.minLength = len;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (propertyValue == null) {
            return false;
         } else {
            String strPropertyValue = propertyValue.toString();
            if (strPropertyValue.length() < this.minLength) {
               return this.not;
            } else {
               int start = 0;
               if (this.startsWithValue != null) {
                  if (!strPropertyValue.startsWith(this.startsWithValue)) {
                     return this.not;
                  }

                  start += this.startsWithValue.length();
               }

               if (this.containsValues != null) {
                  String[] var8 = this.containsValues;
                  int var9 = var8.length;

                  for(int var10 = 0; var10 < var9; ++var10) {
                     String containsValue = var8[var10];
                     int index = strPropertyValue.indexOf(containsValue, start);
                     if (index == -1) {
                        return this.not;
                     }

                     start = index + containsValue.length();
                  }
               }

               if (this.endsWithValue != null && !strPropertyValue.endsWith(this.endsWithValue)) {
                  return this.not;
               } else {
                  return !this.not;
               }
            }
         }
      }
   }

   static class CombineSegement implements JSONPath.Segement {
      public final JSONPath.Segement left;
      public final JSONPath.Segement right;
      public final JSONPath.CombineSegement.Operator operator;

      public CombineSegement(JSONPath.Segement left, JSONPath.Segement right, JSONPath.CombineSegement.Operator operator) {
         this.left = left;
         this.right = right;
         this.operator = operator;
      }

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         return null;
      }

      static enum Operator {
         And,
         Or;
      }
   }

   static class DoubleOpSegement implements JSONPath.Filter {
      private final String propertyName;
      private final double value;
      private final JSONPath.Operator op;
      private final long propertyNameHash;

      public DoubleOpSegement(String propertyName, double value, JSONPath.Operator op) {
         this.propertyName = propertyName;
         this.value = value;
         this.op = op;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (propertyValue == null) {
            return false;
         } else if (!(propertyValue instanceof Number)) {
            return false;
         } else {
            double doubleValue = ((Number)propertyValue).doubleValue();
            if (this.op == JSONPath.Operator.EQ) {
               return doubleValue == this.value;
            } else if (this.op == JSONPath.Operator.NE) {
               return doubleValue != this.value;
            } else if (this.op == JSONPath.Operator.GE) {
               return doubleValue >= this.value;
            } else if (this.op == JSONPath.Operator.GT) {
               return doubleValue > this.value;
            } else if (this.op == JSONPath.Operator.LE) {
               return doubleValue <= this.value;
            } else if (this.op == JSONPath.Operator.LT) {
               return doubleValue < this.value;
            } else {
               return false;
            }
         }
      }
   }

   static class IntOpSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final long value;
      private final JSONPath.Operator op;

      public IntOpSegement(String propertyName, long value, JSONPath.Operator op) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.value = value;
         this.op = op;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (propertyValue == null) {
            return false;
         } else if (!(propertyValue instanceof Number)) {
            return false;
         } else {
            long longValue = ((Number)propertyValue).longValue();
            if (this.op == JSONPath.Operator.EQ) {
               return longValue == this.value;
            } else if (this.op == JSONPath.Operator.NE) {
               return longValue != this.value;
            } else if (this.op == JSONPath.Operator.GE) {
               return longValue >= this.value;
            } else if (this.op == JSONPath.Operator.GT) {
               return longValue > this.value;
            } else if (this.op == JSONPath.Operator.LE) {
               return longValue <= this.value;
            } else if (this.op == JSONPath.Operator.LT) {
               return longValue < this.value;
            } else {
               return false;
            }
         }
      }
   }

   static class StringInSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final String[] values;
      private final boolean not;

      public StringInSegement(String propertyName, String[] values, boolean not) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.values = values;
         this.not = not;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         String[] var6 = this.values;
         int var7 = var6.length;

         for(int var8 = 0; var8 < var7; ++var8) {
            String value = var6[var8];
            if (value == propertyValue) {
               return !this.not;
            }

            if (value != null && value.equals(propertyValue)) {
               return !this.not;
            }
         }

         return this.not;
      }
   }

   static class IntObjInSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final Long[] values;
      private final boolean not;

      public IntObjInSegement(String propertyName, Long[] values, boolean not) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.values = values;
         this.not = not;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (propertyValue == null) {
            Long[] var12 = this.values;
            int var7 = var12.length;

            for(int var13 = 0; var13 < var7; ++var13) {
               Long value = var12[var13];
               if (value == null) {
                  return !this.not;
               }
            }

            return this.not;
         } else {
            if (propertyValue instanceof Number) {
               long longPropertyValue = ((Number)propertyValue).longValue();
               Long[] var8 = this.values;
               int var9 = var8.length;

               for(int var10 = 0; var10 < var9; ++var10) {
                  Long value = var8[var10];
                  if (value != null && value == longPropertyValue) {
                     return !this.not;
                  }
               }
            }

            return this.not;
         }
      }
   }

   static class IntBetweenSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final long startValue;
      private final long endValue;
      private final boolean not;

      public IntBetweenSegement(String propertyName, long startValue, long endValue, boolean not) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.startValue = startValue;
         this.endValue = endValue;
         this.not = not;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (propertyValue == null) {
            return false;
         } else {
            if (propertyValue instanceof Number) {
               long longPropertyValue = ((Number)propertyValue).longValue();
               if (longPropertyValue >= this.startValue && longPropertyValue <= this.endValue) {
                  return !this.not;
               }
            }

            return this.not;
         }
      }
   }

   static class IntInSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final long[] values;
      private final boolean not;

      public IntInSegement(String propertyName, long[] values, boolean not) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.values = values;
         this.not = not;
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         if (propertyValue == null) {
            return false;
         } else {
            if (propertyValue instanceof Number) {
               long longPropertyValue = ((Number)propertyValue).longValue();
               long[] var8 = this.values;
               int var9 = var8.length;

               for(int var10 = 0; var10 < var9; ++var10) {
                  long value = var8[var10];
                  if (value == longPropertyValue) {
                     return !this.not;
                  }
               }
            }

            return this.not;
         }
      }
   }

   static class ValueSegment implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;
      private final Object value;
      private boolean eq = true;

      public ValueSegment(String propertyName, Object value, boolean eq) {
         if (value == null) {
            throw new IllegalArgumentException("value is null");
         } else {
            this.propertyName = propertyName;
            this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
            this.value = value;
            this.eq = eq;
         }
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         boolean result = this.value.equals(propertyValue);
         if (!this.eq) {
            result = !result;
         }

         return result;
      }
   }

   static class NullSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;

      public NullSegement(String propertyName) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         return propertyValue == null;
      }
   }

   static class NotNullSegement implements JSONPath.Filter {
      private final String propertyName;
      private final long propertyNameHash;

      public NotNullSegement(String propertyName) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
      }

      public boolean apply(JSONPath path, Object rootObject, Object currentObject, Object item) {
         Object propertyValue = path.getPropertyValue(item, this.propertyName, this.propertyNameHash);
         return propertyValue != null;
      }
   }

   static class RangeSegement implements JSONPath.Segement {
      private final int start;
      private final int end;
      private final int step;

      public RangeSegement(int start, int end, int step) {
         this.start = start;
         this.end = end;
         this.step = step;
      }

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         int size = JSONPath.SizeSegement.instance.eval(path, rootObject, currentObject);
         int start = this.start >= 0 ? this.start : this.start + size;
         int end = this.end >= 0 ? this.end : this.end + size;
         int array_size = (end - start) / this.step + 1;
         if (array_size == -1) {
            return null;
         } else {
            List<Object> items = new ArrayList(array_size);

            for(int i = start; i <= end && i < size; i += this.step) {
               Object item = path.getArrayItem(currentObject, i);
               items.add(item);
            }

            return items;
         }
      }
   }

   static class MultiIndexSegement implements JSONPath.Segement {
      private final int[] indexes;

      public MultiIndexSegement(int[] indexes) {
         this.indexes = indexes;
      }

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         List<Object> items = new ArrayList(this.indexes.length);

         for(int i = 0; i < this.indexes.length; ++i) {
            Object item = path.getArrayItem(currentObject, this.indexes[i]);
            items.add(item);
         }

         return items;
      }
   }

   static class ArrayAccessSegement implements JSONPath.Segement {
      private final int index;

      public ArrayAccessSegement(int index) {
         this.index = index;
      }

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         return path.getArrayItem(currentObject, this.index);
      }

      public boolean setValue(JSONPath path, Object currentObject, Object value) {
         return path.setArrayItem(path, currentObject, this.index, value);
      }

      public boolean remove(JSONPath path, Object currentObject) {
         return path.removeArrayItem(path, currentObject, this.index);
      }
   }

   static class WildCardSegement implements JSONPath.Segement {
      public static JSONPath.WildCardSegement instance = new JSONPath.WildCardSegement();

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         return path.getPropertyValues(currentObject);
      }
   }

   static class MultiPropertySegement implements JSONPath.Segement {
      private final String[] propertyNames;
      private final long[] propertyNamesHash;

      public MultiPropertySegement(String[] propertyNames) {
         this.propertyNames = propertyNames;
         this.propertyNamesHash = new long[propertyNames.length];

         for(int i = 0; i < this.propertyNamesHash.length; ++i) {
            this.propertyNamesHash[i] = TypeUtils.fnv1a_64(propertyNames[i]);
         }

      }

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         List<Object> fieldValues = new ArrayList(this.propertyNames.length);

         for(int i = 0; i < this.propertyNames.length; ++i) {
            Object fieldValue = path.getPropertyValue(currentObject, this.propertyNames[i], this.propertyNamesHash[i]);
            fieldValues.add(fieldValue);
         }

         return fieldValues;
      }
   }

   static class PropertySegement implements JSONPath.Segement {
      private final String propertyName;
      private final long propertyNameHash;
      private final boolean deep;

      public PropertySegement(String propertyName, boolean deep) {
         this.propertyName = propertyName;
         this.propertyNameHash = TypeUtils.fnv1a_64(propertyName);
         this.deep = deep;
      }

      public Object eval(JSONPath path, Object rootObject, Object currentObject) {
         if (this.deep) {
            List<Object> results = new ArrayList();
            path.deepScan(currentObject, this.propertyName, results);
            return results;
         } else {
            return path.getPropertyValue(currentObject, this.propertyName, this.propertyNameHash);
         }
      }

      public void setValue(JSONPath path, Object parent, Object value) {
         if (this.deep) {
            path.deepSet(parent, this.propertyName, this.propertyNameHash, value);
         } else {
            path.setPropertyValue(parent, this.propertyName, this.propertyNameHash, value);
         }

      }

      public boolean remove(JSONPath path, Object parent) {
         return path.removePropertyValue(parent, this.propertyName);
      }
   }

   static class SizeSegement implements JSONPath.Segement {
      public static final JSONPath.SizeSegement instance = new JSONPath.SizeSegement();

      public Integer eval(JSONPath path, Object rootObject, Object currentObject) {
         return path.evalSize(currentObject);
      }
   }

   interface Segement {
      Object eval(JSONPath var1, Object var2, Object var3);
   }

   static class JSONPathParser {
      private final String path;
      private int pos;
      private char ch;
      private int level;

      public JSONPathParser(String path) {
         this.path = path;
         this.next();
      }

      void next() {
         this.ch = this.path.charAt(this.pos++);
      }

      char getNextChar() {
         return this.path.charAt(this.pos);
      }

      boolean isEOF() {
         return this.pos >= this.path.length();
      }

      JSONPath.Segement readSegement() {
         if (this.level == 0 && this.path.length() == 1) {
            if (isDigitFirst(this.ch)) {
               int index = this.ch - 48;
               return new JSONPath.ArrayAccessSegement(index);
            }

            if (this.ch >= 'a' && this.ch <= 'z' || this.ch >= 'A' && this.ch <= 'Z') {
               return new JSONPath.PropertySegement(Character.toString(this.ch), false);
            }
         }

         while(!this.isEOF()) {
            this.skipWhitespace();
            if (this.ch != '$') {
               if (this.ch != '.' && this.ch != '/') {
                  if (this.ch == '[') {
                     return this.parseArrayAccess(true);
                  }

                  if (this.level == 0) {
                     String propertyName = this.readName();
                     return new JSONPath.PropertySegement(propertyName, false);
                  }

                  throw new JSONPathException("not support jsonpath : " + this.path);
               }

               int c0 = this.ch;
               boolean deep = false;
               this.next();
               if (c0 == '.' && this.ch == '.') {
                  this.next();
                  deep = true;
                  if (this.path.length() > this.pos + 3 && this.ch == '[' && this.path.charAt(this.pos) == '*' && this.path.charAt(this.pos + 1) == ']' && this.path.charAt(this.pos + 2) == '.') {
                     this.next();
                     this.next();
                     this.next();
                     this.next();
                  }
               }

               if (this.ch == '*') {
                  if (!this.isEOF()) {
                     this.next();
                  }

                  return JSONPath.WildCardSegement.instance;
               }

               if (isDigitFirst(this.ch)) {
                  return this.parseArrayAccess(false);
               }

               String propertyName = this.readName();
               if (this.ch == '(') {
                  this.next();
                  if (this.ch == ')') {
                     if (!this.isEOF()) {
                        this.next();
                     }

                     if (!"size".equals(propertyName) && !"length".equals(propertyName)) {
                        throw new JSONPathException("not support jsonpath : " + this.path);
                     }

                     return JSONPath.SizeSegement.instance;
                  }

                  throw new JSONPathException("not support jsonpath : " + this.path);
               }

               return new JSONPath.PropertySegement(propertyName, deep);
            }

            this.next();
         }

         return null;
      }

      public final void skipWhitespace() {
         while(this.ch <= ' ' && (this.ch == ' ' || this.ch == '\r' || this.ch == '\n' || this.ch == '\t' || this.ch == '\f' || this.ch == '\b')) {
            this.next();
         }

      }

      JSONPath.Segement parseArrayAccess(boolean acceptBracket) {
         Object object = this.parseArrayAccessFilter(acceptBracket);
         return (JSONPath.Segement)(object instanceof JSONPath.Segement ? (JSONPath.Segement)object : new JSONPath.FilterSegement((JSONPath.Filter)object));
      }

      Object parseArrayAccessFilter(boolean acceptBracket) {
         if (acceptBracket) {
            this.accept('[');
         }

         boolean predicateFlag = false;
         if (this.ch == '?') {
            this.next();
            this.accept('(');
            if (this.ch == '@') {
               this.next();
               this.accept('.');
            }

            predicateFlag = true;
         }

         String strValue;
         if (!predicateFlag && !IOUtils.firstIdentifier(this.ch)) {
            int start;
            for(start = this.pos - 1; this.ch != ']' && this.ch != '/' && !this.isEOF() && (this.ch != '.' || predicateFlag || predicateFlag); this.next()) {
               if (this.ch == '\\') {
                  this.next();
               }
            }

            int end;
            if (acceptBracket) {
               end = this.pos - 1;
            } else if (this.ch != '/' && this.ch != '.') {
               end = this.pos;
            } else {
               end = this.pos - 1;
            }

            strValue = this.path.substring(start, end);
            if (strValue.indexOf("\\.") != -1) {
               String propName = strValue.replaceAll("\\\\\\.", "\\.");
               if (propName.indexOf("\\-") != -1) {
                  propName = propName.replaceAll("\\\\-", "-");
               }

               if (predicateFlag) {
                  this.accept(')');
               }

               return new JSONPath.PropertySegement(propName, false);
            } else {
               JSONPath.Segement segment = this.buildArraySegement(strValue);
               if (acceptBracket && !this.isEOF()) {
                  this.accept(']');
               }

               return segment;
            }
         } else {
            String propertyName = this.readName();
            this.skipWhitespace();
            Object filter;
            if (predicateFlag && this.ch == ')') {
               this.next();
               filter = new JSONPath.NotNullSegement(propertyName);

               while(this.ch == ' ') {
                  this.next();
               }

               if (this.ch == '&' || this.ch == '|') {
                  filter = this.filterRest((JSONPath.Filter)filter);
               }

               if (acceptBracket) {
                  this.accept(']');
               }

               return filter;
            } else if (acceptBracket && this.ch == ']') {
               this.next();
               filter = new JSONPath.NotNullSegement(propertyName);

               while(this.ch == ' ') {
                  this.next();
               }

               if (this.ch == '&' || this.ch == '|') {
                  filter = this.filterRest((JSONPath.Filter)filter);
               }

               this.accept(')');
               if (predicateFlag) {
                  this.accept(')');
               }

               if (acceptBracket) {
                  this.accept(']');
               }

               return filter;
            } else {
               JSONPath.Operator op = this.readOp();
               this.skipWhitespace();
               boolean not;
               Object filter;
               if (op != JSONPath.Operator.BETWEEN && op != JSONPath.Operator.NOT_BETWEEN) {
                  boolean not;
                  String startsWithValue;
                  if (op != JSONPath.Operator.IN && op != JSONPath.Operator.NOT_IN) {
                     if (this.ch != '\'' && this.ch != '"') {
                        if (isDigitFirst(this.ch)) {
                           long value = this.readLongValue();
                           double doubleValue = 0.0D;
                           if (this.ch == '.') {
                              doubleValue = this.readDoubleValue(value);
                           }

                           Object filter;
                           if (doubleValue == 0.0D) {
                              filter = new JSONPath.IntOpSegement(propertyName, value, op);
                           } else {
                              filter = new JSONPath.DoubleOpSegement(propertyName, doubleValue, op);
                           }

                           while(this.ch == ' ') {
                              this.next();
                           }

                           if (this.ch == '&' || this.ch == '|') {
                              filter = this.filterRest((JSONPath.Filter)filter);
                           }

                           if (predicateFlag) {
                              this.accept(')');
                           }

                           if (acceptBracket) {
                              this.accept(']');
                           }

                           return doubleValue == 0.0D ? filter : filter;
                        } else {
                           if (this.ch == 'n') {
                              strValue = this.readName();
                              if ("null".equals(strValue)) {
                                 filter = null;
                                 if (op == JSONPath.Operator.EQ) {
                                    filter = new JSONPath.NullSegement(propertyName);
                                 } else if (op == JSONPath.Operator.NE) {
                                    filter = new JSONPath.NotNullSegement(propertyName);
                                 }

                                 if (filter != null) {
                                    while(this.ch == ' ') {
                                       this.next();
                                    }

                                    if (this.ch == '&' || this.ch == '|') {
                                       filter = this.filterRest((JSONPath.Filter)filter);
                                    }
                                 }

                                 if (predicateFlag) {
                                    this.accept(')');
                                 }

                                 this.accept(']');
                                 if (filter != null) {
                                    return filter;
                                 }

                                 throw new UnsupportedOperationException();
                              }
                           } else if (this.ch == 't') {
                              strValue = this.readName();
                              if ("true".equals(strValue)) {
                                 filter = null;
                                 if (op == JSONPath.Operator.EQ) {
                                    filter = new JSONPath.ValueSegment(propertyName, Boolean.TRUE, true);
                                 } else if (op == JSONPath.Operator.NE) {
                                    filter = new JSONPath.ValueSegment(propertyName, Boolean.TRUE, false);
                                 }

                                 if (filter != null) {
                                    while(this.ch == ' ') {
                                       this.next();
                                    }

                                    if (this.ch == '&' || this.ch == '|') {
                                       filter = this.filterRest((JSONPath.Filter)filter);
                                    }
                                 }

                                 if (predicateFlag) {
                                    this.accept(')');
                                 }

                                 this.accept(']');
                                 if (filter != null) {
                                    return filter;
                                 }

                                 throw new UnsupportedOperationException();
                              }
                           } else if (this.ch == 'f') {
                              strValue = this.readName();
                              if ("false".equals(strValue)) {
                                 filter = null;
                                 if (op == JSONPath.Operator.EQ) {
                                    filter = new JSONPath.ValueSegment(propertyName, Boolean.FALSE, true);
                                 } else if (op == JSONPath.Operator.NE) {
                                    filter = new JSONPath.ValueSegment(propertyName, Boolean.FALSE, false);
                                 }

                                 if (filter != null) {
                                    while(this.ch == ' ') {
                                       this.next();
                                    }

                                    if (this.ch == '&' || this.ch == '|') {
                                       filter = this.filterRest((JSONPath.Filter)filter);
                                    }
                                 }

                                 if (predicateFlag) {
                                    this.accept(')');
                                 }

                                 this.accept(']');
                                 if (filter != null) {
                                    return filter;
                                 }

                                 throw new UnsupportedOperationException();
                              }
                           }

                           throw new UnsupportedOperationException();
                        }
                     } else {
                        strValue = this.readString();
                        filter = null;
                        if (op == JSONPath.Operator.RLIKE) {
                           filter = new JSONPath.RlikeSegement(propertyName, strValue, false);
                        } else if (op == JSONPath.Operator.NOT_RLIKE) {
                           filter = new JSONPath.RlikeSegement(propertyName, strValue, true);
                        } else if (op != JSONPath.Operator.LIKE && op != JSONPath.Operator.NOT_LIKE) {
                           filter = new JSONPath.StringOpSegement(propertyName, strValue, op);
                        } else {
                           while(strValue.indexOf("%%") != -1) {
                              strValue = strValue.replaceAll("%%", "%");
                           }

                           not = op == JSONPath.Operator.NOT_LIKE;
                           int p0 = strValue.indexOf(37);
                           if (p0 == -1) {
                              if (op == JSONPath.Operator.LIKE) {
                                 op = JSONPath.Operator.EQ;
                              } else {
                                 op = JSONPath.Operator.NE;
                              }

                              filter = new JSONPath.StringOpSegement(propertyName, strValue, op);
                           } else {
                              String[] items = strValue.split("%");
                              startsWithValue = null;
                              String endsWithValue = null;
                              String[] containsValues = null;
                              if (p0 == 0) {
                                 if (strValue.charAt(strValue.length() - 1) == '%') {
                                    containsValues = new String[items.length - 1];
                                    System.arraycopy(items, 1, containsValues, 0, containsValues.length);
                                 } else {
                                    endsWithValue = items[items.length - 1];
                                    if (items.length > 2) {
                                       containsValues = new String[items.length - 2];
                                       System.arraycopy(items, 1, containsValues, 0, containsValues.length);
                                    }
                                 }
                              } else if (strValue.charAt(strValue.length() - 1) == '%') {
                                 containsValues = items;
                              } else if (items.length == 1) {
                                 startsWithValue = items[0];
                              } else if (items.length == 2) {
                                 startsWithValue = items[0];
                                 endsWithValue = items[1];
                              } else {
                                 startsWithValue = items[0];
                                 endsWithValue = items[items.length - 1];
                                 containsValues = new String[items.length - 2];
                                 System.arraycopy(items, 1, containsValues, 0, containsValues.length);
                              }

                              filter = new JSONPath.MatchSegement(propertyName, startsWithValue, endsWithValue, containsValues, not);
                           }
                        }

                        while(this.ch == ' ') {
                           this.next();
                        }

                        if (this.ch == '&' || this.ch == '|') {
                           filter = this.filterRest((JSONPath.Filter)filter);
                        }

                        if (predicateFlag) {
                           this.accept(')');
                        }

                        if (acceptBracket) {
                           this.accept(']');
                        }

                        return filter;
                     }
                  } else {
                     not = op == JSONPath.Operator.NOT_IN;
                     this.accept('(');
                     List<Object> valueList = new JSONArray();
                     Object value = this.readValue();
                     valueList.add(value);

                     while(true) {
                        this.skipWhitespace();
                        if (this.ch != ',') {
                           not = true;
                           boolean isIntObj = true;
                           boolean isString = true;
                           Iterator var10 = valueList.iterator();

                           Object filter;
                           while(var10.hasNext()) {
                              filter = var10.next();
                              if (filter == null) {
                                 if (not) {
                                    not = false;
                                 }
                              } else {
                                 Class<?> clazz = filter.getClass();
                                 if (not && clazz != Byte.class && clazz != Short.class && clazz != Integer.class && clazz != Long.class) {
                                    not = false;
                                    isIntObj = false;
                                 }

                                 if (isString && clazz != String.class) {
                                    isString = false;
                                 }
                              }
                           }

                           if (valueList.size() == 1 && valueList.get(0) == null) {
                              Object filter;
                              if (not) {
                                 filter = new JSONPath.NotNullSegement(propertyName);
                              } else {
                                 filter = new JSONPath.NullSegement(propertyName);
                              }

                              while(this.ch == ' ') {
                                 this.next();
                              }

                              if (this.ch == '&' || this.ch == '|') {
                                 filter = this.filterRest((JSONPath.Filter)filter);
                              }

                              this.accept(')');
                              if (predicateFlag) {
                                 this.accept(')');
                              }

                              if (acceptBracket) {
                                 this.accept(']');
                              }

                              return filter;
                           } else {
                              int i;
                              if (not) {
                                 if (valueList.size() == 1) {
                                    long value = ((Number)valueList.get(0)).longValue();
                                    JSONPath.Operator intOp = not ? JSONPath.Operator.NE : JSONPath.Operator.EQ;
                                    Object filter = new JSONPath.IntOpSegement(propertyName, value, intOp);

                                    while(this.ch == ' ') {
                                       this.next();
                                    }

                                    if (this.ch == '&' || this.ch == '|') {
                                       filter = this.filterRest((JSONPath.Filter)filter);
                                    }

                                    this.accept(')');
                                    if (predicateFlag) {
                                       this.accept(')');
                                    }

                                    if (acceptBracket) {
                                       this.accept(']');
                                    }

                                    return filter;
                                 } else {
                                    long[] values = new long[valueList.size()];

                                    for(i = 0; i < values.length; ++i) {
                                       values[i] = ((Number)valueList.get(i)).longValue();
                                    }

                                    filter = new JSONPath.IntInSegement(propertyName, values, not);

                                    while(this.ch == ' ') {
                                       this.next();
                                    }

                                    if (this.ch == '&' || this.ch == '|') {
                                       filter = this.filterRest((JSONPath.Filter)filter);
                                    }

                                    this.accept(')');
                                    if (predicateFlag) {
                                       this.accept(')');
                                    }

                                    if (acceptBracket) {
                                       this.accept(']');
                                    }

                                    return filter;
                                 }
                              } else if (isString) {
                                 if (valueList.size() == 1) {
                                    startsWithValue = (String)valueList.get(0);
                                    JSONPath.Operator intOp = not ? JSONPath.Operator.NE : JSONPath.Operator.EQ;
                                    Object filter = new JSONPath.StringOpSegement(propertyName, startsWithValue, intOp);

                                    while(this.ch == ' ') {
                                       this.next();
                                    }

                                    if (this.ch == '&' || this.ch == '|') {
                                       filter = this.filterRest((JSONPath.Filter)filter);
                                    }

                                    this.accept(')');
                                    if (predicateFlag) {
                                       this.accept(')');
                                    }

                                    if (acceptBracket) {
                                       this.accept(']');
                                    }

                                    return filter;
                                 } else {
                                    String[] values = new String[valueList.size()];
                                    valueList.toArray(values);
                                    filter = new JSONPath.StringInSegement(propertyName, values, not);

                                    while(this.ch == ' ') {
                                       this.next();
                                    }

                                    if (this.ch == '&' || this.ch == '|') {
                                       filter = this.filterRest((JSONPath.Filter)filter);
                                    }

                                    this.accept(')');
                                    if (predicateFlag) {
                                       this.accept(')');
                                    }

                                    if (acceptBracket) {
                                       this.accept(']');
                                    }

                                    return filter;
                                 }
                              } else if (!isIntObj) {
                                 throw new UnsupportedOperationException();
                              } else {
                                 Long[] values = new Long[valueList.size()];

                                 for(i = 0; i < values.length; ++i) {
                                    Number item = (Number)valueList.get(i);
                                    if (item != null) {
                                       values[i] = item.longValue();
                                    }
                                 }

                                 filter = new JSONPath.IntObjInSegement(propertyName, values, not);

                                 while(this.ch == ' ') {
                                    this.next();
                                 }

                                 if (this.ch == '&' || this.ch == '|') {
                                    filter = this.filterRest((JSONPath.Filter)filter);
                                 }

                                 this.accept(')');
                                 if (predicateFlag) {
                                    this.accept(')');
                                 }

                                 if (acceptBracket) {
                                    this.accept(']');
                                 }

                                 return filter;
                              }
                           }
                        }

                        this.next();
                        value = this.readValue();
                        valueList.add(value);
                     }
                  }
               } else {
                  not = op == JSONPath.Operator.NOT_BETWEEN;
                  filter = this.readValue();
                  String name = this.readName();
                  if (!"and".equalsIgnoreCase(name)) {
                     throw new JSONPathException(this.path);
                  } else {
                     Object endValue = this.readValue();
                     if (filter != null && endValue != null) {
                        if (JSONPath.isInt(filter.getClass()) && JSONPath.isInt(endValue.getClass())) {
                           JSONPath.Filter filter = new JSONPath.IntBetweenSegement(propertyName, ((Number)filter).longValue(), ((Number)endValue).longValue(), not);
                           return filter;
                        } else {
                           throw new JSONPathException(this.path);
                        }
                     } else {
                        throw new JSONPathException(this.path);
                     }
                  }
               }
            }
         }
      }

      JSONPath.Filter filterRest(JSONPath.Filter filter) {
         boolean and = this.ch == '&';
         if (this.ch == '&' && this.getNextChar() == '&' || this.ch == '|' && this.getNextChar() == '|') {
            this.next();
            this.next();

            while(this.ch == ' ') {
               this.next();
            }

            JSONPath.Filter right = (JSONPath.Filter)this.parseArrayAccessFilter(false);
            filter = new JSONPath.FilterGroup((JSONPath.Filter)filter, right, and);
         }

         return (JSONPath.Filter)filter;
      }

      protected long readLongValue() {
         int beginIndex = this.pos - 1;
         if (this.ch == '+' || this.ch == '-') {
            this.next();
         }

         while(this.ch >= '0' && this.ch <= '9') {
            this.next();
         }

         int endIndex = this.pos - 1;
         String text = this.path.substring(beginIndex, endIndex);
         long value = Long.parseLong(text);
         return value;
      }

      protected double readDoubleValue(long longValue) {
         int beginIndex = this.pos - 1;
         this.next();

         while(this.ch >= '0' && this.ch <= '9') {
            this.next();
         }

         int endIndex = this.pos - 1;
         String text = this.path.substring(beginIndex, endIndex);
         double value = Double.parseDouble(text);
         value += (double)longValue;
         return value;
      }

      protected Object readValue() {
         this.skipWhitespace();
         if (isDigitFirst(this.ch)) {
            return this.readLongValue();
         } else if (this.ch != '"' && this.ch != '\'') {
            if (this.ch == 'n') {
               String name = this.readName();
               if ("null".equals(name)) {
                  return null;
               } else {
                  throw new JSONPathException(this.path);
               }
            } else {
               throw new UnsupportedOperationException();
            }
         } else {
            return this.readString();
         }
      }

      static boolean isDigitFirst(char ch) {
         return ch == '-' || ch == '+' || ch >= '0' && ch <= '9';
      }

      protected JSONPath.Operator readOp() {
         JSONPath.Operator op = null;
         if (this.ch == '=') {
            this.next();
            op = JSONPath.Operator.EQ;
         } else if (this.ch == '!') {
            this.next();
            this.accept('=');
            op = JSONPath.Operator.NE;
         } else if (this.ch == '<') {
            this.next();
            if (this.ch == '=') {
               this.next();
               op = JSONPath.Operator.LE;
            } else {
               op = JSONPath.Operator.LT;
            }
         } else if (this.ch == '>') {
            this.next();
            if (this.ch == '=') {
               this.next();
               op = JSONPath.Operator.GE;
            } else {
               op = JSONPath.Operator.GT;
            }
         }

         if (op == null) {
            String name = this.readName();
            if ("not".equalsIgnoreCase(name)) {
               this.skipWhitespace();
               name = this.readName();
               if ("like".equalsIgnoreCase(name)) {
                  op = JSONPath.Operator.NOT_LIKE;
               } else if ("rlike".equalsIgnoreCase(name)) {
                  op = JSONPath.Operator.NOT_RLIKE;
               } else if ("in".equalsIgnoreCase(name)) {
                  op = JSONPath.Operator.NOT_IN;
               } else {
                  if (!"between".equalsIgnoreCase(name)) {
                     throw new UnsupportedOperationException();
                  }

                  op = JSONPath.Operator.NOT_BETWEEN;
               }
            } else if ("like".equalsIgnoreCase(name)) {
               op = JSONPath.Operator.LIKE;
            } else if ("rlike".equalsIgnoreCase(name)) {
               op = JSONPath.Operator.RLIKE;
            } else if ("in".equalsIgnoreCase(name)) {
               op = JSONPath.Operator.IN;
            } else {
               if (!"between".equalsIgnoreCase(name)) {
                  throw new UnsupportedOperationException();
               }

               op = JSONPath.Operator.BETWEEN;
            }
         }

         return op;
      }

      String readName() {
         this.skipWhitespace();
         if (this.ch != '\\' && !IOUtils.firstIdentifier(this.ch)) {
            throw new JSONPathException("illeal jsonpath syntax. " + this.path);
         } else {
            StringBuilder buf = new StringBuilder();

            while(!this.isEOF()) {
               if (this.ch == '\\') {
                  this.next();
                  buf.append(this.ch);
                  if (this.isEOF()) {
                     break;
                  }

                  this.next();
               } else {
                  boolean identifierFlag = IOUtils.isIdent(this.ch);
                  if (!identifierFlag) {
                     break;
                  }

                  buf.append(this.ch);
                  this.next();
               }
            }

            if (this.isEOF() && IOUtils.isIdent(this.ch)) {
               buf.append(this.ch);
            }

            String propertyName = buf.toString();
            return propertyName;
         }
      }

      String readString() {
         char quoate = this.ch;
         this.next();
         int beginIndex = this.pos - 1;

         while(this.ch != quoate && !this.isEOF()) {
            this.next();
         }

         String strValue = this.path.substring(beginIndex, this.isEOF() ? this.pos : this.pos - 1);
         this.accept(quoate);
         return strValue;
      }

      void accept(char expect) {
         if (this.ch != expect) {
            throw new JSONPathException("expect '" + expect + ", but '" + this.ch + "'");
         } else {
            if (!this.isEOF()) {
               this.next();
            }

         }
      }

      public JSONPath.Segement[] explain() {
         if (this.path != null && this.path.length() != 0) {
            JSONPath.Segement[] segements = new JSONPath.Segement[8];

            while(true) {
               JSONPath.Segement segment = this.readSegement();
               if (segment == null) {
                  if (this.level == segements.length) {
                     return segements;
                  }

                  JSONPath.Segement[] result = new JSONPath.Segement[this.level];
                  System.arraycopy(segements, 0, result, 0, this.level);
                  return result;
               }

               if (this.level == segements.length) {
                  JSONPath.Segement[] t = new JSONPath.Segement[this.level * 3 / 2];
                  System.arraycopy(segements, 0, t, 0, this.level);
                  segements = t;
               }

               segements[this.level++] = segment;
            }
         } else {
            throw new IllegalArgumentException();
         }
      }

      JSONPath.Segement buildArraySegement(String indexText) {
         int indexTextLen = indexText.length();
         char firstChar = indexText.charAt(0);
         char lastChar = indexText.charAt(indexTextLen - 1);
         int commaIndex = indexText.indexOf(44);
         String[] indexesText;
         if (indexText.length() > 2 && firstChar == '\'' && lastChar == '\'') {
            if (commaIndex == -1) {
               String propertyName = indexText.substring(1, indexTextLen - 1);
               return new JSONPath.PropertySegement(propertyName, false);
            } else {
               String[] indexesText = indexText.split(",");
               indexesText = new String[indexesText.length];

               for(int i = 0; i < indexesText.length; ++i) {
                  String indexesTextItem = indexesText[i];
                  indexesText[i] = indexesTextItem.substring(1, indexesTextItem.length() - 1);
               }

               return new JSONPath.MultiPropertySegement(indexesText);
            }
         } else {
            int colonIndex = indexText.indexOf(58);
            if (commaIndex == -1 && colonIndex == -1) {
               if (TypeUtils.isNumber(indexText)) {
                  try {
                     int index = Integer.parseInt(indexText);
                     return new JSONPath.ArrayAccessSegement(index);
                  } catch (NumberFormatException var12) {
                     return new JSONPath.PropertySegement(indexText, false);
                  }
               } else {
                  return new JSONPath.PropertySegement(indexText, false);
               }
            } else {
               int[] indexes;
               int start;
               if (commaIndex == -1) {
                  if (colonIndex != -1) {
                     indexesText = indexText.split(":");
                     indexes = new int[indexesText.length];

                     for(start = 0; start < indexesText.length; ++start) {
                        String str = indexesText[start];
                        if (str.length() == 0) {
                           if (start != 0) {
                              throw new UnsupportedOperationException();
                           }

                           indexes[start] = 0;
                        } else {
                           indexes[start] = Integer.parseInt(str);
                        }
                     }

                     start = indexes[0];
                     int end;
                     if (indexes.length > 1) {
                        end = indexes[1];
                     } else {
                        end = -1;
                     }

                     int step;
                     if (indexes.length == 3) {
                        step = indexes[2];
                     } else {
                        step = 1;
                     }

                     if (end >= 0 && end < start) {
                        throw new UnsupportedOperationException("end must greater than or equals start. start " + start + ",  end " + end);
                     } else if (step <= 0) {
                        throw new UnsupportedOperationException("step must greater than zero : " + step);
                     } else {
                        return new JSONPath.RangeSegement(start, end, step);
                     }
                  } else {
                     throw new UnsupportedOperationException();
                  }
               } else {
                  indexesText = indexText.split(",");
                  indexes = new int[indexesText.length];

                  for(start = 0; start < indexesText.length; ++start) {
                     indexes[start] = Integer.parseInt(indexesText[start]);
                  }

                  return new JSONPath.MultiIndexSegement(indexes);
               }
            }
         }
      }
   }
}
