package com.alibaba.fastjson.parser;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.util.IOUtils;
import java.io.CharArrayReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.math.BigDecimal;

public final class JSONReaderScanner extends JSONLexerBase {
   private static final ThreadLocal<char[]> BUF_LOCAL = new ThreadLocal();
   private Reader reader;
   private char[] buf;
   private int bufLength;

   public JSONReaderScanner(String input) {
      this(input, JSON.DEFAULT_PARSER_FEATURE);
   }

   public JSONReaderScanner(String input, int features) {
      this((Reader)(new StringReader(input)), features);
   }

   public JSONReaderScanner(char[] input, int inputLength) {
      this(input, inputLength, JSON.DEFAULT_PARSER_FEATURE);
   }

   public JSONReaderScanner(Reader reader) {
      this(reader, JSON.DEFAULT_PARSER_FEATURE);
   }

   public JSONReaderScanner(Reader reader, int features) {
      super(features);
      this.reader = reader;
      this.buf = (char[])BUF_LOCAL.get();
      if (this.buf != null) {
         BUF_LOCAL.set((Object)null);
      }

      if (this.buf == null) {
         this.buf = new char[16384];
      }

      try {
         this.bufLength = reader.read(this.buf);
      } catch (IOException var4) {
         throw new JSONException(var4.getMessage(), var4);
      }

      this.bp = -1;
      this.next();
      if (this.ch == '\ufeff') {
         this.next();
      }

   }

   public JSONReaderScanner(char[] input, int inputLength, int features) {
      this((Reader)(new CharArrayReader(input, 0, inputLength)), features);
   }

   public final char charAt(int index) {
      if (index >= this.bufLength) {
         if (this.bufLength == -1) {
            if (index < this.sp) {
               return this.buf[index];
            }

            return '\u001a';
         }

         if (this.bp == 0) {
            char[] buf = new char[this.buf.length * 3 / 2];
            System.arraycopy(this.buf, this.bp, buf, 0, this.bufLength);
            int rest = buf.length - this.bufLength;

            try {
               int len = this.reader.read(buf, this.bufLength, rest);
               this.bufLength += len;
               this.buf = buf;
            } catch (IOException var6) {
               throw new JSONException(var6.getMessage(), var6);
            }
         } else {
            int rest = this.bufLength - this.bp;
            if (rest > 0) {
               System.arraycopy(this.buf, this.bp, this.buf, 0, rest);
            }

            try {
               this.bufLength = this.reader.read(this.buf, rest, this.buf.length - rest);
            } catch (IOException var5) {
               throw new JSONException(var5.getMessage(), var5);
            }

            if (this.bufLength == 0) {
               throw new JSONException("illegal state, textLength is zero");
            }

            if (this.bufLength == -1) {
               return '\u001a';
            }

            this.bufLength += rest;
            index -= this.bp;
            this.np -= this.bp;
            this.bp = 0;
         }
      }

      return this.buf[index];
   }

   public final int indexOf(char ch, int startIndex) {
      int offset = startIndex - this.bp;

      while(true) {
         int index = this.bp + offset;
         char chLoal = this.charAt(index);
         if (ch == chLoal) {
            return offset + this.bp;
         }

         if (chLoal == 26) {
            return -1;
         }

         ++offset;
      }
   }

   public final String addSymbol(int offset, int len, int hash, SymbolTable symbolTable) {
      return symbolTable.addSymbol(this.buf, offset, len, hash);
   }

   public final char next() {
      int index = ++this.bp;
      if (index >= this.bufLength) {
         if (this.bufLength == -1) {
            return '\u001a';
         }

         int startPos;
         if (this.sp > 0) {
            startPos = this.bufLength - this.sp;
            if (this.ch == '"' && startPos > 0) {
               --startPos;
            }

            System.arraycopy(this.buf, startPos, this.buf, 0, this.sp);
         }

         this.np = -1;
         index = this.bp = this.sp;

         try {
            startPos = this.bp;
            int readLength = this.buf.length - startPos;
            if (readLength == 0) {
               char[] newBuf = new char[this.buf.length * 2];
               System.arraycopy(this.buf, 0, newBuf, 0, this.buf.length);
               this.buf = newBuf;
               readLength = this.buf.length - startPos;
            }

            this.bufLength = this.reader.read(this.buf, this.bp, readLength);
         } catch (IOException var5) {
            throw new JSONException(var5.getMessage(), var5);
         }

         if (this.bufLength == 0) {
            throw new JSONException("illegal stat, textLength is zero");
         }

         if (this.bufLength == -1) {
            return this.ch = '\u001a';
         }

         this.bufLength += this.bp;
      }

      return this.ch = this.buf[index];
   }

   protected final void copyTo(int offset, int count, char[] dest) {
      System.arraycopy(this.buf, offset, dest, 0, count);
   }

   public final boolean charArrayCompare(char[] chars) {
      for(int i = 0; i < chars.length; ++i) {
         if (this.charAt(this.bp + i) != chars[i]) {
            return false;
         }
      }

      return true;
   }

   public byte[] bytesValue() {
      if (this.token == 26) {
         throw new JSONException("TODO");
      } else {
         return IOUtils.decodeBase64(this.buf, this.np + 1, this.sp);
      }
   }

   protected final void arrayCopy(int srcPos, char[] dest, int destPos, int length) {
      System.arraycopy(this.buf, srcPos, dest, destPos, length);
   }

   public final String stringVal() {
      if (!this.hasSpecial) {
         int offset = this.np + 1;
         if (offset < 0) {
            throw new IllegalStateException();
         } else if (offset > this.buf.length - this.sp) {
            throw new IllegalStateException();
         } else {
            return new String(this.buf, offset, this.sp);
         }
      } else {
         return new String(this.sbuf, 0, this.sp);
      }
   }

   public final String subString(int offset, int count) {
      if (count < 0) {
         throw new StringIndexOutOfBoundsException(count);
      } else {
         return new String(this.buf, offset, count);
      }
   }

   public final char[] sub_chars(int offset, int count) {
      if (count < 0) {
         throw new StringIndexOutOfBoundsException(count);
      } else if (offset == 0) {
         return this.buf;
      } else {
         char[] chars = new char[count];
         System.arraycopy(this.buf, offset, chars, 0, count);
         return chars;
      }
   }

   public final String numberString() {
      int offset = this.np;
      if (offset == -1) {
         offset = 0;
      }

      char chLocal = this.charAt(offset + this.sp - 1);
      int sp = this.sp;
      if (chLocal == 'L' || chLocal == 'S' || chLocal == 'B' || chLocal == 'F' || chLocal == 'D') {
         --sp;
      }

      String value = new String(this.buf, offset, sp);
      return value;
   }

   public final BigDecimal decimalValue() {
      int offset = this.np;
      if (offset == -1) {
         offset = 0;
      }

      char chLocal = this.charAt(offset + this.sp - 1);
      int sp = this.sp;
      if (chLocal == 'L' || chLocal == 'S' || chLocal == 'B' || chLocal == 'F' || chLocal == 'D') {
         --sp;
      }

      return new BigDecimal(this.buf, offset, sp);
   }

   public void close() {
      super.close();
      if (this.buf.length <= 65536) {
         BUF_LOCAL.set(this.buf);
      }

      this.buf = null;
      IOUtils.close(this.reader);
   }

   public boolean isEOF() {
      return this.bufLength == -1 || this.bp == this.buf.length || this.ch == 26 && this.bp + 1 == this.buf.length;
   }

   public final boolean isBlankInput() {
      int i = 0;

      while(true) {
         char chLocal = this.buf[i];
         if (chLocal == 26) {
            this.token = 20;
            return true;
         }

         if (!isWhitespace(chLocal)) {
            return false;
         }

         ++i;
      }
   }
}
