package com.alibaba.fastjson.parser;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.util.ASMUtils;
import com.alibaba.fastjson.util.IOUtils;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.TimeZone;

public final class JSONScanner extends JSONLexerBase {
   private final String text;
   private final int len;

   public JSONScanner(String input) {
      this(input, JSON.DEFAULT_PARSER_FEATURE);
   }

   public JSONScanner(String input, int features) {
      super(features);
      this.text = input;
      this.len = this.text.length();
      this.bp = -1;
      this.next();
      if (this.ch == '\ufeff') {
         this.next();
      }

   }

   public final char charAt(int index) {
      return index >= this.len ? '\u001a' : this.text.charAt(index);
   }

   public final char next() {
      int index = ++this.bp;
      return this.ch = index >= this.len ? '\u001a' : this.text.charAt(index);
   }

   public JSONScanner(char[] input, int inputLength) {
      this(input, inputLength, JSON.DEFAULT_PARSER_FEATURE);
   }

   public JSONScanner(char[] input, int inputLength, int features) {
      this(new String(input, 0, inputLength), features);
   }

   protected final void copyTo(int offset, int count, char[] dest) {
      this.text.getChars(offset, offset + count, dest, 0);
   }

   static boolean charArrayCompare(String src, int offset, char[] dest) {
      int destLen = dest.length;
      if (destLen + offset > src.length()) {
         return false;
      } else {
         for(int i = 0; i < destLen; ++i) {
            if (dest[i] != src.charAt(offset + i)) {
               return false;
            }
         }

         return true;
      }
   }

   public final boolean charArrayCompare(char[] chars) {
      return charArrayCompare(this.text, this.bp, chars);
   }

   public final int indexOf(char ch, int startIndex) {
      return this.text.indexOf(ch, startIndex);
   }

   public final String addSymbol(int offset, int len, int hash, SymbolTable symbolTable) {
      return symbolTable.addSymbol(this.text, offset, len, hash);
   }

   public byte[] bytesValue() {
      if (this.token == 26) {
         int start = this.np + 1;
         int len = this.sp;
         if (len % 2 != 0) {
            throw new JSONException("illegal state. " + len);
         } else {
            byte[] bytes = new byte[len / 2];

            for(int i = 0; i < bytes.length; ++i) {
               char c0 = this.text.charAt(start + i * 2);
               char c1 = this.text.charAt(start + i * 2 + 1);
               int b0 = c0 - (c0 <= '9' ? 48 : 55);
               int b1 = c1 - (c1 <= '9' ? 48 : 55);
               bytes[i] = (byte)(b0 << 4 | b1);
            }

            return bytes;
         }
      } else {
         return IOUtils.decodeBase64(this.text, this.np + 1, this.sp);
      }
   }

   public final String stringVal() {
      return !this.hasSpecial ? this.subString(this.np + 1, this.sp) : new String(this.sbuf, 0, this.sp);
   }

   public final String subString(int offset, int count) {
      if (ASMUtils.IS_ANDROID) {
         if (count < this.sbuf.length) {
            this.text.getChars(offset, offset + count, this.sbuf, 0);
            return new String(this.sbuf, 0, count);
         } else {
            char[] chars = new char[count];
            this.text.getChars(offset, offset + count, chars, 0);
            return new String(chars);
         }
      } else {
         return this.text.substring(offset, offset + count);
      }
   }

   public final char[] sub_chars(int offset, int count) {
      if (ASMUtils.IS_ANDROID && count < this.sbuf.length) {
         this.text.getChars(offset, offset + count, this.sbuf, 0);
         return this.sbuf;
      } else {
         char[] chars = new char[count];
         this.text.getChars(offset, offset + count, chars, 0);
         return chars;
      }
   }

   public final String numberString() {
      char chLocal = this.charAt(this.np + this.sp - 1);
      int sp = this.sp;
      if (chLocal == 'L' || chLocal == 'S' || chLocal == 'B' || chLocal == 'F' || chLocal == 'D') {
         --sp;
      }

      return this.subString(this.np, sp);
   }

   public final BigDecimal decimalValue() {
      char chLocal = this.charAt(this.np + this.sp - 1);
      int sp = this.sp;
      if (chLocal == 'L' || chLocal == 'S' || chLocal == 'B' || chLocal == 'F' || chLocal == 'D') {
         --sp;
      }

      int offset = this.np;
      if (sp < this.sbuf.length) {
         this.text.getChars(offset, offset + sp, this.sbuf, 0);
         return new BigDecimal(this.sbuf, 0, sp);
      } else {
         char[] chars = new char[sp];
         this.text.getChars(offset, offset + sp, chars, 0);
         return new BigDecimal(chars);
      }
   }

   public boolean scanISO8601DateIfMatch() {
      return this.scanISO8601DateIfMatch(true);
   }

   public boolean scanISO8601DateIfMatch(boolean strict) {
      int rest = this.len - this.bp;
      return this.scanISO8601DateIfMatch(strict, rest);
   }

   private boolean scanISO8601DateIfMatch(boolean strict, int rest) {
      if (rest < 8) {
         return false;
      } else {
         char c0 = this.charAt(this.bp);
         char c1 = this.charAt(this.bp + 1);
         char c2 = this.charAt(this.bp + 2);
         char c3 = this.charAt(this.bp + 3);
         char c4 = this.charAt(this.bp + 4);
         char c5 = this.charAt(this.bp + 5);
         char c6 = this.charAt(this.bp + 6);
         char c7 = this.charAt(this.bp + 7);
         char c10;
         char c8;
         char y0;
         if (!strict && rest > 13) {
            c10 = this.charAt(this.bp + rest - 1);
            c8 = this.charAt(this.bp + rest - 2);
            if (c0 == '/' && c1 == 'D' && c2 == 'a' && c3 == 't' && c4 == 'e' && c5 == '(' && c10 == '/' && c8 == ')') {
               int plusIndex = -1;

               int offset;
               for(offset = 6; offset < rest; ++offset) {
                  y0 = this.charAt(this.bp + offset);
                  if (y0 == '+') {
                     plusIndex = offset;
                  } else if (y0 < '0' || y0 > '9') {
                     break;
                  }
               }

               if (plusIndex == -1) {
                  return false;
               }

               offset = this.bp + 6;
               String numberText = this.subString(offset, this.bp + plusIndex - offset);
               long millis = Long.parseLong(numberText);
               this.calendar = Calendar.getInstance(this.timeZone, this.locale);
               this.calendar.setTimeInMillis(millis);
               this.token = 5;
               return true;
            }
         }

         char c9;
         char y1;
         char y2;
         char y3;
         char M0;
         char M1;
         char s0;
         char s1;
         char dot;
         char S2;
         char timeZoneFlag;
         char t0;
         char t1;
         char t2;
         char t3;
         char t4;
         char S0;
         if (rest == 8 || rest == 14 || rest == 16 && ((c10 = this.charAt(this.bp + 10)) == 'T' || c10 == ' ') || rest == 17 && this.charAt(this.bp + 6) != '-') {
            if (strict) {
               return false;
            } else {
               M1 = this.charAt(this.bp + 8);
               boolean c_47 = c4 == '-' && c7 == '-';
               boolean sperate16 = c_47 && rest == 16;
               boolean sperate17 = c_47 && rest == 17;
               char y2;
               if (!sperate17 && !sperate16) {
                  c8 = c0;
                  c9 = c1;
                  y2 = c2;
                  y0 = c3;
                  y1 = c4;
                  y2 = c5;
                  y3 = c6;
                  M0 = c7;
               } else {
                  c8 = c0;
                  c9 = c1;
                  y2 = c2;
                  y0 = c3;
                  y1 = c5;
                  y2 = c6;
                  y3 = M1;
                  M0 = this.charAt(this.bp + 9);
               }

               if (!checkDate(c8, c9, y2, y0, y1, y2, y3, M0)) {
                  return false;
               } else {
                  this.setCalendar(c8, c9, y2, y0, y1, y2, y3, M0);
                  int hour;
                  int minute;
                  int seconds;
                  int millis;
                  if (rest == 8) {
                     hour = 0;
                     minute = 0;
                     seconds = 0;
                     millis = 0;
                  } else {
                     s0 = this.charAt(this.bp + 9);
                     c10 = this.charAt(this.bp + 10);
                     s1 = this.charAt(this.bp + 11);
                     dot = this.charAt(this.bp + 12);
                     S0 = this.charAt(this.bp + 13);
                     char h0;
                     char h1;
                     if ((!sperate17 || c10 != 'T' || S0 != ':' || this.charAt(this.bp + 16) != 'Z') && (!sperate16 || c10 != ' ' && c10 != 'T' || S0 != ':')) {
                        h0 = M1;
                        h1 = s0;
                        S2 = c10;
                        timeZoneFlag = s1;
                        t0 = dot;
                        t1 = S0;
                     } else {
                        h0 = s1;
                        h1 = dot;
                        S2 = this.charAt(this.bp + 14);
                        timeZoneFlag = this.charAt(this.bp + 15);
                        t0 = '0';
                        t1 = '0';
                     }

                     if (!this.checkTime(h0, h1, S2, timeZoneFlag, t0, t1)) {
                        return false;
                     }

                     if (rest == 17 && !sperate17) {
                        t2 = this.charAt(this.bp + 14);
                        t3 = this.charAt(this.bp + 15);
                        t4 = this.charAt(this.bp + 16);
                        if (t2 < '0' || t2 > '9') {
                           return false;
                        }

                        if (t3 < '0' || t3 > '9') {
                           return false;
                        }

                        if (t4 < '0' || t4 > '9') {
                           return false;
                        }

                        millis = (t2 - 48) * 100 + (t3 - 48) * 10 + (t4 - 48);
                     } else {
                        millis = 0;
                     }

                     hour = (h0 - 48) * 10 + (h1 - 48);
                     minute = (S2 - 48) * 10 + (timeZoneFlag - 48);
                     seconds = (t0 - 48) * 10 + (t1 - 48);
                  }

                  this.calendar.set(11, hour);
                  this.calendar.set(12, minute);
                  this.calendar.set(13, seconds);
                  this.calendar.set(14, millis);
                  this.token = 5;
                  return true;
               }
            }
         } else if (rest < 9) {
            return false;
         } else {
            c8 = this.charAt(this.bp + 8);
            c9 = this.charAt(this.bp + 9);
            int date_len = 10;
            char d0;
            char d1;
            if ((c4 != '-' || c7 != '-') && (c4 != '/' || c7 != '/')) {
               if (c4 == '-' && c6 == '-') {
                  y0 = c0;
                  y1 = c1;
                  y2 = c2;
                  y3 = c3;
                  M0 = '0';
                  M1 = c5;
                  if (c8 == ' ') {
                     d0 = '0';
                     d1 = c7;
                     date_len = 8;
                  } else {
                     d0 = c7;
                     d1 = c8;
                     date_len = 9;
                  }
               } else if ((c2 != '.' || c5 != '.') && (c2 != '-' || c5 != '-')) {
                  if (c4 != 24180 && c4 != '년') {
                     return false;
                  }

                  y0 = c0;
                  y1 = c1;
                  y2 = c2;
                  y3 = c3;
                  if (c7 != 26376 && c7 != '월') {
                     if (c6 != 26376 && c6 != '월') {
                        return false;
                     }

                     M0 = '0';
                     M1 = c5;
                     if (c8 != 26085 && c8 != '일') {
                        if (c9 != 26085 && c9 != '일') {
                           return false;
                        }

                        d0 = c7;
                        d1 = c8;
                     } else {
                        d0 = '0';
                        d1 = c7;
                     }
                  } else {
                     M0 = c5;
                     M1 = c6;
                     if (c9 != 26085 && c9 != '일') {
                        if (this.charAt(this.bp + 10) != 26085 && this.charAt(this.bp + 10) != '일') {
                           return false;
                        }

                        d0 = c8;
                        d1 = c9;
                        date_len = 11;
                     } else {
                        d0 = '0';
                        d1 = c8;
                     }
                  }
               } else {
                  d0 = c0;
                  d1 = c1;
                  M0 = c3;
                  M1 = c4;
                  y0 = c6;
                  y1 = c7;
                  y2 = c8;
                  y3 = c9;
               }
            } else {
               y0 = c0;
               y1 = c1;
               y2 = c2;
               y3 = c3;
               M0 = c5;
               M1 = c6;
               d0 = c8;
               d1 = c9;
            }

            if (!checkDate(y0, y1, y2, y3, M0, M1, d0, d1)) {
               return false;
            } else {
               this.setCalendar(y0, y1, y2, y3, M0, M1, d0, d1);
               char t = this.charAt(this.bp + date_len);
               if (t != 'T' && (t != ' ' || strict)) {
                  if (t != '"' && t != 26 && t != 26085 && t != '일') {
                     if (t != '+' && t != '-') {
                        return false;
                     } else if (this.len == date_len + 6) {
                        if (this.charAt(this.bp + date_len + 3) == ':' && this.charAt(this.bp + date_len + 4) == '0' && this.charAt(this.bp + date_len + 5) == '0') {
                           this.setTime('0', '0', '0', '0', '0', '0');
                           this.calendar.set(14, 0);
                           this.setTimeZone(t, this.charAt(this.bp + date_len + 1), this.charAt(this.bp + date_len + 2));
                           return true;
                        } else {
                           return false;
                        }
                     } else {
                        return false;
                     }
                  } else {
                     this.calendar.set(11, 0);
                     this.calendar.set(12, 0);
                     this.calendar.set(13, 0);
                     this.calendar.set(14, 0);
                     this.ch = this.charAt(this.bp += date_len);
                     this.token = 5;
                     return true;
                  }
               } else if (rest < date_len + 9) {
                  return false;
               } else if (this.charAt(this.bp + date_len + 3) != ':') {
                  return false;
               } else if (this.charAt(this.bp + date_len + 6) != ':') {
                  return false;
               } else {
                  char h0 = this.charAt(this.bp + date_len + 1);
                  char h1 = this.charAt(this.bp + date_len + 2);
                  char m0 = this.charAt(this.bp + date_len + 4);
                  char m1 = this.charAt(this.bp + date_len + 5);
                  s0 = this.charAt(this.bp + date_len + 7);
                  s1 = this.charAt(this.bp + date_len + 8);
                  if (!this.checkTime(h0, h1, m0, m1, s0, s1)) {
                     return false;
                  } else {
                     this.setTime(h0, h1, m0, m1, s0, s1);
                     dot = this.charAt(this.bp + date_len + 9);
                     if (dot == '.') {
                        if (rest < date_len + 11) {
                           return false;
                        } else {
                           S0 = this.charAt(this.bp + date_len + 10);
                           if (S0 >= '0' && S0 <= '9') {
                              int millis = S0 - 48;
                              int millisLen = 1;
                              if (rest > date_len + 11) {
                                 S2 = this.charAt(this.bp + date_len + 11);
                                 if (S2 >= '0' && S2 <= '9') {
                                    millis = millis * 10 + (S2 - 48);
                                    millisLen = 2;
                                 }
                              }

                              if (millisLen == 2) {
                                 S2 = this.charAt(this.bp + date_len + 12);
                                 if (S2 >= '0' && S2 <= '9') {
                                    millis = millis * 10 + (S2 - 48);
                                    millisLen = 3;
                                 }
                              }

                              this.calendar.set(14, millis);
                              int timzeZoneLength = 0;
                              timeZoneFlag = this.charAt(this.bp + date_len + 10 + millisLen);
                              if (timeZoneFlag != '+' && timeZoneFlag != '-') {
                                 if (timeZoneFlag == 'Z') {
                                    timzeZoneLength = 1;
                                    if (this.calendar.getTimeZone().getRawOffset() != 0) {
                                       String[] timeZoneIDs = TimeZone.getAvailableIDs(0);
                                       if (timeZoneIDs.length > 0) {
                                          TimeZone timeZone = TimeZone.getTimeZone(timeZoneIDs[0]);
                                          this.calendar.setTimeZone(timeZone);
                                       }
                                    }
                                 }
                              } else {
                                 label491: {
                                    t0 = this.charAt(this.bp + date_len + 10 + millisLen + 1);
                                    if (t0 >= '0' && t0 <= '1') {
                                       t1 = this.charAt(this.bp + date_len + 10 + millisLen + 2);
                                       if (t1 >= '0' && t1 <= '9') {
                                          t2 = this.charAt(this.bp + date_len + 10 + millisLen + 3);
                                          t3 = '0';
                                          t4 = '0';
                                          if (t2 == ':') {
                                             t3 = this.charAt(this.bp + date_len + 10 + millisLen + 4);
                                             if (t3 != '0' && t3 != '3') {
                                                return false;
                                             }

                                             t4 = this.charAt(this.bp + date_len + 10 + millisLen + 5);
                                             if (t4 != '0') {
                                                return false;
                                             }

                                             timzeZoneLength = 6;
                                          } else if (t2 == '0') {
                                             t3 = this.charAt(this.bp + date_len + 10 + millisLen + 4);
                                             if (t3 != '0' && t3 != '3') {
                                                return false;
                                             }

                                             timzeZoneLength = 5;
                                          } else {
                                             timzeZoneLength = 3;
                                          }

                                          this.setTimeZone(timeZoneFlag, t0, t1, t3, t4);
                                          break label491;
                                       }

                                       return false;
                                    }

                                    return false;
                                 }
                              }

                              t0 = this.charAt(this.bp + date_len + 10 + millisLen + timzeZoneLength);
                              if (t0 != 26 && t0 != '"') {
                                 return false;
                              } else {
                                 this.ch = this.charAt(this.bp += date_len + 10 + millisLen + timzeZoneLength);
                                 this.token = 5;
                                 return true;
                              }
                           } else {
                              return false;
                           }
                        }
                     } else {
                        this.calendar.set(14, 0);
                        this.ch = this.charAt(this.bp += date_len + 9);
                        this.token = 5;
                        if (dot == 'Z' && this.calendar.getTimeZone().getRawOffset() != 0) {
                           String[] timeZoneIDs = TimeZone.getAvailableIDs(0);
                           if (timeZoneIDs.length > 0) {
                              TimeZone timeZone = TimeZone.getTimeZone(timeZoneIDs[0]);
                              this.calendar.setTimeZone(timeZone);
                           }
                        }

                        return true;
                     }
                  }
               }
            }
         }
      }
   }

   protected void setTime(char h0, char h1, char m0, char m1, char s0, char s1) {
      int hour = (h0 - 48) * 10 + (h1 - 48);
      int minute = (m0 - 48) * 10 + (m1 - 48);
      int seconds = (s0 - 48) * 10 + (s1 - 48);
      this.calendar.set(11, hour);
      this.calendar.set(12, minute);
      this.calendar.set(13, seconds);
   }

   protected void setTimeZone(char timeZoneFlag, char t0, char t1) {
      this.setTimeZone(timeZoneFlag, t0, t1, '0', '0');
   }

   protected void setTimeZone(char timeZoneFlag, char t0, char t1, char t3, char t4) {
      int timeZoneOffset = ((t0 - 48) * 10 + (t1 - 48)) * 3600 * 1000;
      timeZoneOffset += ((t3 - 48) * 10 + (t4 - 48)) * 60 * 1000;
      if (timeZoneFlag == '-') {
         timeZoneOffset = -timeZoneOffset;
      }

      if (this.calendar.getTimeZone().getRawOffset() != timeZoneOffset) {
         String[] timeZoneIDs = TimeZone.getAvailableIDs(timeZoneOffset);
         if (timeZoneIDs.length > 0) {
            TimeZone timeZone = TimeZone.getTimeZone(timeZoneIDs[0]);
            this.calendar.setTimeZone(timeZone);
         }
      }

   }

   private boolean checkTime(char h0, char h1, char m0, char m1, char s0, char s1) {
      if (h0 == '0') {
         if (h1 < '0' || h1 > '9') {
            return false;
         }
      } else if (h0 == '1') {
         if (h1 < '0' || h1 > '9') {
            return false;
         }
      } else {
         if (h0 != '2') {
            return false;
         }

         if (h1 < '0' || h1 > '4') {
            return false;
         }
      }

      if (m0 >= '0' && m0 <= '5') {
         if (m1 < '0' || m1 > '9') {
            return false;
         }
      } else {
         if (m0 != '6') {
            return false;
         }

         if (m1 != '0') {
            return false;
         }
      }

      if (s0 >= '0' && s0 <= '5') {
         if (s1 < '0' || s1 > '9') {
            return false;
         }
      } else {
         if (s0 != '6') {
            return false;
         }

         if (s1 != '0') {
            return false;
         }
      }

      return true;
   }

   private void setCalendar(char y0, char y1, char y2, char y3, char M0, char M1, char d0, char d1) {
      this.calendar = Calendar.getInstance(this.timeZone, this.locale);
      int year = (y0 - 48) * 1000 + (y1 - 48) * 100 + (y2 - 48) * 10 + (y3 - 48);
      int month = (M0 - 48) * 10 + (M1 - 48) - 1;
      int day = (d0 - 48) * 10 + (d1 - 48);
      this.calendar.set(1, year);
      this.calendar.set(2, month);
      this.calendar.set(5, day);
   }

   static boolean checkDate(char y0, char y1, char y2, char y3, char M0, char M1, int d0, int d1) {
      if (y0 >= '1' && y0 <= '3') {
         if (y1 >= '0' && y1 <= '9') {
            if (y2 >= '0' && y2 <= '9') {
               if (y3 >= '0' && y3 <= '9') {
                  if (M0 == '0') {
                     if (M1 < '1' || M1 > '9') {
                        return false;
                     }
                  } else {
                     if (M0 != '1') {
                        return false;
                     }

                     if (M1 != '0' && M1 != '1' && M1 != '2') {
                        return false;
                     }
                  }

                  if (d0 == 48) {
                     if (d1 < 49 || d1 > 57) {
                        return false;
                     }
                  } else if (d0 != 49 && d0 != 50) {
                     if (d0 != 51) {
                        return false;
                     }

                     if (d1 != 48 && d1 != 49) {
                        return false;
                     }
                  } else if (d1 < 48 || d1 > 57) {
                     return false;
                  }

                  return true;
               } else {
                  return false;
               }
            } else {
               return false;
            }
         } else {
            return false;
         }
      } else {
         return false;
      }
   }

   public boolean isEOF() {
      return this.bp == this.len || this.ch == 26 && this.bp + 1 == this.len;
   }

   public int scanFieldInt(char[] fieldName) {
      this.matchStat = 0;
      int startPos = this.bp;
      char startChar = this.ch;
      if (!charArrayCompare(this.text, this.bp, fieldName)) {
         this.matchStat = -2;
         return 0;
      } else {
         int index = this.bp + fieldName.length;
         char ch = this.charAt(index++);
         boolean quote = ch == '"';
         if (quote) {
            ch = this.charAt(index++);
         }

         boolean negative = ch == '-';
         if (negative) {
            ch = this.charAt(index++);
         }

         if (ch >= '0' && ch <= '9') {
            int value = ch - 48;

            while(true) {
               ch = this.charAt(index++);
               if (ch < '0' || ch > '9') {
                  if (ch == '.') {
                     this.matchStat = -1;
                     return 0;
                  } else if (value < 0) {
                     this.matchStat = -1;
                     return 0;
                  } else {
                     if (quote) {
                        if (ch != '"') {
                           this.matchStat = -1;
                           return 0;
                        }

                        ch = this.charAt(index++);
                     }

                     while(ch != ',' && ch != '}') {
                        if (!isWhitespace(ch)) {
                           this.matchStat = -1;
                           return 0;
                        }

                        ch = this.charAt(index++);
                     }

                     this.bp = index - 1;
                     if (ch == ',') {
                        this.ch = this.charAt(++this.bp);
                        this.matchStat = 3;
                        this.token = 16;
                        return negative ? -value : value;
                     } else {
                        if (ch == '}') {
                           this.bp = index - 1;

                           label91: {
                              for(ch = this.charAt(++this.bp); ch != ','; ch = this.charAt(++this.bp)) {
                                 if (ch == ']') {
                                    this.token = 15;
                                    this.ch = this.charAt(++this.bp);
                                    break label91;
                                 }

                                 if (ch == '}') {
                                    this.token = 13;
                                    this.ch = this.charAt(++this.bp);
                                    break label91;
                                 }

                                 if (ch == 26) {
                                    this.token = 20;
                                    break label91;
                                 }

                                 if (!isWhitespace(ch)) {
                                    this.bp = startPos;
                                    this.ch = startChar;
                                    this.matchStat = -1;
                                    return 0;
                                 }
                              }

                              this.token = 16;
                              this.ch = this.charAt(++this.bp);
                           }

                           this.matchStat = 4;
                        }

                        return negative ? -value : value;
                     }
                  }
               }

               value = value * 10 + (ch - 48);
            }
         } else {
            this.matchStat = -1;
            return 0;
         }
      }
   }

   public String scanFieldString(char[] fieldName) {
      this.matchStat = 0;
      int startPos = this.bp;
      char startChar = this.ch;

      while(!charArrayCompare(this.text, this.bp, fieldName)) {
         if (!isWhitespace(this.ch)) {
            this.matchStat = -2;
            return this.stringDefaultValue();
         }

         this.next();
      }

      int index = this.bp + fieldName.length;
      char ch = this.charAt(index++);
      if (ch != '"') {
         this.matchStat = -1;
         return this.stringDefaultValue();
      } else {
         int endIndex = this.indexOf('"', index);
         if (endIndex == -1) {
            throw new JSONException("unclosed str");
         } else {
            String stringVal = this.subString(index, endIndex - index);
            if (stringVal.indexOf(92) != -1) {
               while(true) {
                  int chars_len = 0;

                  for(int i = endIndex - 1; i >= 0 && this.charAt(i) == '\\'; --i) {
                     ++chars_len;
                  }

                  if (chars_len % 2 == 0) {
                     chars_len = endIndex - (this.bp + fieldName.length + 1);
                     char[] chars = this.sub_chars(this.bp + fieldName.length + 1, chars_len);
                     stringVal = readString(chars, chars_len);
                     break;
                  }

                  endIndex = this.indexOf('"', endIndex + 1);
               }
            }

            for(ch = this.charAt(endIndex + 1); ch != ',' && ch != '}'; ch = this.charAt(endIndex + 1)) {
               if (!isWhitespace(ch)) {
                  this.matchStat = -1;
                  return this.stringDefaultValue();
               }

               ++endIndex;
            }

            this.bp = endIndex + 1;
            this.ch = ch;
            if (ch == ',') {
               this.ch = this.charAt(++this.bp);
               this.matchStat = 3;
               return stringVal;
            } else {
               ch = this.charAt(++this.bp);
               if (ch == ',') {
                  this.token = 16;
                  this.ch = this.charAt(++this.bp);
               } else if (ch == ']') {
                  this.token = 15;
                  this.ch = this.charAt(++this.bp);
               } else if (ch == '}') {
                  this.token = 13;
                  this.ch = this.charAt(++this.bp);
               } else {
                  if (ch != 26) {
                     this.bp = startPos;
                     this.ch = startChar;
                     this.matchStat = -1;
                     return this.stringDefaultValue();
                  }

                  this.token = 20;
               }

               this.matchStat = 4;
               return stringVal;
            }
         }
      }
   }

   public Date scanFieldDate(char[] fieldName) {
      this.matchStat = 0;
      int startPos = this.bp;
      char startChar = this.ch;
      if (!charArrayCompare(this.text, this.bp, fieldName)) {
         this.matchStat = -2;
         return null;
      } else {
         int index = this.bp + fieldName.length;
         char ch = this.charAt(index++);
         Date dateVal;
         if (ch == '"') {
            int endIndex = this.indexOf('"', index);
            if (endIndex == -1) {
               throw new JSONException("unclosed str");
            }

            int rest = endIndex - index;
            this.bp = index;
            if (!this.scanISO8601DateIfMatch(false, rest)) {
               this.bp = startPos;
               this.matchStat = -1;
               return null;
            }

            dateVal = this.calendar.getTime();
            ch = this.charAt(endIndex + 1);

            for(this.bp = startPos; ch != ',' && ch != '}'; ch = this.charAt(endIndex + 1)) {
               if (!isWhitespace(ch)) {
                  this.matchStat = -1;
                  return null;
               }

               ++endIndex;
            }

            this.bp = endIndex + 1;
            this.ch = ch;
         } else {
            if (ch != '-' && (ch < '0' || ch > '9')) {
               this.matchStat = -1;
               return null;
            }

            long millis = 0L;
            boolean negative = false;
            if (ch == '-') {
               ch = this.charAt(index++);
               negative = true;
            }

            if (ch >= '0' && ch <= '9') {
               millis = (long)(ch - 48);

               while(true) {
                  ch = this.charAt(index++);
                  if (ch < '0' || ch > '9') {
                     if (ch == ',' || ch == '}') {
                        this.bp = index - 1;
                     }
                     break;
                  }

                  millis = millis * 10L + (long)(ch - 48);
               }
            }

            if (millis < 0L) {
               this.matchStat = -1;
               return null;
            }

            if (negative) {
               millis = -millis;
            }

            dateVal = new Date(millis);
         }

         if (ch == ',') {
            this.ch = this.charAt(++this.bp);
            this.matchStat = 3;
            this.token = 16;
            return dateVal;
         } else {
            ch = this.charAt(++this.bp);
            if (ch == ',') {
               this.token = 16;
               this.ch = this.charAt(++this.bp);
            } else if (ch == ']') {
               this.token = 15;
               this.ch = this.charAt(++this.bp);
            } else if (ch == '}') {
               this.token = 13;
               this.ch = this.charAt(++this.bp);
            } else {
               if (ch != 26) {
                  this.bp = startPos;
                  this.ch = startChar;
                  this.matchStat = -1;
                  return null;
               }

               this.token = 20;
            }

            this.matchStat = 4;
            return dateVal;
         }
      }
   }

   public long scanFieldSymbol(char[] fieldName) {
      this.matchStat = 0;
      if (!charArrayCompare(this.text, this.bp, fieldName)) {
         this.matchStat = -2;
         return 0L;
      } else {
         int index = this.bp + fieldName.length;
         char ch = this.charAt(index++);
         if (ch != '"') {
            this.matchStat = -1;
            return 0L;
         } else {
            long hash = -3750763034362895579L;

            while(true) {
               ch = this.charAt(index++);
               if (ch == '"') {
                  this.bp = index;

                  for(this.ch = ch = this.charAt(this.bp); ch != ','; ch = this.charAt(++this.bp)) {
                     if (ch == '}') {
                        this.next();
                        this.skipWhitespace();
                        ch = this.getCurrent();
                        if (ch == ',') {
                           this.token = 16;
                           this.ch = this.charAt(++this.bp);
                        } else if (ch == ']') {
                           this.token = 15;
                           this.ch = this.charAt(++this.bp);
                        } else if (ch == '}') {
                           this.token = 13;
                           this.ch = this.charAt(++this.bp);
                        } else {
                           if (ch != 26) {
                              this.matchStat = -1;
                              return 0L;
                           }

                           this.token = 20;
                        }

                        this.matchStat = 4;
                        return hash;
                     }

                     if (!isWhitespace(ch)) {
                        this.matchStat = -1;
                        return 0L;
                     }
                  }

                  this.ch = this.charAt(++this.bp);
                  this.matchStat = 3;
                  return hash;
               }

               if (index > this.len) {
                  this.matchStat = -1;
                  return 0L;
               }

               hash ^= (long)ch;
               hash *= 1099511628211L;
            }
         }
      }
   }

   public Collection<String> newCollectionByType(Class<?> type) {
      if (type.isAssignableFrom(HashSet.class)) {
         HashSet<String> list = new HashSet();
         return list;
      } else if (type.isAssignableFrom(ArrayList.class)) {
         ArrayList<String> list2 = new ArrayList();
         return list2;
      } else {
         try {
            Collection<String> list = (Collection)type.newInstance();
            return list;
         } catch (Exception var3) {
            throw new JSONException(var3.getMessage(), var3);
         }
      }
   }

   public Collection<String> scanFieldStringArray(char[] fieldName, Class<?> type) {
      this.matchStat = 0;
      if (!charArrayCompare(this.text, this.bp, fieldName)) {
         this.matchStat = -2;
         return null;
      } else {
         Collection<String> list = this.newCollectionByType(type);
         int index = this.bp + fieldName.length;
         char ch = this.charAt(index++);
         if (ch != '[') {
            if (!this.text.startsWith("ull", index)) {
               this.matchStat = -1;
               return null;
            }

            index += 3;
            ch = this.charAt(index++);
            list = null;
         } else {
            ch = this.charAt(index++);

            label124:
            while(true) {
               if (ch == '"') {
                  int endIndex = this.indexOf('"', index);
                  if (endIndex == -1) {
                     throw new JSONException("unclosed str");
                  }

                  String stringVal = this.subString(index, endIndex - index);
                  if (stringVal.indexOf(92) != -1) {
                     while(true) {
                        int chars_len = 0;

                        for(int i = endIndex - 1; i >= 0 && this.charAt(i) == '\\'; --i) {
                           ++chars_len;
                        }

                        if (chars_len % 2 == 0) {
                           chars_len = endIndex - index;
                           char[] chars = this.sub_chars(index, chars_len);
                           stringVal = readString(chars, chars_len);
                           break;
                        }

                        endIndex = this.indexOf('"', endIndex + 1);
                     }
                  }

                  index = endIndex + 1;
                  ch = this.charAt(index++);
                  list.add(stringVal);
               } else {
                  if (ch != 'n' || !this.text.startsWith("ull", index)) {
                     if (ch != ']' || list.size() != 0) {
                        this.matchStat = -1;
                        return null;
                     }

                     ch = this.charAt(index++);
                     break;
                  }

                  index += 3;
                  ch = this.charAt(index++);
                  list.add((Object)null);
               }

               if (ch != ',') {
                  if (ch != ']') {
                     this.matchStat = -1;
                     return null;
                  }

                  ch = this.charAt(index++);

                  while(true) {
                     if (!isWhitespace(ch)) {
                        break label124;
                     }

                     ch = this.charAt(index++);
                  }
               }

               ch = this.charAt(index++);
            }
         }

         this.bp = index;
         if (ch == ',') {
            this.ch = this.charAt(this.bp);
            this.matchStat = 3;
            return list;
         } else if (ch != '}') {
            this.matchStat = -1;
            return null;
         } else {
            ch = this.charAt(this.bp);

            while(true) {
               if (ch == ',') {
                  this.token = 16;
                  this.ch = this.charAt(++this.bp);
                  break;
               }

               if (ch == ']') {
                  this.token = 15;
                  this.ch = this.charAt(++this.bp);
                  break;
               }

               if (ch == '}') {
                  this.token = 13;
                  this.ch = this.charAt(++this.bp);
                  break;
               }

               if (ch == 26) {
                  this.token = 20;
                  this.ch = ch;
                  break;
               }

               boolean space;
               for(space = false; isWhitespace(ch); space = true) {
                  ch = this.charAt(index++);
                  this.bp = index;
               }

               if (!space) {
                  this.matchStat = -1;
                  return null;
               }
            }

            this.matchStat = 4;
            return list;
         }
      }
   }

   public long scanFieldLong(char[] fieldName) {
      this.matchStat = 0;
      int startPos = this.bp;
      char startChar = this.ch;
      if (!charArrayCompare(this.text, this.bp, fieldName)) {
         this.matchStat = -2;
         return 0L;
      } else {
         int index = this.bp + fieldName.length;
         char ch = this.charAt(index++);
         boolean quote = ch == '"';
         if (quote) {
            ch = this.charAt(index++);
         }

         boolean negative = false;
         if (ch == '-') {
            ch = this.charAt(index++);
            negative = true;
         }

         if (ch >= '0' && ch <= '9') {
            long value = (long)(ch - 48);

            while(true) {
               ch = this.charAt(index++);
               if (ch < '0' || ch > '9') {
                  if (ch == '.') {
                     this.matchStat = -1;
                     return 0L;
                  } else {
                     if (quote) {
                        if (ch != '"') {
                           this.matchStat = -1;
                           return 0L;
                        }

                        ch = this.charAt(index++);
                     }

                     if (ch == ',' || ch == '}') {
                        this.bp = index - 1;
                     }

                     boolean valid = value >= 0L || value == Long.MIN_VALUE && negative;
                     if (!valid) {
                        this.bp = startPos;
                        this.ch = startChar;
                        this.matchStat = -1;
                        return 0L;
                     } else {
                        while(ch != ',') {
                           if (ch == '}') {
                              ch = this.charAt(++this.bp);

                              while(true) {
                                 if (ch == ',') {
                                    this.token = 16;
                                    this.ch = this.charAt(++this.bp);
                                    break;
                                 }

                                 if (ch == ']') {
                                    this.token = 15;
                                    this.ch = this.charAt(++this.bp);
                                    break;
                                 }

                                 if (ch == '}') {
                                    this.token = 13;
                                    this.ch = this.charAt(++this.bp);
                                    break;
                                 }

                                 if (ch == 26) {
                                    this.token = 20;
                                    break;
                                 }

                                 if (!isWhitespace(ch)) {
                                    this.bp = startPos;
                                    this.ch = startChar;
                                    this.matchStat = -1;
                                    return 0L;
                                 }

                                 ch = this.charAt(++this.bp);
                              }

                              this.matchStat = 4;
                              return negative ? -value : value;
                           }

                           if (!isWhitespace(ch)) {
                              this.matchStat = -1;
                              return 0L;
                           }

                           this.bp = index;
                           ch = this.charAt(index++);
                        }

                        this.ch = this.charAt(++this.bp);
                        this.matchStat = 3;
                        this.token = 16;
                        return negative ? -value : value;
                     }
                  }
               }

               value = value * 10L + (long)(ch - 48);
            }
         } else {
            this.bp = startPos;
            this.ch = startChar;
            this.matchStat = -1;
            return 0L;
         }
      }
   }

   public boolean scanFieldBoolean(char[] fieldName) {
      this.matchStat = 0;
      if (!charArrayCompare(this.text, this.bp, fieldName)) {
         this.matchStat = -2;
         return false;
      } else {
         int startPos = this.bp;
         int index = this.bp + fieldName.length;
         char ch = this.charAt(index++);
         boolean quote = ch == '"';
         if (quote) {
            ch = this.charAt(index++);
         }

         boolean value;
         if (ch == 't') {
            if (this.charAt(index++) != 'r') {
               this.matchStat = -1;
               return false;
            }

            if (this.charAt(index++) != 'u') {
               this.matchStat = -1;
               return false;
            }

            if (this.charAt(index++) != 'e') {
               this.matchStat = -1;
               return false;
            }

            if (quote && this.charAt(index++) != '"') {
               this.matchStat = -1;
               return false;
            }

            this.bp = index;
            ch = this.charAt(this.bp);
            value = true;
         } else if (ch == 'f') {
            if (this.charAt(index++) != 'a') {
               this.matchStat = -1;
               return false;
            }

            if (this.charAt(index++) != 'l') {
               this.matchStat = -1;
               return false;
            }

            if (this.charAt(index++) != 's') {
               this.matchStat = -1;
               return false;
            }

            if (this.charAt(index++) != 'e') {
               this.matchStat = -1;
               return false;
            }

            if (quote && this.charAt(index++) != '"') {
               this.matchStat = -1;
               return false;
            }

            this.bp = index;
            ch = this.charAt(this.bp);
            value = false;
         } else if (ch == '1') {
            if (quote && this.charAt(index++) != '"') {
               this.matchStat = -1;
               return false;
            }

            this.bp = index;
            ch = this.charAt(this.bp);
            value = true;
         } else {
            if (ch != '0') {
               this.matchStat = -1;
               return false;
            }

            if (quote && this.charAt(index++) != '"') {
               this.matchStat = -1;
               return false;
            }

            this.bp = index;
            ch = this.charAt(this.bp);
            value = false;
         }

         while(true) {
            if (ch == ',') {
               this.ch = this.charAt(++this.bp);
               this.matchStat = 3;
               this.token = 16;
               break;
            }

            if (ch == '}') {
               label118: {
                  for(ch = this.charAt(++this.bp); ch != ','; ch = this.charAt(++this.bp)) {
                     if (ch == ']') {
                        this.token = 15;
                        this.ch = this.charAt(++this.bp);
                        break label118;
                     }

                     if (ch == '}') {
                        this.token = 13;
                        this.ch = this.charAt(++this.bp);
                        break label118;
                     }

                     if (ch == 26) {
                        this.token = 20;
                        break label118;
                     }

                     if (!isWhitespace(ch)) {
                        this.matchStat = -1;
                        return false;
                     }
                  }

                  this.token = 16;
                  this.ch = this.charAt(++this.bp);
               }

               this.matchStat = 4;
               break;
            }

            if (!isWhitespace(ch)) {
               this.bp = startPos;
               ch = this.charAt(this.bp);
               this.matchStat = -1;
               return false;
            }

            ch = this.charAt(++this.bp);
         }

         return value;
      }
   }

   public final int scanInt(char expectNext) {
      this.matchStat = 0;
      int offset = this.bp;

      char chLocal;
      for(chLocal = this.charAt(offset++); isWhitespace(chLocal); chLocal = this.charAt(offset++)) {
      }

      boolean quote = chLocal == '"';
      if (quote) {
         chLocal = this.charAt(offset++);
      }

      boolean negative = chLocal == '-';
      if (negative) {
         chLocal = this.charAt(offset++);
      }

      if (chLocal >= '0' && chLocal <= '9') {
         int value = chLocal - 48;

         while(true) {
            chLocal = this.charAt(offset++);
            if (chLocal < '0' || chLocal > '9') {
               if (chLocal == '.') {
                  this.matchStat = -1;
                  return 0;
               } else {
                  if (quote) {
                     if (chLocal != '"') {
                        this.matchStat = -1;
                        return 0;
                     }

                     chLocal = this.charAt(offset++);
                  }

                  if (value < 0) {
                     this.matchStat = -1;
                     return 0;
                  } else {
                     while(chLocal != expectNext) {
                        if (!isWhitespace(chLocal)) {
                           this.matchStat = -1;
                           return negative ? -value : value;
                        }

                        chLocal = this.charAt(offset++);
                     }

                     this.bp = offset;
                     this.ch = this.charAt(this.bp);
                     this.matchStat = 3;
                     this.token = 16;
                     return negative ? -value : value;
                  }
               }
            }

            value = value * 10 + (chLocal - 48);
         }
      } else if (chLocal == 'n' && this.charAt(offset++) == 'u' && this.charAt(offset++) == 'l' && this.charAt(offset++) == 'l') {
         this.matchStat = 5;
         int value = 0;
         chLocal = this.charAt(offset++);
         if (quote && chLocal == '"') {
            chLocal = this.charAt(offset++);
         }

         while(chLocal != ',') {
            if (chLocal == ']') {
               this.bp = offset;
               this.ch = this.charAt(this.bp);
               this.matchStat = 5;
               this.token = 15;
               return value;
            }

            if (!isWhitespace(chLocal)) {
               this.matchStat = -1;
               return 0;
            }

            chLocal = this.charAt(offset++);
         }

         this.bp = offset;
         this.ch = this.charAt(this.bp);
         this.matchStat = 5;
         this.token = 16;
         return value;
      } else {
         this.matchStat = -1;
         return 0;
      }
   }

   public double scanDouble(char seperator) {
      this.matchStat = 0;
      int offset = this.bp;
      char chLocal = this.charAt(offset++);
      boolean quote = chLocal == '"';
      if (quote) {
         chLocal = this.charAt(offset++);
      }

      boolean negative = chLocal == '-';
      if (negative) {
         chLocal = this.charAt(offset++);
      }

      double value;
      if (chLocal >= '0' && chLocal <= '9') {
         long intVal = (long)(chLocal - 48);

         while(true) {
            chLocal = this.charAt(offset++);
            if (chLocal < '0' || chLocal > '9') {
               long power = 1L;
               boolean small = chLocal == '.';
               if (small) {
                  chLocal = this.charAt(offset++);
                  if (chLocal < '0' || chLocal > '9') {
                     this.matchStat = -1;
                     return 0.0D;
                  }

                  intVal = intVal * 10L + (long)(chLocal - 48);
                  power = 10L;

                  while(true) {
                     chLocal = this.charAt(offset++);
                     if (chLocal < '0' || chLocal > '9') {
                        break;
                     }

                     intVal = intVal * 10L + (long)(chLocal - 48);
                     power *= 10L;
                  }
               }

               boolean exp = chLocal == 'e' || chLocal == 'E';
               if (exp) {
                  chLocal = this.charAt(offset++);
                  if (chLocal == '+' || chLocal == '-') {
                     chLocal = this.charAt(offset++);
                  }

                  while(chLocal >= '0' && chLocal <= '9') {
                     chLocal = this.charAt(offset++);
                  }
               }

               int start;
               int count;
               if (quote) {
                  if (chLocal != '"') {
                     this.matchStat = -1;
                     return 0.0D;
                  }

                  chLocal = this.charAt(offset++);
                  start = this.bp + 1;
                  count = offset - start - 2;
               } else {
                  start = this.bp;
                  count = offset - start - 1;
               }

               if (!exp && count < 20) {
                  value = (double)intVal / (double)power;
                  if (negative) {
                     value = -value;
                  }
               } else {
                  String text = this.subString(start, count);
                  value = Double.parseDouble(text);
               }

               if (chLocal == seperator) {
                  this.bp = offset;
                  this.ch = this.charAt(this.bp);
                  this.matchStat = 3;
                  this.token = 16;
                  return value;
               } else {
                  this.matchStat = -1;
                  return value;
               }
            }

            intVal = intVal * 10L + (long)(chLocal - 48);
         }
      } else if (chLocal == 'n' && this.charAt(offset++) == 'u' && this.charAt(offset++) == 'l' && this.charAt(offset++) == 'l') {
         this.matchStat = 5;
         value = 0.0D;
         chLocal = this.charAt(offset++);
         if (quote && chLocal == '"') {
            chLocal = this.charAt(offset++);
         }

         while(chLocal != ',') {
            if (chLocal == ']') {
               this.bp = offset;
               this.ch = this.charAt(this.bp);
               this.matchStat = 5;
               this.token = 15;
               return value;
            }

            if (!isWhitespace(chLocal)) {
               this.matchStat = -1;
               return 0.0D;
            }

            chLocal = this.charAt(offset++);
         }

         this.bp = offset;
         this.ch = this.charAt(this.bp);
         this.matchStat = 5;
         this.token = 16;
         return value;
      } else {
         this.matchStat = -1;
         return 0.0D;
      }
   }

   public long scanLong(char seperator) {
      this.matchStat = 0;
      int offset = this.bp;
      char chLocal = this.charAt(offset++);
      boolean quote = chLocal == '"';
      if (quote) {
         chLocal = this.charAt(offset++);
      }

      boolean negative = chLocal == '-';
      if (negative) {
         chLocal = this.charAt(offset++);
      }

      long value;
      if (chLocal >= '0' && chLocal <= '9') {
         value = (long)(chLocal - 48);

         while(true) {
            chLocal = this.charAt(offset++);
            if (chLocal < '0' || chLocal > '9') {
               if (chLocal == '.') {
                  this.matchStat = -1;
                  return 0L;
               } else {
                  if (quote) {
                     if (chLocal != '"') {
                        this.matchStat = -1;
                        return 0L;
                     }

                     chLocal = this.charAt(offset++);
                  }

                  boolean valid = value >= 0L || value == Long.MIN_VALUE && negative;
                  if (!valid) {
                     this.matchStat = -1;
                     return 0L;
                  } else {
                     while(chLocal != seperator) {
                        if (!isWhitespace(chLocal)) {
                           this.matchStat = -1;
                           return value;
                        }

                        chLocal = this.charAt(offset++);
                     }

                     this.bp = offset;
                     this.ch = this.charAt(this.bp);
                     this.matchStat = 3;
                     this.token = 16;
                     return negative ? -value : value;
                  }
               }
            }

            value = value * 10L + (long)(chLocal - 48);
         }
      } else if (chLocal == 'n' && this.charAt(offset++) == 'u' && this.charAt(offset++) == 'l' && this.charAt(offset++) == 'l') {
         this.matchStat = 5;
         value = 0L;
         chLocal = this.charAt(offset++);
         if (quote && chLocal == '"') {
            chLocal = this.charAt(offset++);
         }

         while(chLocal != ',') {
            if (chLocal == ']') {
               this.bp = offset;
               this.ch = this.charAt(this.bp);
               this.matchStat = 5;
               this.token = 15;
               return value;
            }

            if (!isWhitespace(chLocal)) {
               this.matchStat = -1;
               return 0L;
            }

            chLocal = this.charAt(offset++);
         }

         this.bp = offset;
         this.ch = this.charAt(this.bp);
         this.matchStat = 5;
         this.token = 16;
         return value;
      } else {
         this.matchStat = -1;
         return 0L;
      }
   }

   public Date scanDate(char seperator) {
      this.matchStat = 0;
      int startPos = this.bp;
      char startChar = this.ch;
      int index = this.bp;
      char ch = this.charAt(index++);
      Date dateVal;
      if (ch == '"') {
         int endIndex = this.indexOf('"', index);
         if (endIndex == -1) {
            throw new JSONException("unclosed str");
         }

         int rest = endIndex - index;
         this.bp = index;
         if (!this.scanISO8601DateIfMatch(false, rest)) {
            this.bp = startPos;
            this.ch = startChar;
            this.matchStat = -1;
            return null;
         }

         dateVal = this.calendar.getTime();
         ch = this.charAt(endIndex + 1);

         for(this.bp = startPos; ch != ',' && ch != ']'; ch = this.charAt(endIndex + 1)) {
            if (!isWhitespace(ch)) {
               this.bp = startPos;
               this.ch = startChar;
               this.matchStat = -1;
               return null;
            }

            ++endIndex;
         }

         this.bp = endIndex + 1;
         this.ch = ch;
      } else if (ch != '-' && (ch < '0' || ch > '9')) {
         if (ch != 'n' || this.charAt(index++) != 'u' || this.charAt(index++) != 'l' || this.charAt(index++) != 'l') {
            this.bp = startPos;
            this.ch = startChar;
            this.matchStat = -1;
            return null;
         }

         dateVal = null;
         ch = this.charAt(index);
         this.bp = index;
      } else {
         long millis = 0L;
         boolean negative = false;
         if (ch == '-') {
            ch = this.charAt(index++);
            negative = true;
         }

         if (ch >= '0' && ch <= '9') {
            millis = (long)(ch - 48);

            while(true) {
               ch = this.charAt(index++);
               if (ch < '0' || ch > '9') {
                  if (ch == ',' || ch == ']') {
                     this.bp = index - 1;
                  }
                  break;
               }

               millis = millis * 10L + (long)(ch - 48);
            }
         }

         if (millis < 0L) {
            this.bp = startPos;
            this.ch = startChar;
            this.matchStat = -1;
            return null;
         }

         if (negative) {
            millis = -millis;
         }

         dateVal = new Date(millis);
      }

      if (ch == ',') {
         this.ch = this.charAt(++this.bp);
         this.matchStat = 3;
         return dateVal;
      } else {
         ch = this.charAt(++this.bp);
         if (ch == ',') {
            this.token = 16;
            this.ch = this.charAt(++this.bp);
         } else if (ch == ']') {
            this.token = 15;
            this.ch = this.charAt(++this.bp);
         } else if (ch == '}') {
            this.token = 13;
            this.ch = this.charAt(++this.bp);
         } else {
            if (ch != 26) {
               this.bp = startPos;
               this.ch = startChar;
               this.matchStat = -1;
               return null;
            }

            this.ch = 26;
            this.token = 20;
         }

         this.matchStat = 4;
         return dateVal;
      }
   }

   protected final void arrayCopy(int srcPos, char[] dest, int destPos, int length) {
      this.text.getChars(srcPos, srcPos + length, dest, destPos);
   }

   public String info() {
      return "pos " + this.bp + ", json : " + (this.text.length() < 65536 ? this.text : this.text.substring(0, 65536));
   }

   public String[] scanFieldStringArray(char[] fieldName, int argTypesCount, SymbolTable typeSymbolTable) {
      int startPos = this.bp;
      char starChar = this.ch;

      while(isWhitespace(this.ch)) {
         this.next();
      }

      int offset;
      char ch;
      if (fieldName != null) {
         this.matchStat = 0;
         if (!this.charArrayCompare(fieldName)) {
            this.matchStat = -2;
            return null;
         }

         offset = this.bp + fieldName.length;

         for(ch = this.text.charAt(offset++); isWhitespace(ch); ch = this.text.charAt(offset++)) {
         }

         if (ch != ':') {
            this.matchStat = -1;
            return null;
         }

         for(ch = this.text.charAt(offset++); isWhitespace(ch); ch = this.text.charAt(offset++)) {
         }
      } else {
         offset = this.bp + 1;
         ch = this.ch;
      }

      if (ch != '[') {
         if (ch == 'n' && this.text.startsWith("ull", this.bp + 1)) {
            this.bp += 4;
            this.ch = this.text.charAt(this.bp);
            return null;
         } else {
            this.matchStat = -1;
            return null;
         }
      } else {
         this.bp = offset;
         this.ch = this.text.charAt(this.bp);
         String[] types = argTypesCount >= 0 ? new String[argTypesCount] : new String[4];
         int typeIndex = 0;

         while(true) {
            while(!isWhitespace(this.ch)) {
               if (this.ch != '"') {
                  this.bp = startPos;
                  this.ch = starChar;
                  this.matchStat = -1;
                  return null;
               }

               String type = this.scanSymbol(typeSymbolTable, '"');
               if (typeIndex == types.length) {
                  int newCapacity = types.length + (types.length >> 1) + 1;
                  String[] array = new String[newCapacity];
                  System.arraycopy(types, 0, array, 0, types.length);
                  types = array;
               }

               types[typeIndex++] = type;

               while(isWhitespace(this.ch)) {
                  this.next();
               }

               if (this.ch != ',') {
                  if (types.length != typeIndex) {
                     String[] array = new String[typeIndex];
                     System.arraycopy(types, 0, array, 0, typeIndex);
                     types = array;
                  }

                  while(isWhitespace(this.ch)) {
                     this.next();
                  }

                  if (this.ch == ']') {
                     this.next();
                     return types;
                  }

                  this.bp = startPos;
                  this.ch = starChar;
                  this.matchStat = -1;
                  return null;
               }

               this.next();
            }

            this.next();
         }
      }
   }

   public boolean matchField2(char[] fieldName) {
      while(isWhitespace(this.ch)) {
         this.next();
      }

      if (!this.charArrayCompare(fieldName)) {
         this.matchStat = -2;
         return false;
      } else {
         int offset = this.bp + fieldName.length;

         char ch;
         for(ch = this.text.charAt(offset++); isWhitespace(ch); ch = this.text.charAt(offset++)) {
         }

         if (ch == ':') {
            this.bp = offset;
            this.ch = this.charAt(this.bp);
            return true;
         } else {
            this.matchStat = -2;
            return false;
         }
      }
   }
}
