package com.alibaba.fastjson.parser.deserializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.JSONLexerBase;
import com.alibaba.fastjson.parser.JSONToken;
import com.alibaba.fastjson.parser.ParseContext;
import com.alibaba.fastjson.parser.ParserConfig;
import com.alibaba.fastjson.util.FieldInfo;
import com.alibaba.fastjson.util.JavaBeanInfo;
import com.alibaba.fastjson.util.TypeUtils;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class JavaBeanDeserializer implements ObjectDeserializer {
   private final FieldDeserializer[] fieldDeserializers;
   protected final FieldDeserializer[] sortedFieldDeserializers;
   protected final Class<?> clazz;
   public final JavaBeanInfo beanInfo;
   private ConcurrentMap<String, Object> extraFieldDeserializers;
   private final Map<String, FieldDeserializer> alterNameFieldDeserializers;
   private transient long[] smartMatchHashArray;
   private transient short[] smartMatchHashArrayMapping;
   private transient long[] hashArray;
   private transient short[] hashArrayMapping;

   public JavaBeanDeserializer(ParserConfig config, Class<?> clazz) {
      this(config, clazz, clazz);
   }

   public JavaBeanDeserializer(ParserConfig config, Class<?> clazz, Type type) {
      this(config, JavaBeanInfo.build(clazz, type, config.propertyNamingStrategy, config.fieldBased, config.compatibleWithJavaBean));
   }

   public JavaBeanDeserializer(ParserConfig config, JavaBeanInfo beanInfo) {
      this.clazz = beanInfo.clazz;
      this.beanInfo = beanInfo;
      Map<String, FieldDeserializer> alterNameFieldDeserializers = null;
      this.sortedFieldDeserializers = new FieldDeserializer[beanInfo.sortedFields.length];
      int i = 0;

      int size;
      FieldInfo fieldInfo;
      FieldDeserializer fieldDeserializer;
      for(size = beanInfo.sortedFields.length; i < size; ++i) {
         fieldInfo = beanInfo.sortedFields[i];
         fieldDeserializer = config.createFieldDeserializer(config, beanInfo, fieldInfo);
         this.sortedFieldDeserializers[i] = fieldDeserializer;
         String[] var8 = fieldInfo.alternateNames;
         int var9 = var8.length;

         for(int var10 = 0; var10 < var9; ++var10) {
            String name = var8[var10];
            if (alterNameFieldDeserializers == null) {
               alterNameFieldDeserializers = new HashMap();
            }

            alterNameFieldDeserializers.put(name, fieldDeserializer);
         }
      }

      this.alterNameFieldDeserializers = alterNameFieldDeserializers;
      this.fieldDeserializers = new FieldDeserializer[beanInfo.fields.length];
      i = 0;

      for(size = beanInfo.fields.length; i < size; ++i) {
         fieldInfo = beanInfo.fields[i];
         fieldDeserializer = this.getFieldDeserializer(fieldInfo.name);
         this.fieldDeserializers[i] = fieldDeserializer;
      }

   }

   public FieldDeserializer getFieldDeserializer(String key) {
      return this.getFieldDeserializer(key, (int[])null);
   }

   public FieldDeserializer getFieldDeserializer(String key, int[] setFlags) {
      if (key == null) {
         return null;
      } else {
         int low = 0;
         int high = this.sortedFieldDeserializers.length - 1;

         while(low <= high) {
            int mid = low + high >>> 1;
            String fieldName = this.sortedFieldDeserializers[mid].fieldInfo.name;
            int cmp = fieldName.compareTo(key);
            if (cmp < 0) {
               low = mid + 1;
            } else {
               if (cmp <= 0) {
                  if (isSetFlag(mid, setFlags)) {
                     return null;
                  }

                  return this.sortedFieldDeserializers[mid];
               }

               high = mid - 1;
            }
         }

         if (this.alterNameFieldDeserializers != null) {
            return (FieldDeserializer)this.alterNameFieldDeserializers.get(key);
         } else {
            return null;
         }
      }
   }

   public FieldDeserializer getFieldDeserializer(long hash) {
      if (this.hashArray == null) {
         long[] hashArray = new long[this.sortedFieldDeserializers.length];

         for(int i = 0; i < this.sortedFieldDeserializers.length; ++i) {
            hashArray[i] = TypeUtils.fnv1a_64(this.sortedFieldDeserializers[i].fieldInfo.name);
         }

         Arrays.sort(hashArray);
         this.hashArray = hashArray;
      }

      int pos = Arrays.binarySearch(this.hashArray, hash);
      if (pos < 0) {
         return null;
      } else {
         if (this.hashArrayMapping == null) {
            short[] mapping = new short[this.hashArray.length];
            Arrays.fill(mapping, (short)-1);

            for(int i = 0; i < this.sortedFieldDeserializers.length; ++i) {
               int p = Arrays.binarySearch(this.hashArray, TypeUtils.fnv1a_64(this.sortedFieldDeserializers[i].fieldInfo.name));
               if (p >= 0) {
                  mapping[p] = (short)i;
               }
            }

            this.hashArrayMapping = mapping;
         }

         int setterIndex = this.hashArrayMapping[pos];
         return setterIndex != -1 ? this.sortedFieldDeserializers[setterIndex] : null;
      }
   }

   static boolean isSetFlag(int i, int[] setFlags) {
      if (setFlags == null) {
         return false;
      } else {
         int flagIndex = i / 32;
         int bitIndex = i % 32;
         return flagIndex < setFlags.length && (setFlags[flagIndex] & 1 << bitIndex) != 0;
      }
   }

   public Object createInstance(DefaultJSONParser parser, Type type) {
      if (type instanceof Class && this.clazz.isInterface()) {
         Class<?> clazz = (Class)type;
         ClassLoader loader = Thread.currentThread().getContextClassLoader();
         JSONObject obj = new JSONObject();
         Object proxy = Proxy.newProxyInstance(loader, new Class[]{clazz}, obj);
         return proxy;
      } else if (this.beanInfo.defaultConstructor == null && this.beanInfo.factoryMethod == null) {
         return null;
      } else if (this.beanInfo.factoryMethod != null && this.beanInfo.defaultConstructorParameterSize > 0) {
         return null;
      } else {
         Object object;
         try {
            Constructor<?> constructor = this.beanInfo.defaultConstructor;
            if (this.beanInfo.defaultConstructorParameterSize == 0) {
               if (constructor != null) {
                  object = constructor.newInstance();
               } else {
                  object = this.beanInfo.factoryMethod.invoke((Object)null);
               }
            } else {
               ParseContext context = parser.getContext();
               if (context == null || context.object == null) {
                  throw new JSONException("can't create non-static inner class instance.");
               }

               if (!(type instanceof Class)) {
                  throw new JSONException("can't create non-static inner class instance.");
               }

               String typeName = ((Class)type).getName();
               int lastIndex = typeName.lastIndexOf(36);
               String parentClassName = typeName.substring(0, lastIndex);
               Object ctxObj = context.object;
               String parentName = ctxObj.getClass().getName();
               Object param = null;
               if (!parentName.equals(parentClassName)) {
                  ParseContext parentContext = context.parent;
                  if (parentContext != null && parentContext.object != null && ("java.util.ArrayList".equals(parentName) || "java.util.List".equals(parentName) || "java.util.Collection".equals(parentName) || "java.util.Map".equals(parentName) || "java.util.HashMap".equals(parentName))) {
                     parentName = parentContext.object.getClass().getName();
                     if (parentName.equals(parentClassName)) {
                        param = parentContext.object;
                     }
                  }
               } else {
                  param = ctxObj;
               }

               if (param == null) {
                  throw new JSONException("can't create non-static inner class instance.");
               }

               object = constructor.newInstance(param);
            }
         } catch (JSONException var14) {
            throw var14;
         } catch (Exception var15) {
            throw new JSONException("create instance error, class " + this.clazz.getName(), var15);
         }

         if (parser != null && parser.lexer.isEnabled(Feature.InitStringFieldAsEmpty)) {
            FieldInfo[] var16 = this.beanInfo.fields;
            int var19 = var16.length;

            for(int var21 = 0; var21 < var19; ++var21) {
               FieldInfo fieldInfo = var16[var21];
               if (fieldInfo.fieldClass == String.class) {
                  try {
                     fieldInfo.set(object, "");
                  } catch (Exception var13) {
                     throw new JSONException("create instance error, class " + this.clazz.getName(), var13);
                  }
               }
            }
         }

         return object;
      }
   }

   public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName) {
      return this.deserialze(parser, type, fieldName, 0);
   }

   public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName, int features) {
      return this.deserialze(parser, type, fieldName, (Object)null, features, (int[])null);
   }

   public <T> T deserialzeArrayMapping(DefaultJSONParser parser, Type type, Object fieldName, Object object) {
      JSONLexer lexer = parser.lexer;
      if (lexer.token() != 14) {
         throw new JSONException("error");
      } else {
         object = this.createInstance(parser, type);
         int i = 0;

         for(int size = this.sortedFieldDeserializers.length; i < size; ++i) {
            char seperator = i == size - 1 ? 93 : 44;
            FieldDeserializer fieldDeser = this.sortedFieldDeserializers[i];
            Class<?> fieldClass = fieldDeser.fieldInfo.fieldClass;
            if (fieldClass == Integer.TYPE) {
               int value = lexer.scanInt((char)seperator);
               fieldDeser.setValue(object, value);
            } else if (fieldClass == String.class) {
               String value = lexer.scanString((char)seperator);
               fieldDeser.setValue(object, value);
            } else {
               long longValue;
               if (fieldClass == Long.TYPE) {
                  longValue = lexer.scanLong((char)seperator);
                  fieldDeser.setValue(object, longValue);
               } else if (fieldClass.isEnum()) {
                  char ch = lexer.getCurrent();
                  Enum value;
                  if (ch != '"' && ch != 'n') {
                     if (ch >= '0' && ch <= '9') {
                        int ordinal = lexer.scanInt((char)seperator);
                        EnumDeserializer enumDeser = (EnumDeserializer)((DefaultFieldDeserializer)fieldDeser).getFieldValueDeserilizer(parser.getConfig());
                        value = enumDeser.valueOf(ordinal);
                     } else {
                        value = this.scanEnum(lexer, (char)seperator);
                     }
                  } else {
                     value = lexer.scanEnum(fieldClass, parser.getSymbolTable(), (char)seperator);
                  }

                  fieldDeser.setValue(object, (Object)value);
               } else if (fieldClass == Boolean.TYPE) {
                  boolean value = lexer.scanBoolean((char)seperator);
                  fieldDeser.setValue(object, value);
               } else if (fieldClass == Float.TYPE) {
                  float value = lexer.scanFloat((char)seperator);
                  fieldDeser.setValue(object, (Object)value);
               } else if (fieldClass == Double.TYPE) {
                  double value = lexer.scanDouble((char)seperator);
                  fieldDeser.setValue(object, (Object)value);
               } else if (fieldClass == Date.class && lexer.getCurrent() == '1') {
                  longValue = lexer.scanLong((char)seperator);
                  fieldDeser.setValue(object, (Object)(new Date(longValue)));
               } else if (fieldClass == BigDecimal.class) {
                  BigDecimal value = lexer.scanDecimal((char)seperator);
                  fieldDeser.setValue(object, (Object)value);
               } else {
                  lexer.nextToken(14);
                  Object value = parser.parseObject((Type)fieldDeser.fieldInfo.fieldType, fieldDeser.fieldInfo.name);
                  fieldDeser.setValue(object, value);
                  if (lexer.token() == 15) {
                     break;
                  }

                  this.check(lexer, seperator == 93 ? 15 : 16);
               }
            }
         }

         lexer.nextToken(16);
         return object;
      }
   }

   protected void check(JSONLexer lexer, int token) {
      if (lexer.token() != token) {
         throw new JSONException("syntax error");
      }
   }

   protected Enum<?> scanEnum(JSONLexer lexer, char seperator) {
      throw new JSONException("illegal enum. " + lexer.info());
   }

   protected <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName, Object object, int features, int[] setFlags) {
      if (type != JSON.class && type != JSONObject.class) {
         JSONLexerBase lexer = (JSONLexerBase)parser.lexer;
         ParserConfig config = parser.getConfig();
         int token = lexer.token();
         if (token == 8) {
            lexer.nextToken(16);
            return null;
         } else {
            ParseContext context = parser.getContext();
            if (object != null && context != null) {
               context = context.parent;
            }

            ParseContext childContext = null;

            try {
               Map<String, Object> fieldValues = null;
               if (token == 13) {
                  lexer.nextToken(16);
                  if (object == null) {
                     object = this.createInstance(parser, type);
                  }

                  Object var46 = object;
                  return var46;
               } else {
                  Object builtObj;
                  if (token == 14) {
                     int mask = Feature.SupportArrayToBean.mask;
                     boolean isSupportArrayToBean = (this.beanInfo.parserFeatures & mask) != 0 || lexer.isEnabled(Feature.SupportArrayToBean) || (features & mask) != 0;
                     if (isSupportArrayToBean) {
                        builtObj = this.deserialzeArrayMapping(parser, type, fieldName, object);
                        return builtObj;
                     }
                  }

                  String typeKey;
                  String[] paramNames;
                  int i;
                  if (token != 12 && token != 16) {
                     if (!lexer.isBlankInput()) {
                        if (token != 4) {
                           if (token == 5) {
                              Calendar var43 = lexer.getCalendar();
                           }
                        } else {
                           typeKey = lexer.stringVal();
                           if (typeKey.length() == 0) {
                              lexer.nextToken();
                              paramNames = null;
                              return paramNames;
                           }

                           if (this.beanInfo.jsonType != null) {
                              Class[] var49 = this.beanInfo.jsonType.seeAlso();
                              int var54 = var49.length;

                              for(i = 0; i < var54; ++i) {
                                 Class<?> seeAlsoClass = var49[i];
                                 if (Enum.class.isAssignableFrom(seeAlsoClass)) {
                                    try {
                                       Enum<?> e = Enum.valueOf(seeAlsoClass, typeKey);
                                       Enum var65 = e;
                                       return var65;
                                    } catch (IllegalArgumentException var39) {
                                    }
                                 }
                              }
                           }
                        }

                        if (token == 14 && lexer.getCurrent() == ']') {
                           lexer.next();
                           lexer.nextToken();
                           typeKey = null;
                           return typeKey;
                        } else {
                           StringBuffer buf = (new StringBuffer()).append("syntax error, expect {, actual ").append(lexer.tokenName()).append(", pos ").append(lexer.pos());
                           if (fieldName instanceof String) {
                              buf.append(", fieldName ").append(fieldName);
                           }

                           buf.append(", fastjson-version ").append("1.2.47");
                           throw new JSONException(buf.toString());
                        }
                     } else {
                        typeKey = null;
                        return typeKey;
                     }
                  } else {
                     if (parser.resolveStatus == 2) {
                        parser.resolveStatus = 0;
                     }

                     typeKey = this.beanInfo.typeKey;
                     int fieldIndex = 0;

                     while(true) {
                        String key = null;
                        FieldDeserializer fieldDeser = null;
                        FieldInfo fieldInfo = null;
                        Class<?> fieldClass = null;
                        JSONField feildAnnotation = null;
                        boolean customDeserilizer = false;
                        if (fieldIndex < this.sortedFieldDeserializers.length) {
                           fieldDeser = this.sortedFieldDeserializers[fieldIndex];
                           fieldInfo = fieldDeser.fieldInfo;
                           fieldClass = fieldInfo.fieldClass;
                           feildAnnotation = fieldInfo.getAnnotation();
                           if (feildAnnotation != null && fieldDeser instanceof DefaultFieldDeserializer) {
                              customDeserilizer = ((DefaultFieldDeserializer)fieldDeser).customDeserilizer;
                           }
                        }

                        label1376: {
                           boolean matchField = false;
                           boolean valueParsed = false;
                           Object fieldValue = null;
                           if (fieldDeser != null) {
                              char[] name_chars = fieldInfo.name_chars;
                              if (customDeserilizer && lexer.matchField(name_chars)) {
                                 matchField = true;
                              } else if (fieldClass != Integer.TYPE && fieldClass != Integer.class) {
                                 if (fieldClass != Long.TYPE && fieldClass != Long.class) {
                                    if (fieldClass == String.class) {
                                       fieldValue = lexer.scanFieldString(name_chars);
                                       if (lexer.matchStat > 0) {
                                          matchField = true;
                                          valueParsed = true;
                                       } else if (lexer.matchStat == -2) {
                                          break label1376;
                                       }
                                    } else if (fieldClass == Date.class && fieldInfo.format == null) {
                                       fieldValue = lexer.scanFieldDate(name_chars);
                                       if (lexer.matchStat > 0) {
                                          matchField = true;
                                          valueParsed = true;
                                       } else if (lexer.matchStat == -2) {
                                          break label1376;
                                       }
                                    } else if (fieldClass == BigDecimal.class) {
                                       fieldValue = lexer.scanFieldDecimal(name_chars);
                                       if (lexer.matchStat > 0) {
                                          matchField = true;
                                          valueParsed = true;
                                       } else if (lexer.matchStat == -2) {
                                          break label1376;
                                       }
                                    } else if (fieldClass == BigInteger.class) {
                                       fieldValue = lexer.scanFieldBigInteger(name_chars);
                                       if (lexer.matchStat > 0) {
                                          matchField = true;
                                          valueParsed = true;
                                       } else if (lexer.matchStat == -2) {
                                          break label1376;
                                       }
                                    } else if (fieldClass != Boolean.TYPE && fieldClass != Boolean.class) {
                                       if (fieldClass != Float.TYPE && fieldClass != Float.class) {
                                          if (fieldClass != Double.TYPE && fieldClass != Double.class) {
                                             if (fieldClass.isEnum() && parser.getConfig().getDeserializer((Type)fieldClass) instanceof EnumDeserializer && (feildAnnotation == null || feildAnnotation.deserializeUsing() == Void.class)) {
                                                if (fieldDeser instanceof DefaultFieldDeserializer) {
                                                   ObjectDeserializer fieldValueDeserilizer = ((DefaultFieldDeserializer)fieldDeser).fieldValueDeserilizer;
                                                   fieldValue = this.scanEnum(lexer, name_chars, fieldValueDeserilizer);
                                                   if (lexer.matchStat > 0) {
                                                      matchField = true;
                                                      valueParsed = true;
                                                   } else if (lexer.matchStat == -2) {
                                                      break label1376;
                                                   }
                                                }
                                             } else if (fieldClass == int[].class) {
                                                fieldValue = lexer.scanFieldIntArray(name_chars);
                                                if (lexer.matchStat > 0) {
                                                   matchField = true;
                                                   valueParsed = true;
                                                } else if (lexer.matchStat == -2) {
                                                   break label1376;
                                                }
                                             } else if (fieldClass == float[].class) {
                                                fieldValue = lexer.scanFieldFloatArray(name_chars);
                                                if (lexer.matchStat > 0) {
                                                   matchField = true;
                                                   valueParsed = true;
                                                } else if (lexer.matchStat == -2) {
                                                   break label1376;
                                                }
                                             } else if (fieldClass == float[][].class) {
                                                fieldValue = lexer.scanFieldFloatArray2(name_chars);
                                                if (lexer.matchStat > 0) {
                                                   matchField = true;
                                                   valueParsed = true;
                                                } else if (lexer.matchStat == -2) {
                                                   break label1376;
                                                }
                                             } else {
                                                if (!lexer.matchField(name_chars)) {
                                                   break label1376;
                                                }

                                                matchField = true;
                                             }
                                          } else {
                                             fieldValue = lexer.scanFieldDouble(name_chars);
                                             if (lexer.matchStat > 0) {
                                                matchField = true;
                                                valueParsed = true;
                                             } else if (lexer.matchStat == -2) {
                                                break label1376;
                                             }
                                          }
                                       } else {
                                          fieldValue = lexer.scanFieldFloat(name_chars);
                                          if (lexer.matchStat > 0) {
                                             matchField = true;
                                             valueParsed = true;
                                          } else if (lexer.matchStat == -2) {
                                             break label1376;
                                          }
                                       }
                                    } else {
                                       fieldValue = lexer.scanFieldBoolean(name_chars);
                                       if (lexer.matchStat > 0) {
                                          matchField = true;
                                          valueParsed = true;
                                       } else if (lexer.matchStat == -2) {
                                          break label1376;
                                       }
                                    }
                                 } else {
                                    fieldValue = lexer.scanFieldLong(name_chars);
                                    if (lexer.matchStat > 0) {
                                       matchField = true;
                                       valueParsed = true;
                                    } else if (lexer.matchStat == -2) {
                                       break label1376;
                                    }
                                 }
                              } else {
                                 fieldValue = lexer.scanFieldInt(name_chars);
                                 if (lexer.matchStat > 0) {
                                    matchField = true;
                                    valueParsed = true;
                                 } else if (lexer.matchStat == -2) {
                                    break label1376;
                                 }
                              }
                           }

                           if (!matchField) {
                              key = lexer.scanSymbol(parser.symbolTable);
                              if (key == null) {
                                 token = lexer.token();
                                 if (token == 13) {
                                    lexer.nextToken(16);
                                    break;
                                 }

                                 if (token == 16 && lexer.isEnabled(Feature.AllowArbitraryCommas)) {
                                    break label1376;
                                 }
                              }

                              String ref;
                              Object refObj;
                              if ("$ref" == key && context != null) {
                                 lexer.nextTokenWithColon(4);
                                 token = lexer.token();
                                 if (token != 4) {
                                    throw new JSONException("illegal ref, " + JSONToken.name(token));
                                 }

                                 ref = lexer.stringVal();
                                 if ("@".equals(ref)) {
                                    object = context.object;
                                 } else {
                                    ParseContext rootContext;
                                    if ("..".equals(ref)) {
                                       rootContext = context.parent;
                                       if (rootContext.object != null) {
                                          object = rootContext.object;
                                       } else {
                                          parser.addResolveTask(new DefaultJSONParser.ResolveTask(rootContext, ref));
                                          parser.resolveStatus = 1;
                                       }
                                    } else if ("$".equals(ref)) {
                                       for(rootContext = context; rootContext.parent != null; rootContext = rootContext.parent) {
                                       }

                                       if (rootContext.object != null) {
                                          object = rootContext.object;
                                       } else {
                                          parser.addResolveTask(new DefaultJSONParser.ResolveTask(rootContext, ref));
                                          parser.resolveStatus = 1;
                                       }
                                    } else {
                                       if (ref.indexOf(92) > 0) {
                                          StringBuilder buf = new StringBuilder();

                                          for(int i = 0; i < ref.length(); ++i) {
                                             char ch = ref.charAt(i);
                                             if (ch == '\\') {
                                                ++i;
                                                ch = ref.charAt(i);
                                             }

                                             buf.append(ch);
                                          }

                                          ref = buf.toString();
                                       }

                                       refObj = parser.resolveReference(ref);
                                       if (refObj != null) {
                                          object = refObj;
                                       } else {
                                          parser.addResolveTask(new DefaultJSONParser.ResolveTask(context, ref));
                                          parser.resolveStatus = 1;
                                       }
                                    }
                                 }

                                 lexer.nextToken(13);
                                 if (lexer.token() != 13) {
                                    throw new JSONException("illegal ref");
                                 }

                                 lexer.nextToken(16);
                                 parser.setContext(context, object, fieldName);
                                 Object var75 = object;
                                 return var75;
                              }

                              if (typeKey != null && typeKey.equals(key) || JSON.DEFAULT_TYPE_KEY == key) {
                                 lexer.nextTokenWithColon(4);
                                 if (lexer.token() != 4) {
                                    throw new JSONException("syntax error");
                                 }

                                 ref = lexer.stringVal();
                                 lexer.nextToken(16);
                                 if (!ref.equals(this.beanInfo.typeName) && !parser.isEnabled(Feature.IgnoreAutoType)) {
                                    refObj = this.getSeeAlso(config, this.beanInfo, ref);
                                    Class<?> userType = null;
                                    if (refObj == null) {
                                       Class<?> expectClass = TypeUtils.getClass(type);
                                       userType = config.checkAutoType(ref, expectClass, lexer.getFeatures());
                                       refObj = parser.getConfig().getDeserializer((Type)userType);
                                    }

                                    Object typedObject = ((ObjectDeserializer)refObj).deserialze(parser, userType, fieldName);
                                    if (refObj instanceof JavaBeanDeserializer) {
                                       JavaBeanDeserializer javaBeanDeserializer = (JavaBeanDeserializer)refObj;
                                       if (typeKey != null) {
                                          FieldDeserializer typeKeyFieldDeser = javaBeanDeserializer.getFieldDeserializer(typeKey);
                                          typeKeyFieldDeser.setValue(typedObject, ref);
                                       }
                                    }

                                    Object var82 = typedObject;
                                    return var82;
                                 }

                                 if (lexer.token() == 13) {
                                    lexer.nextToken();
                                    break;
                                 }
                                 break label1376;
                              }
                           }

                           if (object == null && fieldValues == null) {
                              object = this.createInstance(parser, type);
                              if (object == null) {
                                 fieldValues = new HashMap(this.fieldDeserializers.length);
                              }

                              childContext = parser.setContext(context, object, fieldName);
                              if (setFlags == null) {
                                 setFlags = new int[this.fieldDeserializers.length / 32 + 1];
                              }
                           }

                           if (matchField) {
                              if (!valueParsed) {
                                 fieldDeser.parseField(parser, object, type, fieldValues);
                              } else {
                                 if (object == null) {
                                    fieldValues.put(fieldInfo.name, fieldValue);
                                 } else if (fieldValue == null) {
                                    if (fieldClass != Integer.TYPE && fieldClass != Long.TYPE && fieldClass != Float.TYPE && fieldClass != Double.TYPE && fieldClass != Boolean.TYPE) {
                                       fieldDeser.setValue(object, fieldValue);
                                    }
                                 } else {
                                    fieldDeser.setValue(object, fieldValue);
                                 }

                                 if (setFlags != null) {
                                    int flagIndex = fieldIndex / 32;
                                    int bitIndex = fieldIndex % 32;
                                    setFlags[flagIndex] |= 1 >> bitIndex;
                                 }

                                 if (lexer.matchStat == 4) {
                                    break;
                                 }
                              }
                           } else {
                              boolean match = this.parseField(parser, key, object, type, fieldValues, setFlags);
                              if (!match) {
                                 if (lexer.token() == 13) {
                                    lexer.nextToken();
                                    break;
                                 }
                                 break label1376;
                              }

                              if (lexer.token() == 17) {
                                 throw new JSONException("syntax error, unexpect token ':'");
                              }
                           }

                           if (lexer.token() != 16) {
                              if (lexer.token() == 13) {
                                 lexer.nextToken(16);
                                 break;
                              }

                              if (lexer.token() == 18 || lexer.token() == 1) {
                                 throw new JSONException("syntax error, unexpect token " + JSONToken.name(lexer.token()));
                              }
                           }
                        }

                        ++fieldIndex;
                     }

                     if (object == null) {
                        if (fieldValues == null) {
                           object = this.createInstance(parser, type);
                           if (childContext == null) {
                              childContext = parser.setContext(context, object, fieldName);
                           }

                           Object var48 = object;
                           return var48;
                        }

                        paramNames = this.beanInfo.creatorConstructorParameters;
                        Object[] params;
                        if (paramNames != null) {
                           params = new Object[paramNames.length];

                           for(i = 0; i < paramNames.length; ++i) {
                              String paramName = paramNames[i];
                              Object param = fieldValues.remove(paramName);
                              if (param == null) {
                                 Type fieldType = this.beanInfo.creatorConstructorParameterTypes[i];
                                 FieldInfo fieldInfo = this.beanInfo.fields[i];
                                 if (fieldType == Byte.TYPE) {
                                    param = 0;
                                 } else if (fieldType == Short.TYPE) {
                                    param = Short.valueOf((short)0);
                                 } else if (fieldType == Integer.TYPE) {
                                    param = 0;
                                 } else if (fieldType == Long.TYPE) {
                                    param = 0L;
                                 } else if (fieldType == Float.TYPE) {
                                    param = 0.0F;
                                 } else if (fieldType == Double.TYPE) {
                                    param = 0.0D;
                                 } else if (fieldType == Boolean.TYPE) {
                                    param = Boolean.FALSE;
                                 } else if (fieldType == String.class && (fieldInfo.parserFeatures & Feature.InitStringFieldAsEmpty.mask) != 0) {
                                    param = "";
                                 }
                              }

                              params[i] = param;
                           }
                        } else {
                           FieldInfo[] fieldInfoList = this.beanInfo.fields;
                           int size = fieldInfoList.length;
                           params = new Object[size];

                           for(int i = 0; i < size; ++i) {
                              FieldInfo fieldInfo = fieldInfoList[i];
                              Object param = fieldValues.get(fieldInfo.name);
                              if (param == null) {
                                 Type fieldType = fieldInfo.fieldType;
                                 if (fieldType == Byte.TYPE) {
                                    param = 0;
                                 } else if (fieldType == Short.TYPE) {
                                    param = Short.valueOf((short)0);
                                 } else if (fieldType == Integer.TYPE) {
                                    param = 0;
                                 } else if (fieldType == Long.TYPE) {
                                    param = 0L;
                                 } else if (fieldType == Float.TYPE) {
                                    param = 0.0F;
                                 } else if (fieldType == Double.TYPE) {
                                    param = 0.0D;
                                 } else if (fieldType == Boolean.TYPE) {
                                    param = Boolean.FALSE;
                                 } else if (fieldType == String.class && (fieldInfo.parserFeatures & Feature.InitStringFieldAsEmpty.mask) != 0) {
                                    param = "";
                                 }
                              }

                              params[i] = param;
                           }
                        }

                        if (this.beanInfo.creatorConstructor != null) {
                           try {
                              object = this.beanInfo.creatorConstructor.newInstance(params);
                           } catch (Exception var38) {
                              throw new JSONException("create instance error, " + paramNames + ", " + this.beanInfo.creatorConstructor.toGenericString(), var38);
                           }

                           if (paramNames != null) {
                              Iterator var55 = fieldValues.entrySet().iterator();

                              while(var55.hasNext()) {
                                 Entry<String, Object> entry = (Entry)var55.next();
                                 FieldDeserializer fieldDeserializer = this.getFieldDeserializer((String)entry.getKey());
                                 if (fieldDeserializer != null) {
                                    fieldDeserializer.setValue(object, entry.getValue());
                                 }
                              }
                           }
                        } else if (this.beanInfo.factoryMethod != null) {
                           try {
                              object = this.beanInfo.factoryMethod.invoke((Object)null, params);
                           } catch (Exception var37) {
                              throw new JSONException("create factory method error, " + this.beanInfo.factoryMethod.toString(), var37);
                           }
                        }

                        childContext.object = object;
                     }

                     Method buildMethod = this.beanInfo.buildMethod;
                     if (buildMethod == null) {
                        builtObj = object;
                        return builtObj;
                     } else {
                        try {
                           builtObj = buildMethod.invoke(object);
                        } catch (Exception var36) {
                           throw new JSONException("build object error", var36);
                        }

                        Object var56 = builtObj;
                        return var56;
                     }
                  }
               }
            } finally {
               if (childContext != null) {
                  childContext.object = object;
               }

               parser.setContext(context);
            }
         }
      } else {
         return parser.parse();
      }
   }

   protected Enum scanEnum(JSONLexerBase lexer, char[] name_chars, ObjectDeserializer fieldValueDeserilizer) {
      EnumDeserializer enumDeserializer = null;
      if (fieldValueDeserilizer instanceof EnumDeserializer) {
         enumDeserializer = (EnumDeserializer)fieldValueDeserilizer;
      }

      if (enumDeserializer == null) {
         lexer.matchStat = -1;
         return null;
      } else {
         long enumNameHashCode = lexer.scanFieldSymbol(name_chars);
         return lexer.matchStat > 0 ? enumDeserializer.getEnumByHashCode(enumNameHashCode) : null;
      }
   }

   public boolean parseField(DefaultJSONParser parser, String key, Object object, Type objectType, Map<String, Object> fieldValues) {
      return this.parseField(parser, key, object, objectType, fieldValues, (int[])null);
   }

   public boolean parseField(DefaultJSONParser parser, String key, Object object, Type objectType, Map<String, Object> fieldValues, int[] setFlags) {
      JSONLexer lexer = parser.lexer;
      int disableFieldSmartMatchMask = Feature.DisableFieldSmartMatch.mask;
      Object fieldDeserializer;
      if (!lexer.isEnabled(disableFieldSmartMatchMask) && (this.beanInfo.parserFeatures & disableFieldSmartMatchMask) == 0) {
         fieldDeserializer = this.smartMatch(key, setFlags);
      } else {
         fieldDeserializer = this.getFieldDeserializer(key);
      }

      int mask = Feature.SupportNonPublicField.mask;
      if (fieldDeserializer == null && (lexer.isEnabled(mask) || (this.beanInfo.parserFeatures & mask) != 0)) {
         if (this.extraFieldDeserializers == null) {
            ConcurrentHashMap extraFieldDeserializers = new ConcurrentHashMap(1, 0.75F, 1);

            for(Class c = this.clazz; c != null && c != Object.class; c = c.getSuperclass()) {
               Field[] fields = c.getDeclaredFields();
               Field[] var14 = fields;
               int var15 = fields.length;

               for(int var16 = 0; var16 < var15; ++var16) {
                  Field field = var14[var16];
                  String fieldName = field.getName();
                  if (this.getFieldDeserializer(fieldName) == null) {
                     int fieldModifiers = field.getModifiers();
                     if ((fieldModifiers & 16) == 0 && (fieldModifiers & 8) == 0) {
                        extraFieldDeserializers.put(fieldName, field);
                     }
                  }
               }
            }

            this.extraFieldDeserializers = extraFieldDeserializers;
         }

         Object deserOrField = this.extraFieldDeserializers.get(key);
         if (deserOrField != null) {
            if (deserOrField instanceof FieldDeserializer) {
               fieldDeserializer = (FieldDeserializer)deserOrField;
            } else {
               Field field = (Field)deserOrField;
               field.setAccessible(true);
               FieldInfo fieldInfo = new FieldInfo(key, field.getDeclaringClass(), field.getType(), field.getGenericType(), field, 0, 0, 0);
               fieldDeserializer = new DefaultFieldDeserializer(parser.getConfig(), this.clazz, fieldInfo);
               this.extraFieldDeserializers.put(key, fieldDeserializer);
            }
         }
      }

      int i;
      if (fieldDeserializer == null) {
         if (!lexer.isEnabled(Feature.IgnoreNotMatch)) {
            throw new JSONException("setter not found, class " + this.clazz.getName() + ", property " + key);
         } else {
            FieldDeserializer[] var27 = this.sortedFieldDeserializers;
            i = var27.length;

            for(int var31 = 0; var31 < i; ++var31) {
               FieldDeserializer fieldDeser = var27[var31];
               FieldInfo fieldInfo = fieldDeser.fieldInfo;
               if (fieldInfo.unwrapped && fieldDeser instanceof DefaultFieldDeserializer) {
                  if (fieldInfo.field != null) {
                     DefaultFieldDeserializer defaultFieldDeserializer = (DefaultFieldDeserializer)fieldDeser;
                     ObjectDeserializer fieldValueDeser = defaultFieldDeserializer.getFieldValueDeserilizer(parser.getConfig());
                     Object fieldValue;
                     if (fieldValueDeser instanceof JavaBeanDeserializer) {
                        JavaBeanDeserializer javaBeanFieldValueDeserializer = (JavaBeanDeserializer)fieldValueDeser;
                        FieldDeserializer unwrappedFieldDeser = javaBeanFieldValueDeserializer.getFieldDeserializer(key);
                        if (unwrappedFieldDeser != null) {
                           try {
                              fieldValue = fieldInfo.field.get(object);
                              if (fieldValue == null) {
                                 fieldValue = ((JavaBeanDeserializer)fieldValueDeser).createInstance(parser, fieldInfo.fieldType);
                                 fieldDeser.setValue(object, fieldValue);
                              }

                              lexer.nextTokenWithColon(defaultFieldDeserializer.getFastMatchToken());
                              unwrappedFieldDeser.parseField(parser, fieldValue, objectType, fieldValues);
                              return true;
                           } catch (Exception var22) {
                              throw new JSONException("parse unwrapped field error.", var22);
                           }
                        }
                     } else if (fieldValueDeser instanceof MapDeserializer) {
                        MapDeserializer javaBeanFieldValueDeserializer = (MapDeserializer)fieldValueDeser;

                        try {
                           Map fieldObject = (Map)fieldInfo.field.get(object);
                           if (fieldObject == null) {
                              fieldObject = javaBeanFieldValueDeserializer.createMap(fieldInfo.fieldType);
                              fieldDeser.setValue(object, (Object)fieldObject);
                           }

                           lexer.nextTokenWithColon();
                           fieldValue = parser.parse(key);
                           fieldObject.put(key, fieldValue);
                           return true;
                        } catch (Exception var23) {
                           throw new JSONException("parse unwrapped field error.", var23);
                        }
                     }
                  } else if (fieldInfo.method.getParameterTypes().length == 2) {
                     lexer.nextTokenWithColon();
                     Object fieldValue = parser.parse(key);

                     try {
                        fieldInfo.method.invoke(object, key, fieldValue);
                        return true;
                     } catch (Exception var24) {
                        throw new JSONException("parse unwrapped field error.", var24);
                     }
                  }
               }
            }

            parser.parseExtra(object, key);
            return false;
         }
      } else {
         int fieldIndex = -1;

         for(i = 0; i < this.sortedFieldDeserializers.length; ++i) {
            if (this.sortedFieldDeserializers[i] == fieldDeserializer) {
               fieldIndex = i;
               break;
            }
         }

         if (fieldIndex != -1 && setFlags != null && key.startsWith("_") && isSetFlag(fieldIndex, setFlags)) {
            parser.parseExtra(object, key);
            return false;
         } else {
            lexer.nextTokenWithColon(((FieldDeserializer)fieldDeserializer).getFastMatchToken());
            ((FieldDeserializer)fieldDeserializer).parseField(parser, object, objectType, fieldValues);
            return true;
         }
      }
   }

   public FieldDeserializer smartMatch(String key) {
      return this.smartMatch(key, (int[])null);
   }

   public FieldDeserializer smartMatch(String key, int[] setFlags) {
      if (key == null) {
         return null;
      } else {
         FieldDeserializer fieldDeserializer = this.getFieldDeserializer(key, setFlags);
         if (fieldDeserializer == null) {
            long smartKeyHash = TypeUtils.fnv1a_64_lower(key);
            if (this.smartMatchHashArray == null) {
               long[] hashArray = new long[this.sortedFieldDeserializers.length];

               for(int i = 0; i < this.sortedFieldDeserializers.length; ++i) {
                  hashArray[i] = TypeUtils.fnv1a_64_lower(this.sortedFieldDeserializers[i].fieldInfo.name);
               }

               Arrays.sort(hashArray);
               this.smartMatchHashArray = hashArray;
            }

            int pos = Arrays.binarySearch(this.smartMatchHashArray, smartKeyHash);
            boolean is = false;
            if (pos < 0 && (is = key.startsWith("is"))) {
               smartKeyHash = TypeUtils.fnv1a_64_lower(key.substring(2));
               pos = Arrays.binarySearch(this.smartMatchHashArray, smartKeyHash);
            }

            if (pos >= 0) {
               if (this.smartMatchHashArrayMapping == null) {
                  short[] mapping = new short[this.smartMatchHashArray.length];
                  Arrays.fill(mapping, (short)-1);

                  for(int i = 0; i < this.sortedFieldDeserializers.length; ++i) {
                     int p = Arrays.binarySearch(this.smartMatchHashArray, TypeUtils.fnv1a_64_lower(this.sortedFieldDeserializers[i].fieldInfo.name));
                     if (p >= 0) {
                        mapping[p] = (short)i;
                     }
                  }

                  this.smartMatchHashArrayMapping = mapping;
               }

               int deserIndex = this.smartMatchHashArrayMapping[pos];
               if (deserIndex != -1 && !isSetFlag(deserIndex, setFlags)) {
                  fieldDeserializer = this.sortedFieldDeserializers[deserIndex];
               }
            }

            if (fieldDeserializer != null) {
               FieldInfo fieldInfo = fieldDeserializer.fieldInfo;
               if ((fieldInfo.parserFeatures & Feature.DisableFieldSmartMatch.mask) != 0) {
                  return null;
               }

               Class fieldClass = fieldInfo.fieldClass;
               if (is && fieldClass != Boolean.TYPE && fieldClass != Boolean.class) {
                  fieldDeserializer = null;
               }
            }
         }

         return fieldDeserializer;
      }
   }

   public int getFastMatchToken() {
      return 12;
   }

   public Object createInstance(Map<String, Object> map, ParserConfig config) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException {
      Object object = null;
      FieldInfo fieldInfo;
      if (this.beanInfo.creatorConstructor == null && this.beanInfo.factoryMethod == null) {
         object = this.createInstance((DefaultJSONParser)null, (Type)this.clazz);
         Iterator var17 = map.entrySet().iterator();

         while(true) {
            Object value;
            FieldDeserializer fieldDeser;
            do {
               if (!var17.hasNext()) {
                  if (this.beanInfo.buildMethod != null) {
                     try {
                        Object builtObj = this.beanInfo.buildMethod.invoke(object);
                        return builtObj;
                     } catch (Exception var14) {
                        throw new JSONException("build object error", var14);
                     }
                  }

                  return object;
               }

               Entry<String, Object> entry = (Entry)var17.next();
               String key = (String)entry.getKey();
               value = entry.getValue();
               fieldDeser = this.smartMatch(key);
            } while(fieldDeser == null);

            fieldInfo = fieldDeser.fieldInfo;
            Type paramType = fieldInfo.fieldType;
            String format = fieldInfo.format;
            if (format != null && paramType == Date.class) {
               value = TypeUtils.castToDate(value, format);
            } else {
               value = TypeUtils.cast(value, paramType, config);
            }

            fieldDeser.setValue(object, value);
         }
      } else {
         FieldInfo[] fieldInfoList = this.beanInfo.fields;
         int size = fieldInfoList.length;
         Object[] params = new Object[size];
         Map<String, Integer> missFields = null;

         for(int i = 0; i < size; ++i) {
            fieldInfo = fieldInfoList[i];
            Object param = map.get(fieldInfo.name);
            if (param == null) {
               Class<?> fieldClass = fieldInfo.fieldClass;
               if (fieldClass == Integer.TYPE) {
                  param = 0;
               } else if (fieldClass == Long.TYPE) {
                  param = 0L;
               } else if (fieldClass == Short.TYPE) {
                  param = Short.valueOf((short)0);
               } else if (fieldClass == Byte.TYPE) {
                  param = 0;
               } else if (fieldClass == Float.TYPE) {
                  param = 0.0F;
               } else if (fieldClass == Double.TYPE) {
                  param = 0.0D;
               } else if (fieldClass == Character.TYPE) {
                  param = '0';
               } else if (fieldClass == Boolean.TYPE) {
                  param = false;
               }

               if (missFields == null) {
                  missFields = new HashMap();
               }

               missFields.put(fieldInfo.name, i);
            }

            params[i] = param;
         }

         if (missFields != null) {
            Iterator var22 = map.entrySet().iterator();

            while(var22.hasNext()) {
               Entry<String, Object> entry = (Entry)var22.next();
               String key = (String)entry.getKey();
               Object value = entry.getValue();
               FieldDeserializer fieldDeser = this.smartMatch(key);
               if (fieldDeser != null) {
                  Integer index = (Integer)missFields.get(fieldDeser.fieldInfo.name);
                  if (index != null) {
                     params[index] = value;
                  }
               }
            }
         }

         if (this.beanInfo.creatorConstructor != null) {
            try {
               object = this.beanInfo.creatorConstructor.newInstance(params);
            } catch (Exception var16) {
               throw new JSONException("create instance error, " + this.beanInfo.creatorConstructor.toGenericString(), var16);
            }
         } else if (this.beanInfo.factoryMethod != null) {
            try {
               object = this.beanInfo.factoryMethod.invoke((Object)null, params);
            } catch (Exception var15) {
               throw new JSONException("create factory method error, " + this.beanInfo.factoryMethod.toString(), var15);
            }
         }

         return object;
      }
   }

   public Type getFieldType(int ordinal) {
      return this.sortedFieldDeserializers[ordinal].fieldInfo.fieldType;
   }

   protected Object parseRest(DefaultJSONParser parser, Type type, Object fieldName, Object instance, int features) {
      return this.parseRest(parser, type, fieldName, instance, features, new int[0]);
   }

   protected Object parseRest(DefaultJSONParser parser, Type type, Object fieldName, Object instance, int features, int[] setFlags) {
      Object value = this.deserialze(parser, type, fieldName, instance, features, setFlags);
      return value;
   }

   protected JavaBeanDeserializer getSeeAlso(ParserConfig config, JavaBeanInfo beanInfo, String typeName) {
      if (beanInfo.jsonType == null) {
         return null;
      } else {
         Class[] var4 = beanInfo.jsonType.seeAlso();
         int var5 = var4.length;

         for(int var6 = 0; var6 < var5; ++var6) {
            Class<?> seeAlsoClass = var4[var6];
            ObjectDeserializer seeAlsoDeser = config.getDeserializer((Type)seeAlsoClass);
            if (seeAlsoDeser instanceof JavaBeanDeserializer) {
               JavaBeanDeserializer seeAlsoJavaBeanDeser = (JavaBeanDeserializer)seeAlsoDeser;
               JavaBeanInfo subBeanInfo = seeAlsoJavaBeanDeser.beanInfo;
               if (subBeanInfo.typeName.equals(typeName)) {
                  return seeAlsoJavaBeanDeser;
               }

               JavaBeanDeserializer subSeeAlso = this.getSeeAlso(config, subBeanInfo, typeName);
               if (subSeeAlso != null) {
                  return subSeeAlso;
               }
            }
         }

         return null;
      }
   }

   protected static void parseArray(Collection collection, ObjectDeserializer deser, DefaultJSONParser parser, Type type, Object fieldName) {
      JSONLexerBase lexer = (JSONLexerBase)parser.lexer;
      int token = lexer.token();
      if (token == 8) {
         lexer.nextToken(16);
         token = lexer.token();
      } else {
         if (token != 14) {
            parser.throwException(token);
         }

         char ch = lexer.getCurrent();
         if (ch == '[') {
            lexer.next();
            lexer.setToken(14);
         } else {
            lexer.nextToken(14);
         }

         if (lexer.token() == 15) {
            lexer.nextToken();
         } else {
            int index = 0;

            while(true) {
               Object item = deser.deserialze(parser, type, index);
               collection.add(item);
               ++index;
               if (lexer.token() != 16) {
                  token = lexer.token();
                  if (token != 15) {
                     parser.throwException(token);
                  }

                  ch = lexer.getCurrent();
                  if (ch == ',') {
                     lexer.next();
                     lexer.setToken(16);
                  } else {
                     lexer.nextToken(16);
                  }

                  return;
               }

               ch = lexer.getCurrent();
               if (ch == '[') {
                  lexer.next();
                  lexer.setToken(14);
               } else {
                  lexer.nextToken(14);
               }
            }
         }
      }
   }
}
