package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.util.IOUtils;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.Type;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.TimeZone;
import java.util.zip.GZIPOutputStream;

public class JSONSerializer extends SerializeFilterable {
   protected final SerializeConfig config;
   public final SerializeWriter out;
   private int indentCount;
   private String indent;
   private String dateFormatPattern;
   private DateFormat dateFormat;
   protected IdentityHashMap<Object, SerialContext> references;
   protected SerialContext context;
   protected TimeZone timeZone;
   protected Locale locale;

   public JSONSerializer() {
      this(new SerializeWriter(), SerializeConfig.getGlobalInstance());
   }

   public JSONSerializer(SerializeWriter out) {
      this(out, SerializeConfig.getGlobalInstance());
   }

   public JSONSerializer(SerializeConfig config) {
      this(new SerializeWriter(), config);
   }

   public JSONSerializer(SerializeWriter out, SerializeConfig config) {
      this.indentCount = 0;
      this.indent = "\t";
      this.references = null;
      this.timeZone = JSON.defaultTimeZone;
      this.locale = JSON.defaultLocale;
      this.out = out;
      this.config = config;
   }

   public String getDateFormatPattern() {
      return this.dateFormat instanceof SimpleDateFormat ? ((SimpleDateFormat)this.dateFormat).toPattern() : this.dateFormatPattern;
   }

   public DateFormat getDateFormat() {
      if (this.dateFormat == null && this.dateFormatPattern != null) {
         this.dateFormat = new SimpleDateFormat(this.dateFormatPattern, this.locale);
         this.dateFormat.setTimeZone(this.timeZone);
      }

      return this.dateFormat;
   }

   public void setDateFormat(DateFormat dateFormat) {
      this.dateFormat = dateFormat;
      if (this.dateFormatPattern != null) {
         this.dateFormatPattern = null;
      }

   }

   public void setDateFormat(String dateFormat) {
      this.dateFormatPattern = dateFormat;
      if (this.dateFormat != null) {
         this.dateFormat = null;
      }

   }

   public SerialContext getContext() {
      return this.context;
   }

   public void setContext(SerialContext context) {
      this.context = context;
   }

   public void setContext(SerialContext parent, Object object, Object fieldName, int features) {
      this.setContext(parent, object, fieldName, features, 0);
   }

   public void setContext(SerialContext parent, Object object, Object fieldName, int features, int fieldFeatures) {
      if (!this.out.disableCircularReferenceDetect) {
         this.context = new SerialContext(parent, object, fieldName, features, fieldFeatures);
         if (this.references == null) {
            this.references = new IdentityHashMap();
         }

         this.references.put(object, this.context);
      }
   }

   public void setContext(Object object, Object fieldName) {
      this.setContext(this.context, object, fieldName, 0);
   }

   public void popContext() {
      if (this.context != null) {
         this.context = this.context.parent;
      }

   }

   public final boolean isWriteClassName(Type fieldType, Object obj) {
      return this.out.isEnabled(SerializerFeature.WriteClassName) && (fieldType != null || !this.out.isEnabled(SerializerFeature.NotWriteRootClassName) || this.context != null && this.context.parent != null);
   }

   public boolean containsReference(Object value) {
      if (this.references == null) {
         return false;
      } else {
         SerialContext refContext = (SerialContext)this.references.get(value);
         if (refContext == null) {
            return false;
         } else {
            Object fieldName = refContext.fieldName;
            return fieldName == null || fieldName instanceof Integer || fieldName instanceof String;
         }
      }
   }

   public void writeReference(Object object) {
      SerialContext context = this.context;
      Object current = context.object;
      if (object == current) {
         this.out.write("{\"$ref\":\"@\"}");
      } else {
         SerialContext parentContext = context.parent;
         if (parentContext != null && object == parentContext.object) {
            this.out.write("{\"$ref\":\"..\"}");
         } else {
            SerialContext rootContext;
            for(rootContext = context; rootContext.parent != null; rootContext = rootContext.parent) {
            }

            if (object == rootContext.object) {
               this.out.write("{\"$ref\":\"$\"}");
            } else {
               this.out.write("{\"$ref\":\"");
               String path = ((SerialContext)this.references.get(object)).toString();
               this.out.write(path);
               this.out.write("\"}");
            }

         }
      }
   }

   public boolean checkValue(SerializeFilterable filterable) {
      return this.valueFilters != null && this.valueFilters.size() > 0 || this.contextValueFilters != null && this.contextValueFilters.size() > 0 || filterable.valueFilters != null && filterable.valueFilters.size() > 0 || filterable.contextValueFilters != null && filterable.contextValueFilters.size() > 0 || this.out.writeNonStringValueAsString;
   }

   public boolean hasNameFilters(SerializeFilterable filterable) {
      return this.nameFilters != null && this.nameFilters.size() > 0 || filterable.nameFilters != null && filterable.nameFilters.size() > 0;
   }

   public boolean hasPropertyFilters(SerializeFilterable filterable) {
      return this.propertyFilters != null && this.propertyFilters.size() > 0 || filterable.propertyFilters != null && filterable.propertyFilters.size() > 0;
   }

   public int getIndentCount() {
      return this.indentCount;
   }

   public void incrementIndent() {
      ++this.indentCount;
   }

   public void decrementIdent() {
      --this.indentCount;
   }

   public void println() {
      this.out.write(10);

      for(int i = 0; i < this.indentCount; ++i) {
         this.out.write(this.indent);
      }

   }

   public SerializeWriter getWriter() {
      return this.out;
   }

   public String toString() {
      return this.out.toString();
   }

   public void config(SerializerFeature feature, boolean state) {
      this.out.config(feature, state);
   }

   public boolean isEnabled(SerializerFeature feature) {
      return this.out.isEnabled(feature);
   }

   public void writeNull() {
      this.out.writeNull();
   }

   public SerializeConfig getMapping() {
      return this.config;
   }

   public static void write(Writer out, Object object) {
      SerializeWriter writer = new SerializeWriter();

      try {
         JSONSerializer serializer = new JSONSerializer(writer);
         serializer.write(object);
         writer.writeTo(out);
      } catch (IOException var7) {
         throw new JSONException(var7.getMessage(), var7);
      } finally {
         writer.close();
      }

   }

   public static void write(SerializeWriter out, Object object) {
      JSONSerializer serializer = new JSONSerializer(out);
      serializer.write(object);
   }

   public final void write(Object object) {
      if (object == null) {
         this.out.writeNull();
      } else {
         Class<?> clazz = object.getClass();
         ObjectSerializer writer = this.getObjectWriter(clazz);

         try {
            writer.write(this, object, (Object)null, (Type)null, 0);
         } catch (IOException var5) {
            throw new JSONException(var5.getMessage(), var5);
         }
      }
   }

   public final void writeWithFieldName(Object object, Object fieldName) {
      this.writeWithFieldName(object, fieldName, (Type)null, 0);
   }

   protected final void writeKeyValue(char seperator, String key, Object value) {
      if (seperator != 0) {
         this.out.write(seperator);
      }

      this.out.writeFieldName(key);
      this.write(value);
   }

   public final void writeWithFieldName(Object object, Object fieldName, Type fieldType, int fieldFeatures) {
      try {
         if (object == null) {
            this.out.writeNull();
         } else {
            Class<?> clazz = object.getClass();
            ObjectSerializer writer = this.getObjectWriter(clazz);
            writer.write(this, object, fieldName, fieldType, fieldFeatures);
         }
      } catch (IOException var7) {
         throw new JSONException(var7.getMessage(), var7);
      }
   }

   public final void writeWithFormat(Object object, String format) {
      if (object instanceof Date) {
         DateFormat dateFormat = this.getDateFormat();
         if (dateFormat == null) {
            dateFormat = new SimpleDateFormat(format, this.locale);
            ((DateFormat)dateFormat).setTimeZone(this.timeZone);
         }

         String text = ((DateFormat)dateFormat).format((Date)object);
         this.out.writeString(text);
      } else if (!(object instanceof byte[])) {
         if (object instanceof Collection) {
            Collection collection = (Collection)object;
            Iterator iterator = collection.iterator();
            this.out.write(91);

            for(int i = 0; i < collection.size(); ++i) {
               Object item = iterator.next();
               if (i != 0) {
                  this.out.write(44);
               }

               this.writeWithFormat(item, format);
            }

            this.out.write(93);
         } else {
            this.write(object);
         }
      } else {
         byte[] bytes = (byte[])((byte[])object);
         if (!"gzip".equals(format) && !"gzip,base64".equals(format)) {
            if ("hex".equals(format)) {
               this.out.writeHex(bytes);
            } else {
               this.out.writeByteArray(bytes);
            }
         } else {
            GZIPOutputStream gzipOut = null;

            try {
               ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
               if (bytes.length < 512) {
                  gzipOut = new GZIPOutputStream(byteOut, bytes.length);
               } else {
                  gzipOut = new GZIPOutputStream(byteOut);
               }

               gzipOut.write(bytes);
               gzipOut.finish();
               this.out.writeByteArray(byteOut.toByteArray());
            } catch (IOException var9) {
               throw new JSONException("write gzipBytes error", var9);
            } finally {
               IOUtils.close(gzipOut);
            }
         }

      }
   }

   public final void write(String text) {
      StringCodec.instance.write(this, text);
   }

   public ObjectSerializer getObjectWriter(Class<?> clazz) {
      return this.config.getObjectWriter(clazz);
   }

   public void close() {
      this.out.close();
   }
}
