package com.alibaba.fastjson.serializer;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONLexer;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import com.alibaba.fastjson.util.TypeUtils;
import java.io.IOException;
import java.lang.reflect.Array;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;

public class ObjectArrayCodec implements ObjectSerializer, ObjectDeserializer {
   public static final ObjectArrayCodec instance = new ObjectArrayCodec();

   public final void write(JSONSerializer serializer, Object object, Object fieldName, Type fieldType, int features) throws IOException {
      SerializeWriter out = serializer.out;
      Object[] array = (Object[])((Object[])object);
      if (object == null) {
         out.writeNull(SerializerFeature.WriteNullListAsEmpty);
      } else {
         int size = array.length;
         int end = size - 1;
         if (end == -1) {
            out.append("[]");
         } else {
            SerialContext context = serializer.context;
            serializer.setContext(context, object, fieldName, 0);

            try {
               Class<?> preClazz = null;
               ObjectSerializer preWriter = null;
               out.append('[');
               int i;
               if (out.isEnabled(SerializerFeature.PrettyFormat)) {
                  serializer.incrementIndent();
                  serializer.println();

                  for(i = 0; i < size; ++i) {
                     if (i != 0) {
                        out.write(44);
                        serializer.println();
                     }

                     serializer.write(array[i]);
                  }

                  serializer.decrementIdent();
                  serializer.println();
                  out.write(93);
                  return;
               }

               for(i = 0; i < end; ++i) {
                  Object item = array[i];
                  if (item == null) {
                     out.append("null,");
                  } else {
                     if (serializer.containsReference(item)) {
                        serializer.writeReference(item);
                     } else {
                        Class<?> clazz = item.getClass();
                        if (clazz == preClazz) {
                           preWriter.write(serializer, item, (Object)null, (Type)null, 0);
                        } else {
                           preClazz = clazz;
                           preWriter = serializer.getObjectWriter(clazz);
                           preWriter.write(serializer, item, (Object)null, (Type)null, 0);
                        }
                     }

                     out.append(',');
                  }
               }

               Object item = array[end];
               if (item == null) {
                  out.append("null]");
               } else {
                  if (serializer.containsReference(item)) {
                     serializer.writeReference(item);
                  } else {
                     serializer.writeWithFieldName(item, end);
                  }

                  out.append(']');
               }
            } finally {
               serializer.context = context;
            }

         }
      }
   }

   public <T> T deserialze(DefaultJSONParser parser, Type type, Object fieldName) {
      JSONLexer lexer = parser.lexer;
      int token = lexer.token();
      if (token == 8) {
         lexer.nextToken(16);
         return null;
      } else if (token != 4 && token != 26) {
         Object componentType;
         Class componentClass;
         if (type instanceof GenericArrayType) {
            GenericArrayType clazz = (GenericArrayType)type;
            componentType = clazz.getGenericComponentType();
            if (componentType instanceof TypeVariable) {
               TypeVariable typeVar = (TypeVariable)componentType;
               Type objType = parser.getContext().type;
               if (objType instanceof ParameterizedType) {
                  ParameterizedType objParamType = (ParameterizedType)objType;
                  Type objRawType = objParamType.getRawType();
                  Type actualType = null;
                  if (objRawType instanceof Class) {
                     TypeVariable[] objTypeParams = ((Class)objRawType).getTypeParameters();

                     for(int i = 0; i < objTypeParams.length; ++i) {
                        if (objTypeParams[i].getName().equals(typeVar.getName())) {
                           actualType = objParamType.getActualTypeArguments()[i];
                        }
                     }
                  }

                  if (actualType instanceof Class) {
                     componentClass = (Class)actualType;
                  } else {
                     componentClass = Object.class;
                  }
               } else {
                  componentClass = TypeUtils.getClass(typeVar.getBounds()[0]);
               }
            } else {
               componentClass = TypeUtils.getClass((Type)componentType);
            }
         } else {
            Class clazz = (Class)type;
            componentType = componentClass = clazz.getComponentType();
         }

         JSONArray array = new JSONArray();
         parser.parseArray((Type)componentType, array, fieldName);
         return this.toObjectArray(parser, componentClass, array);
      } else {
         byte[] bytes = lexer.bytesValue();
         lexer.nextToken(16);
         return bytes.length == 0 && type != byte[].class ? null : bytes;
      }
   }

   private <T> T toObjectArray(DefaultJSONParser parser, Class<?> componentType, JSONArray array) {
      if (array == null) {
         return null;
      } else {
         int size = array.size();
         Object objArray = Array.newInstance(componentType, size);

         for(int i = 0; i < size; ++i) {
            Object value = array.get(i);
            if (value == array) {
               Array.set(objArray, i, objArray);
            } else {
               Object element;
               if (componentType.isArray()) {
                  if (componentType.isInstance(value)) {
                     element = value;
                  } else {
                     element = this.toObjectArray(parser, componentType, (JSONArray)value);
                  }

                  Array.set(objArray, i, element);
               } else {
                  element = null;
                  if (value instanceof JSONArray) {
                     boolean contains = false;
                     JSONArray valueArray = (JSONArray)value;
                     int valueArraySize = valueArray.size();

                     for(int y = 0; y < valueArraySize; ++y) {
                        Object valueItem = valueArray.get(y);
                        if (valueItem == array) {
                           valueArray.set(i, objArray);
                           contains = true;
                        }
                     }

                     if (contains) {
                        element = valueArray.toArray();
                     }
                  }

                  if (element == null) {
                     element = TypeUtils.cast(value, componentType, parser.getConfig());
                  }

                  Array.set(objArray, i, element);
               }
            }
         }

         array.setRelatedArray(objArray);
         array.setComponentType(componentType);
         return objArray;
      }
   }

   public int getFastMatchToken() {
      return 14;
   }
}
