package com.alibaba.fastjson.util;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.alibaba.fastjson.annotation.JSONCreator;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.annotation.JSONPOJOBuilder;
import com.alibaba.fastjson.annotation.JSONType;
import com.alibaba.fastjson.parser.Feature;
import com.alibaba.fastjson.serializer.SerializerFeature;
import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

public class JavaBeanInfo {
   public final Class<?> clazz;
   public final Class<?> builderClass;
   public final Constructor<?> defaultConstructor;
   public final Constructor<?> creatorConstructor;
   public final Method factoryMethod;
   public final Method buildMethod;
   public final int defaultConstructorParameterSize;
   public final FieldInfo[] fields;
   public final FieldInfo[] sortedFields;
   public final int parserFeatures;
   public final JSONType jsonType;
   public final String typeName;
   public final String typeKey;
   public String[] orders;
   public Type[] creatorConstructorParameterTypes;
   public String[] creatorConstructorParameters;

   public JavaBeanInfo(Class<?> clazz, Class<?> builderClass, Constructor<?> defaultConstructor, Constructor<?> creatorConstructor, Method factoryMethod, Method buildMethod, JSONType jsonType, List<FieldInfo> fieldList) {
      this.clazz = clazz;
      this.builderClass = builderClass;
      this.defaultConstructor = defaultConstructor;
      this.creatorConstructor = creatorConstructor;
      this.factoryMethod = factoryMethod;
      this.parserFeatures = TypeUtils.getParserFeatures(clazz);
      this.buildMethod = buildMethod;
      this.jsonType = jsonType;
      if (jsonType != null) {
         String typeName = jsonType.typeName();
         String typeKey = jsonType.typeKey();
         this.typeKey = typeKey.length() > 0 ? typeKey : null;
         if (typeName.length() != 0) {
            this.typeName = typeName;
         } else {
            this.typeName = clazz.getName();
         }

         String[] orders = jsonType.orders();
         this.orders = orders.length == 0 ? null : orders;
      } else {
         this.typeName = clazz.getName();
         this.typeKey = null;
         this.orders = null;
      }

      this.fields = new FieldInfo[fieldList.size()];
      fieldList.toArray(this.fields);
      FieldInfo[] sortedFields = new FieldInfo[this.fields.length];
      int i;
      String fieldAnnotationName;
      if (this.orders != null) {
         LinkedHashMap<String, FieldInfo> map = new LinkedHashMap(fieldList.size());
         FieldInfo[] var22 = this.fields;
         i = var22.length;

         int var13;
         for(var13 = 0; var13 < i; ++var13) {
            FieldInfo field = var22[var13];
            map.put(field.name, field);
         }

         int i = 0;
         String[] var24 = this.orders;
         var13 = var24.length;

         for(int var29 = 0; var29 < var13; ++var29) {
            fieldAnnotationName = var24[var29];
            FieldInfo field = (FieldInfo)map.get(fieldAnnotationName);
            if (field != null) {
               sortedFields[i++] = field;
               map.remove(fieldAnnotationName);
            }
         }

         FieldInfo field;
         for(Iterator var25 = map.values().iterator(); var25.hasNext(); sortedFields[i++] = field) {
            field = (FieldInfo)var25.next();
         }
      } else {
         System.arraycopy(this.fields, 0, sortedFields, 0, this.fields.length);
         Arrays.sort(sortedFields);
      }

      if (Arrays.equals(this.fields, sortedFields)) {
         sortedFields = this.fields;
      }

      this.sortedFields = sortedFields;
      if (defaultConstructor != null) {
         this.defaultConstructorParameterSize = defaultConstructor.getParameterTypes().length;
      } else if (factoryMethod != null) {
         this.defaultConstructorParameterSize = factoryMethod.getParameterTypes().length;
      } else {
         this.defaultConstructorParameterSize = 0;
      }

      if (creatorConstructor != null) {
         this.creatorConstructorParameterTypes = creatorConstructor.getParameterTypes();
         boolean kotlin = TypeUtils.isKotlin(clazz);
         if (kotlin) {
            this.creatorConstructorParameters = TypeUtils.getKoltinConstructorParameters(clazz);
            Annotation[][] paramAnnotationArrays = creatorConstructor.getParameterAnnotations();

            for(i = 0; i < this.creatorConstructorParameters.length && i < paramAnnotationArrays.length; ++i) {
               Annotation[] paramAnnotations = paramAnnotationArrays[i];
               JSONField fieldAnnotation = null;
               Annotation[] var32 = paramAnnotations;
               int var33 = paramAnnotations.length;

               for(int var17 = 0; var17 < var33; ++var17) {
                  Annotation paramAnnotation = var32[var17];
                  if (paramAnnotation instanceof JSONField) {
                     fieldAnnotation = (JSONField)paramAnnotation;
                     break;
                  }
               }

               if (fieldAnnotation != null) {
                  fieldAnnotationName = fieldAnnotation.name();
                  if (fieldAnnotationName.length() > 0) {
                     this.creatorConstructorParameters[i] = fieldAnnotationName;
                  }
               }
            }
         } else {
            boolean match;
            if (this.creatorConstructorParameterTypes.length != this.fields.length) {
               match = false;
            } else {
               match = true;

               for(i = 0; i < this.creatorConstructorParameterTypes.length; ++i) {
                  if (this.creatorConstructorParameterTypes[i] != this.fields[i].fieldClass) {
                     match = false;
                     break;
                  }
               }
            }

            if (!match) {
               this.creatorConstructorParameters = ASMUtils.lookupParameterNames(creatorConstructor);
            }
         }
      }

   }

   private static FieldInfo getField(List<FieldInfo> fieldList, String propertyName) {
      Iterator var2 = fieldList.iterator();

      FieldInfo item;
      Field field;
      do {
         if (!var2.hasNext()) {
            return null;
         }

         item = (FieldInfo)var2.next();
         if (item.name.equals(propertyName)) {
            return item;
         }

         field = item.field;
      } while(field == null || item.getAnnotation() == null || !field.getName().equals(propertyName));

      return item;
   }

   static boolean add(List<FieldInfo> fieldList, FieldInfo field) {
      for(int i = fieldList.size() - 1; i >= 0; --i) {
         FieldInfo item = (FieldInfo)fieldList.get(i);
         if (item.name.equals(field.name) && (!item.getOnly || field.getOnly)) {
            if (item.fieldClass.isAssignableFrom(field.fieldClass)) {
               fieldList.remove(i);
            } else {
               int result = item.compareTo(field);
               if (result >= 0) {
                  return false;
               }

               fieldList.remove(i);
            }
            break;
         }
      }

      fieldList.add(field);
      return true;
   }

   public static JavaBeanInfo build(Class<?> clazz, Type type, PropertyNamingStrategy propertyNamingStrategy) {
      return build(clazz, type, propertyNamingStrategy, false, TypeUtils.compatibleWithJavaBean);
   }

   public static JavaBeanInfo build(Class<?> clazz, Type type, PropertyNamingStrategy propertyNamingStrategy, boolean fieldBased, boolean compatibleWithJavaBean) {
      JSONType jsonType = (JSONType)TypeUtils.getAnnotation(clazz, JSONType.class);
      if (jsonType != null) {
         PropertyNamingStrategy jsonTypeNaming = jsonType.naming();
         if (jsonTypeNaming != null && jsonTypeNaming != PropertyNamingStrategy.CamelCase) {
            propertyNamingStrategy = jsonTypeNaming;
         }
      }

      Class<?> builderClass = getBuilderClass(clazz, jsonType);
      Field[] declaredFields = clazz.getDeclaredFields();
      Method[] methods = clazz.getMethods();
      boolean kotlin = TypeUtils.isKotlin(clazz);
      Constructor[] constructors = clazz.getDeclaredConstructors();
      Constructor<?> defaultConstructor = null;
      if (!kotlin || constructors.length == 1) {
         if (builderClass == null) {
            defaultConstructor = getDefaultConstructor(clazz, constructors);
         } else {
            defaultConstructor = getDefaultConstructor(builderClass, builderClass.getDeclaredConstructors());
         }
      }

      Constructor<?> creatorConstructor = null;
      Method buildMethod = null;
      Method factoryMethod = null;
      List<FieldInfo> fieldList = new ArrayList();
      Field[] fields;
      if (fieldBased) {
         for(Class currentClass = clazz; currentClass != null; currentClass = currentClass.getSuperclass()) {
            fields = currentClass.getDeclaredFields();
            computeFields(clazz, type, propertyNamingStrategy, fieldList, fields);
         }

         return new JavaBeanInfo(clazz, builderClass, defaultConstructor, (Constructor)null, factoryMethod, buildMethod, jsonType, fieldList);
      } else {
         boolean isInterfaceOrAbstract = clazz.isInterface() || Modifier.isAbstract(clazz.getModifiers());
         int i;
         int parserFeatures;
         int serialzeFeatures;
         String isFieldName;
         String className;
         int i;
         int ordinal;
         String paramName;
         int parserFeatures;
         JSONField fieldAnnotation;
         Class fieldClass;
         if (defaultConstructor == null && builderClass == null || isInterfaceOrAbstract) {
            creatorConstructor = getCreatorConstructor(constructors);
            Class[] types;
            int parserFeatures;
            int serialzeFeatures;
            String[] paramNames;
            int ordinal;
            if (creatorConstructor != null && !isInterfaceOrAbstract) {
               TypeUtils.setAccessible(creatorConstructor);
               types = creatorConstructor.getParameterTypes();
               paramNames = null;
               if (types.length > 0) {
                  Annotation[][] paramAnnotationArrays = creatorConstructor.getParameterAnnotations();

                  for(i = 0; i < types.length; ++i) {
                     Annotation[] paramAnnotations = paramAnnotationArrays[i];
                     JSONField fieldAnnotation = null;
                     Annotation[] var65 = paramAnnotations;
                     serialzeFeatures = paramAnnotations.length;

                     for(parserFeatures = 0; parserFeatures < serialzeFeatures; ++parserFeatures) {
                        Annotation paramAnnotation = var65[parserFeatures];
                        if (paramAnnotation instanceof JSONField) {
                           fieldAnnotation = (JSONField)paramAnnotation;
                           break;
                        }
                     }

                     Class<?> fieldClass = types[i];
                     Type fieldType = creatorConstructor.getGenericParameterTypes()[i];
                     String fieldName = null;
                     Field field = null;
                     parserFeatures = 0;
                     ordinal = 0;
                     serialzeFeatures = 0;
                     if (fieldAnnotation != null) {
                        field = TypeUtils.getField(clazz, fieldAnnotation.name(), declaredFields);
                        parserFeatures = fieldAnnotation.ordinal();
                        ordinal = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                        serialzeFeatures = Feature.of(fieldAnnotation.parseFeatures());
                        fieldName = fieldAnnotation.name();
                     }

                     if (fieldName == null || fieldName.length() == 0) {
                        if (paramNames == null) {
                           paramNames = ASMUtils.lookupParameterNames(creatorConstructor);
                        }

                        fieldName = paramNames[i];
                     }

                     FieldInfo fieldInfo = new FieldInfo(fieldName, clazz, fieldClass, fieldType, field, parserFeatures, ordinal, serialzeFeatures);
                     add(fieldList, fieldInfo);
                  }
               }
            } else {
               Annotation[] paramAnnotations;
               int serialzeFeatures;
               if ((factoryMethod = getFactoryMethod(clazz, methods)) != null) {
                  TypeUtils.setAccessible(factoryMethod);
                  types = factoryMethod.getParameterTypes();
                  if (types.length > 0) {
                     Annotation[][] paramAnnotationArrays = factoryMethod.getParameterAnnotations();

                     for(i = 0; i < types.length; ++i) {
                        Annotation[] paramAnnotations = paramAnnotationArrays[i];
                        JSONField fieldAnnotation = null;
                        paramAnnotations = paramAnnotations;
                        parserFeatures = paramAnnotations.length;

                        for(serialzeFeatures = 0; serialzeFeatures < parserFeatures; ++serialzeFeatures) {
                           Annotation paramAnnotation = paramAnnotations[serialzeFeatures];
                           if (paramAnnotation instanceof JSONField) {
                              fieldAnnotation = (JSONField)paramAnnotation;
                              break;
                           }
                        }

                        if (fieldAnnotation == null) {
                           throw new JSONException("illegal json creator");
                        }

                        Class<?> fieldClass = types[i];
                        Type fieldType = factoryMethod.getGenericParameterTypes()[i];
                        Field field = TypeUtils.getField(clazz, fieldAnnotation.name(), declaredFields);
                        parserFeatures = fieldAnnotation.ordinal();
                        serialzeFeatures = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                        parserFeatures = Feature.of(fieldAnnotation.parseFeatures());
                        FieldInfo fieldInfo = new FieldInfo(fieldAnnotation.name(), clazz, fieldClass, fieldType, field, parserFeatures, serialzeFeatures, parserFeatures);
                        add(fieldList, fieldInfo);
                     }

                     return new JavaBeanInfo(clazz, builderClass, (Constructor)null, (Constructor)null, factoryMethod, (Method)null, jsonType, fieldList);
                  }
               } else if (!isInterfaceOrAbstract) {
                  className = clazz.getName();
                  paramNames = null;
                  if (kotlin && constructors.length > 0) {
                     paramNames = TypeUtils.getKoltinConstructorParameters(clazz);
                     creatorConstructor = TypeUtils.getKoltinConstructor(constructors);
                     TypeUtils.setAccessible(creatorConstructor);
                  } else {
                     Constructor[] var41 = constructors;
                     i = constructors.length;

                     for(ordinal = 0; ordinal < i; ++ordinal) {
                        Constructor constructor = var41[ordinal];
                        Class<?>[] parameterTypes = constructor.getParameterTypes();
                        if (className.equals("org.springframework.security.web.authentication.WebAuthenticationDetails") && parameterTypes.length == 2 && parameterTypes[0] == String.class && parameterTypes[1] == String.class) {
                           creatorConstructor = constructor;
                           constructor.setAccessible(true);
                           paramNames = ASMUtils.lookupParameterNames(constructor);
                           break;
                        }

                        if (className.equals("org.springframework.security.web.authentication.preauth.PreAuthenticatedAuthenticationToken") && parameterTypes.length == 3 && parameterTypes[0] == Object.class && parameterTypes[1] == Object.class && parameterTypes[2] == Collection.class) {
                           creatorConstructor = constructor;
                           constructor.setAccessible(true);
                           paramNames = new String[]{"principal", "credentials", "authorities"};
                           break;
                        }

                        if (className.equals("org.springframework.security.core.authority.SimpleGrantedAuthority") && parameterTypes.length == 1 && parameterTypes[0] == String.class) {
                           creatorConstructor = constructor;
                           paramNames = new String[]{"authority"};
                           break;
                        }

                        boolean is_public = (constructor.getModifiers() & 1) != 0;
                        if (is_public) {
                           String[] lookupParameterNames = ASMUtils.lookupParameterNames(constructor);
                           if (lookupParameterNames != null && lookupParameterNames.length != 0 && (creatorConstructor == null || paramNames == null || lookupParameterNames.length > paramNames.length)) {
                              paramNames = lookupParameterNames;
                              creatorConstructor = constructor;
                           }
                        }
                     }
                  }

                  Class<?>[] types = null;
                  if (paramNames != null) {
                     types = creatorConstructor.getParameterTypes();
                  }

                  if (paramNames == null || types.length != paramNames.length) {
                     throw new JSONException("default constructor not found. " + clazz);
                  }

                  Annotation[][] paramAnnotationArrays = creatorConstructor.getParameterAnnotations();

                  for(ordinal = 0; ordinal < types.length; ++ordinal) {
                     paramAnnotations = paramAnnotationArrays[ordinal];
                     paramName = paramNames[ordinal];
                     fieldAnnotation = null;
                     Annotation[] var73 = paramAnnotations;
                     serialzeFeatures = paramAnnotations.length;

                     for(parserFeatures = 0; parserFeatures < serialzeFeatures; ++parserFeatures) {
                        Annotation paramAnnotation = var73[parserFeatures];
                        if (paramAnnotation instanceof JSONField) {
                           fieldAnnotation = (JSONField)paramAnnotation;
                           break;
                        }
                     }

                     fieldClass = types[ordinal];
                     Type fieldType = creatorConstructor.getGenericParameterTypes()[ordinal];
                     Field field = TypeUtils.getField(clazz, paramName, declaredFields);
                     if (field != null && fieldAnnotation == null) {
                        fieldAnnotation = (JSONField)field.getAnnotation(JSONField.class);
                     }

                     int parserFeatures;
                     if (fieldAnnotation == null) {
                        ordinal = 0;
                        serialzeFeatures = 0;
                        if ("org.springframework.security.core.userdetails.User".equals(className) && "password".equals(paramName)) {
                           parserFeatures = Feature.InitStringFieldAsEmpty.mask;
                        } else {
                           parserFeatures = 0;
                        }
                     } else {
                        isFieldName = fieldAnnotation.name();
                        if (isFieldName.length() != 0) {
                           paramName = isFieldName;
                        }

                        ordinal = fieldAnnotation.ordinal();
                        serialzeFeatures = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                        parserFeatures = Feature.of(fieldAnnotation.parseFeatures());
                     }

                     FieldInfo fieldInfo = new FieldInfo(paramName, clazz, fieldClass, fieldType, field, ordinal, serialzeFeatures, parserFeatures);
                     add(fieldList, fieldInfo);
                  }

                  if (!kotlin && !clazz.getName().equals("javax.servlet.http.Cookie")) {
                     return new JavaBeanInfo(clazz, builderClass, (Constructor)null, creatorConstructor, (Method)null, (Method)null, jsonType, fieldList);
                  }
               }
            }
         }

         if (defaultConstructor != null) {
            TypeUtils.setAccessible(defaultConstructor);
         }

         JSONField annotation;
         if (builderClass != null) {
            className = null;
            JSONPOJOBuilder builderAnno = (JSONPOJOBuilder)builderClass.getAnnotation(JSONPOJOBuilder.class);
            if (builderAnno != null) {
               className = builderAnno.withPrefix();
            }

            if (className == null || className.length() == 0) {
               className = "with";
            }

            Method[] var47 = builderClass.getMethods();
            i = var47.length;

            for(ordinal = 0; ordinal < i; ++ordinal) {
               Method method = var47[ordinal];
               if (!Modifier.isStatic(method.getModifiers()) && method.getReturnType().equals(builderClass)) {
                  parserFeatures = 0;
                  serialzeFeatures = 0;
                  parserFeatures = 0;
                  annotation = (JSONField)method.getAnnotation(JSONField.class);
                  if (annotation == null) {
                     annotation = TypeUtils.getSuperMethodAnnotation(clazz, method);
                  }

                  String methodName;
                  if (annotation != null) {
                     if (!annotation.deserialize()) {
                        continue;
                     }

                     parserFeatures = annotation.ordinal();
                     serialzeFeatures = SerializerFeature.of(annotation.serialzeFeatures());
                     parserFeatures = Feature.of(annotation.parseFeatures());
                     if (annotation.name().length() != 0) {
                        methodName = annotation.name();
                        add(fieldList, new FieldInfo(methodName, method, (Field)null, clazz, type, parserFeatures, serialzeFeatures, parserFeatures, annotation, (JSONField)null, (String)null));
                        continue;
                     }
                  }

                  methodName = method.getName();
                  StringBuilder properNameBuilder;
                  if (methodName.startsWith("set") && methodName.length() > 3) {
                     properNameBuilder = new StringBuilder(methodName.substring(3));
                  } else {
                     if (!methodName.startsWith(className) || methodName.length() <= className.length()) {
                        continue;
                     }

                     properNameBuilder = new StringBuilder(methodName.substring(className.length()));
                  }

                  char c0 = properNameBuilder.charAt(0);
                  if (Character.isUpperCase(c0)) {
                     properNameBuilder.setCharAt(0, Character.toLowerCase(c0));
                     String propertyName = properNameBuilder.toString();
                     add(fieldList, new FieldInfo(propertyName, method, (Field)null, clazz, type, parserFeatures, serialzeFeatures, parserFeatures, annotation, (JSONField)null, (String)null));
                  }
               }
            }

            if (builderClass != null) {
               JSONPOJOBuilder builderAnnotation = (JSONPOJOBuilder)builderClass.getAnnotation(JSONPOJOBuilder.class);
               String buildMethodName = null;
               if (builderAnnotation != null) {
                  buildMethodName = builderAnnotation.buildMethod();
               }

               if (buildMethodName == null || buildMethodName.length() == 0) {
                  buildMethodName = "build";
               }

               try {
                  buildMethod = builderClass.getMethod(buildMethodName);
               } catch (NoSuchMethodException var34) {
               } catch (SecurityException var35) {
               }

               if (buildMethod == null) {
                  try {
                     buildMethod = builderClass.getMethod("create");
                  } catch (NoSuchMethodException var32) {
                  } catch (SecurityException var33) {
                  }
               }

               if (buildMethod == null) {
                  throw new JSONException("buildMethod not found.");
               }

               TypeUtils.setAccessible(buildMethod);
            }
         }

         Method[] var40 = methods;
         int var50 = methods.length;

         for(i = 0; i < var50; ++i) {
            Method method = var40[i];
            ordinal = 0;
            int serialzeFeatures = 0;
            parserFeatures = 0;
            String methodName = method.getName();
            if (!Modifier.isStatic(method.getModifiers())) {
               fieldClass = method.getReturnType();
               if ((fieldClass.equals(Void.TYPE) || fieldClass.equals(method.getDeclaringClass())) && method.getDeclaringClass() != Object.class) {
                  Class<?>[] types = method.getParameterTypes();
                  if (types.length != 0 && types.length <= 2) {
                     JSONField annotation = (JSONField)method.getAnnotation(JSONField.class);
                     if (annotation != null && types.length == 2 && types[0] == String.class && types[1] == Object.class) {
                        add(fieldList, new FieldInfo("", method, (Field)null, clazz, type, ordinal, serialzeFeatures, parserFeatures, annotation, (JSONField)null, (String)null));
                     } else if (types.length == 1) {
                        if (annotation == null) {
                           annotation = TypeUtils.getSuperMethodAnnotation(clazz, method);
                        }

                        if (annotation != null || methodName.length() >= 4) {
                           if (annotation != null) {
                              if (!annotation.deserialize()) {
                                 continue;
                              }

                              ordinal = annotation.ordinal();
                              serialzeFeatures = SerializerFeature.of(annotation.serialzeFeatures());
                              parserFeatures = Feature.of(annotation.parseFeatures());
                              if (annotation.name().length() != 0) {
                                 String propertyName = annotation.name();
                                 add(fieldList, new FieldInfo(propertyName, method, (Field)null, clazz, type, ordinal, serialzeFeatures, parserFeatures, annotation, (JSONField)null, (String)null));
                                 continue;
                              }
                           }

                           if (annotation != null || methodName.startsWith("set")) {
                              char c3 = methodName.charAt(3);
                              String propertyName;
                              if (!Character.isUpperCase(c3) && c3 <= 512) {
                                 if (c3 == '_') {
                                    propertyName = methodName.substring(4);
                                 } else if (c3 == 'f') {
                                    propertyName = methodName.substring(3);
                                 } else {
                                    if (methodName.length() < 5 || !Character.isUpperCase(methodName.charAt(4))) {
                                       continue;
                                    }

                                    propertyName = TypeUtils.decapitalize(methodName.substring(3));
                                 }
                              } else if (TypeUtils.compatibleWithJavaBean) {
                                 propertyName = TypeUtils.decapitalize(methodName.substring(3));
                              } else {
                                 propertyName = Character.toLowerCase(methodName.charAt(3)) + methodName.substring(4);
                              }

                              Field field = TypeUtils.getField(clazz, propertyName, declaredFields);
                              if (field == null && types[0] == Boolean.TYPE) {
                                 isFieldName = "is" + Character.toUpperCase(propertyName.charAt(0)) + propertyName.substring(1);
                                 field = TypeUtils.getField(clazz, isFieldName, declaredFields);
                              }

                              JSONField fieldAnnotation = null;
                              if (field != null) {
                                 fieldAnnotation = (JSONField)field.getAnnotation(JSONField.class);
                                 if (fieldAnnotation != null) {
                                    if (!fieldAnnotation.deserialize()) {
                                       continue;
                                    }

                                    ordinal = fieldAnnotation.ordinal();
                                    serialzeFeatures = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                                    parserFeatures = Feature.of(fieldAnnotation.parseFeatures());
                                    if (fieldAnnotation.name().length() != 0) {
                                       propertyName = fieldAnnotation.name();
                                       add(fieldList, new FieldInfo(propertyName, method, field, clazz, type, ordinal, serialzeFeatures, parserFeatures, annotation, fieldAnnotation, (String)null));
                                       continue;
                                    }
                                 }
                              }

                              if (propertyNamingStrategy != null) {
                                 propertyName = propertyNamingStrategy.translate(propertyName);
                              }

                              add(fieldList, new FieldInfo(propertyName, method, field, clazz, type, ordinal, serialzeFeatures, parserFeatures, annotation, fieldAnnotation, (String)null));
                           }
                        }
                     }
                  }
               }
            }
         }

         fields = clazz.getFields();
         computeFields(clazz, type, propertyNamingStrategy, fieldList, fields);
         Method[] var53 = clazz.getMethods();
         i = var53.length;

         for(i = 0; i < i; ++i) {
            Method method = var53[i];
            String methodName = method.getName();
            if (methodName.length() >= 4 && !Modifier.isStatic(method.getModifiers()) && builderClass == null && methodName.startsWith("get") && Character.isUpperCase(methodName.charAt(3)) && method.getParameterTypes().length == 0 && (Collection.class.isAssignableFrom(method.getReturnType()) || Map.class.isAssignableFrom(method.getReturnType()) || AtomicBoolean.class == method.getReturnType() || AtomicInteger.class == method.getReturnType() || AtomicLong.class == method.getReturnType())) {
               fieldAnnotation = (JSONField)method.getAnnotation(JSONField.class);
               if (fieldAnnotation == null || !fieldAnnotation.deserialize()) {
                  if (fieldAnnotation != null && fieldAnnotation.name().length() > 0) {
                     paramName = fieldAnnotation.name();
                  } else {
                     paramName = Character.toLowerCase(methodName.charAt(3)) + methodName.substring(4);
                     Field field = TypeUtils.getField(clazz, paramName, declaredFields);
                     if (field != null) {
                        annotation = (JSONField)field.getAnnotation(JSONField.class);
                        if (annotation != null && !annotation.deserialize()) {
                           continue;
                        }
                     }
                  }

                  if (propertyNamingStrategy != null) {
                     paramName = propertyNamingStrategy.translate(paramName);
                  }

                  FieldInfo fieldInfo = getField(fieldList, paramName);
                  if (fieldInfo == null) {
                     add(fieldList, new FieldInfo(paramName, method, (Field)null, clazz, type, 0, 0, 0, fieldAnnotation, (JSONField)null, (String)null));
                  }
               }
            }
         }

         return new JavaBeanInfo(clazz, builderClass, defaultConstructor, creatorConstructor, factoryMethod, buildMethod, jsonType, fieldList);
      }
   }

   private static void computeFields(Class<?> clazz, Type type, PropertyNamingStrategy propertyNamingStrategy, List<FieldInfo> fieldList, Field[] fields) {
      Field[] var5 = fields;
      int var6 = fields.length;

      for(int var7 = 0; var7 < var6; ++var7) {
         Field field = var5[var7];
         int modifiers = field.getModifiers();
         if ((modifiers & 8) == 0) {
            if ((modifiers & 16) != 0) {
               Class<?> fieldType = field.getType();
               boolean supportReadOnly = Map.class.isAssignableFrom(fieldType) || Collection.class.isAssignableFrom(fieldType) || AtomicLong.class.equals(fieldType) || AtomicInteger.class.equals(fieldType) || AtomicBoolean.class.equals(fieldType);
               if (!supportReadOnly) {
                  continue;
               }
            }

            boolean contains = false;
            Iterator var17 = fieldList.iterator();

            while(var17.hasNext()) {
               FieldInfo item = (FieldInfo)var17.next();
               if (item.name.equals(field.getName())) {
                  contains = true;
                  break;
               }
            }

            if (!contains) {
               int ordinal = 0;
               int serialzeFeatures = 0;
               int parserFeatures = 0;
               String propertyName = field.getName();
               JSONField fieldAnnotation = (JSONField)field.getAnnotation(JSONField.class);
               if (fieldAnnotation != null) {
                  if (!fieldAnnotation.deserialize()) {
                     continue;
                  }

                  ordinal = fieldAnnotation.ordinal();
                  serialzeFeatures = SerializerFeature.of(fieldAnnotation.serialzeFeatures());
                  parserFeatures = Feature.of(fieldAnnotation.parseFeatures());
                  if (fieldAnnotation.name().length() != 0) {
                     propertyName = fieldAnnotation.name();
                  }
               }

               if (propertyNamingStrategy != null) {
                  propertyName = propertyNamingStrategy.translate(propertyName);
               }

               add(fieldList, new FieldInfo(propertyName, (Method)null, field, clazz, type, ordinal, serialzeFeatures, parserFeatures, (JSONField)null, fieldAnnotation, (String)null));
            }
         }
      }

   }

   static Constructor<?> getDefaultConstructor(Class<?> clazz, Constructor<?>[] constructors) {
      if (Modifier.isAbstract(clazz.getModifiers())) {
         return null;
      } else {
         Constructor<?> defaultConstructor = null;
         Constructor[] var3 = constructors;
         int var4 = constructors.length;

         int var5;
         for(var5 = 0; var5 < var4; ++var5) {
            Constructor<?> constructor = var3[var5];
            if (constructor.getParameterTypes().length == 0) {
               defaultConstructor = constructor;
               break;
            }
         }

         if (defaultConstructor == null && clazz.isMemberClass() && !Modifier.isStatic(clazz.getModifiers())) {
            Constructor[] var9 = constructors;
            var5 = constructors.length;

            for(int var10 = 0; var10 < var5; ++var10) {
               Constructor<?> constructor = var9[var10];
               Class[] types;
               if ((types = constructor.getParameterTypes()).length == 1 && types[0].equals(clazz.getDeclaringClass())) {
                  defaultConstructor = constructor;
                  break;
               }
            }
         }

         return defaultConstructor;
      }
   }

   public static Constructor<?> getCreatorConstructor(Constructor[] constructors) {
      Constructor<?> creatorConstructor = null;
      Constructor[] var2 = constructors;
      int var3 = constructors.length;

      int var4;
      Constructor constructor;
      for(var4 = 0; var4 < var3; ++var4) {
         constructor = var2[var4];
         JSONCreator annotation = (JSONCreator)constructor.getAnnotation(JSONCreator.class);
         if (annotation != null) {
            if (creatorConstructor != null) {
               throw new JSONException("multi-JSONCreator");
            }

            creatorConstructor = constructor;
         }
      }

      if (creatorConstructor != null) {
         return creatorConstructor;
      } else {
         var2 = constructors;
         var3 = constructors.length;

         for(var4 = 0; var4 < var3; ++var4) {
            constructor = var2[var4];
            Annotation[][] paramAnnotationArrays = constructor.getParameterAnnotations();
            if (paramAnnotationArrays.length != 0) {
               boolean match = true;
               Annotation[][] var8 = paramAnnotationArrays;
               int var9 = paramAnnotationArrays.length;

               for(int var10 = 0; var10 < var9; ++var10) {
                  Annotation[] paramAnnotationArray = var8[var10];
                  boolean paramMatch = false;
                  Annotation[] var13 = paramAnnotationArray;
                  int var14 = paramAnnotationArray.length;

                  for(int var15 = 0; var15 < var14; ++var15) {
                     Annotation paramAnnotation = var13[var15];
                     if (paramAnnotation instanceof JSONField) {
                        paramMatch = true;
                        break;
                     }
                  }

                  if (!paramMatch) {
                     match = false;
                     break;
                  }
               }

               if (match) {
                  if (creatorConstructor != null) {
                     throw new JSONException("multi-JSONCreator");
                  }

                  creatorConstructor = constructor;
               }
            }
         }

         if (creatorConstructor != null) {
            return creatorConstructor;
         } else {
            return creatorConstructor;
         }
      }
   }

   private static Method getFactoryMethod(Class<?> clazz, Method[] methods) {
      Method factoryMethod = null;
      Method[] var3 = methods;
      int var4 = methods.length;

      for(int var5 = 0; var5 < var4; ++var5) {
         Method method = var3[var5];
         if (Modifier.isStatic(method.getModifiers()) && clazz.isAssignableFrom(method.getReturnType())) {
            JSONCreator annotation = (JSONCreator)method.getAnnotation(JSONCreator.class);
            if (annotation != null) {
               if (factoryMethod != null) {
                  throw new JSONException("multi-JSONCreator");
               }

               factoryMethod = method;
            }
         }
      }

      return factoryMethod;
   }

   public static Class<?> getBuilderClass(JSONType type) {
      return getBuilderClass((Class)null, type);
   }

   public static Class<?> getBuilderClass(Class<?> clazz, JSONType type) {
      if (clazz != null && clazz.getName().equals("org.springframework.security.web.savedrequest.DefaultSavedRequest")) {
         return TypeUtils.loadClass("org.springframework.security.web.savedrequest.DefaultSavedRequest$Builder");
      } else if (type == null) {
         return null;
      } else {
         Class<?> builderClass = type.builder();
         return builderClass == Void.class ? null : builderClass;
      }
   }
}
